{*****************************************************************************
' Build 0135
'
' Project:  GDAL Lazarus Bindings
' Purpose:  Lazarus GDALColorTable Shadow Class.
' Author:   Frank Warmerdam, warmerdam@pobox.com
' Lazarus Wrapper : Prajuab Riabroy
'*****************************************************************************
' Copyright (c) 2005, Frank Warmerdam <warmerdam@pobox.com>
'
' Permission is hereby granted, free of charge, to any person obtaining a
' copy of this software and associated documentation files (the "Software"),
' to deal in the Software without restriction, including without limitation
' the rights to use, copy, modify, merge, publish, distribute, sublicense,
' and/or sell copies of the Software, and to permit persons to whom the
' Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included
' in all copies or substantial portions of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
' OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
' FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
' DEALINGS IN THE SOFTWARE.
'*****************************************************************************
}
unit gdalcolortable;

{$IFDEF FPC}
{$mode objfpc}{$H+}
{$ENDIF}

interface

uses
  Classes, SysUtils, gdal, gdalcore;
type
  TIntArrays4 = Array[0..3] of integer;

  TGDALColorTable = class
  private
    FObj : TGDALColorTableH;
    FOwned : longint;
    FEntryCount : longint;
    FPaletteInterpretation : longint;
  public
    property  EntryCount : longint read FEntryCount write FEntryCount;
    property  PaletteInterpretation : longint read FPaletteInterpretation write FPaletteInterpretation;
    procedure CInit(objA : TGDALColorTableH; ownedA : longint );
    procedure DestroyColorTable;
    function  GetObjPtr : TGDALColorTableH;
    function  IsValid : boolean;
    function  Clone : TGDALColorTable;
    procedure SetColorEntry(const ColorIndex : longint; ColorEntrys : TIntArrays4);
    function  GetColorEntry(const ColorIndex : longint; ColorEntrys : TIntArrays4) : longint;
    function  GetColorEntryAsRGB(const ColorIndex : longint; ColorEntrys : TIntArrays4) : longint;
    function  CreateColorTable (const PaletteInterp : TGDALPaletteInterp) : TGDALColorTableH;
//    function  CloneColorTable(hTable : TGDALColorTableH) : TGDALColorTableH;
    constructor Create;
    destructor Destroy; override;
  end;

implementation

constructor TGDALColorTable.Create;
var
  objH : TGDALColorTableH;
begin
  inherited create;
  FObj := NIL;
  FOwned := 1;
  FEntryCount := 0;
  FPaletteInterpretation := 0;
end;

destructor TGDALColorTable.Destroy;
begin
  if (FObj <> NIL) and (FOwned <> 0) then
    DestroyColorTable;
  inherited Destroy;
end;

function  TGDALColorTable.CreateColorTable (const PaletteInterp : TGDALPaletteInterp) : TGDALColorTableH;
begin
  result := GDALCore.GDALCreateColorTable(PaletteInterp);
end;

//' ----------------------------------------------------------------------------
procedure TGDALColorTable.CInit(ObjA : TGDALColorTableH; ownedA : longint );
begin
  if (FObj <> NIL) and (FOwned <> 0) Then
    DestroyColorTable;

  FObj := ObjA;
  FOwned := ownedA;
    if (FObj <> NIL) then
    begin
      FEntryCount := GDALCore.GDALGetColorEntryCount(FObj);
      FPaletteInterpretation := GDALCore.GDALGetPaletteInterpretation(FObj);
    end;
end;

//' ----------------------------------------------------------------------------
procedure TGDALColorTable.DestroyColorTable;
begin
  if (FObj <> NIL) then
  begin
    //Call MsgBox( "Destroying ColorTable" )
    GDALCore.GDALDestroyColorTable( FObj );
    FObj :=  NIL;
    FOwned := 0;
  end;
end;

//' ----------------------------------------------------------------------------
function TGDALColorTable.GetObjPtr : TGDALColorTableH;
begin
    result := FObj;
end;


//' ----------------------------------------------------------------------------
function TGDALColorTable.IsValid : boolean;
begin
  If (FObj = NIL) then
      IsValid := false
  else
      IsValid := true;
end;

//' ----------------------------------------------------------------------------
function TGDALColorTable.Clone : TGDALColorTable;
var
  other_c_FObj : TGDALColorTableH;
  other_ct : TGDALColorTable;
begin
  if (FObj <> NIL) then
  begin
    other_c_FObj := GDALCore.GDALCloneColorTable(FObj);
    other_ct := TGDALColorTable.Create;
    other_ct.CInit(other_c_FObj,1);
    result := other_ct
  end;
end;

//' ----------------------------------------------------------------------------
procedure TGDALColorTable.SetColorEntry(const ColorIndex : longint; ColorEntrys : TIntArrays4);
var
  SafeColorEntry : array[0..3] of integer;
begin
  if (FObj <> NIL) then
  begin
    SafeColorEntry[0] := ColorEntrys[0];
    SafeColorEntry[1] := ColorEntrys[1];
    SafeColorEntry[2] := ColorEntrys[2];
    SafeColorEntry[3] := ColorEntrys[3];

    GDALCore.GDALSetColorEntry(FObj, ColorIndex, SafeColorEntry[0]);
  end;
end;

//' ----------------------------------------------------------------------------
function TGDALColorTable.GetColorEntry(const ColorIndex : longint; ColorEntrys : TIntArrays4) : longint;
var
  SafeColorEntry : array[0..3] of integer;
begin
  if (FObj <> NIL) then
  begin
    GetColorEntry := GDALCore.GDALGetColorEntry(FObj, ColorIndex, SafeColorEntry[0]);
    ColorEntrys[0] := SafeColorEntry[0];
    ColorEntrys[1] := SafeColorEntry[1];
    ColorEntrys[2] := SafeColorEntry[2];
    ColorEntrys[3] := SafeColorEntry[3];
  end
  else
    GetColorEntry := GDALCore.ObjIsNULLError;
end;

//' ----------------------------------------------------------------------------
function TGDALColorTable.GetColorEntryAsRGB(const ColorIndex : longint;
                                            ColorEntrys : TIntArrays4) : longint;
var
  SafeColorEntry : array[0..3] of Integer;

begin
  if (FObj <> NIL) then
  begin
    GetColorEntryAsRGB := GDALCore.GDALGetColorEntryAsRGB(
                                  FObj, ColorIndex, SafeColorEntry[0]);
    ColorEntrys[0] := SafeColorEntry[0];
    ColorEntrys[1] := SafeColorEntry[1];
    ColorEntrys[2] := SafeColorEntry[2];
    ColorEntrys[3] := SafeColorEntry[3];
  end
  else
    GetColorEntryAsRGB := GDALCore.ObjIsNULLError
end;

end.

