{
'*****************************************************************************
' Build 0135
'
' Project:  GDAL Lazarus Bindings
' Purpose:  GDAL Lazarus internal support functions.
' Author:   Frank Warmerdam, warmerdam@pobox.com
' Lazarus Wrapper : Prajuab Riabroy
'*****************************************************************************
' Copyright (c) 2005, Frank Warmerdam <warmerdam@pobox.com>
'
' Permission is hereby granted, free of charge, to any person obtaining a
' copy of this software and associated documentation files (the "Software"),
' to deal in the Software without restriction, including without limitation
' the rights to use, copy, modify, merge, publish, distribute, sublicense,
' and/or sell copies of the Software, and to permit persons to whom the
' Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included
' in all copies or substantial portions of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
' OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
' FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
' DEALINGS IN THE SOFTWARE.
'*****************************************************************************
}
unit gdalcore;

{$I gdal.inc}

interface

uses
  Classes, SysUtils, gdal;


//' ****************************************************************************
//'               Declarations for C API functions.
//' ****************************************************************************

type
  TGDALMajorObject = class
  private
    FObj : TGDALMajorObjectH;
    //FOwned : boolean;
  public
    //property  Owned : boolean read FOwned write FOwned default false;
    property  Handle : TGDALMajorObjectH read FObj write FObj;
    function  GetDescription : PCHAR; virtual;
    procedure SetDescription (const Description : PCHAR);
    function  GetMetadata(Domain : PCHAR) : PPCHAR;
    function  SetMetadata(MetaData : PPCHAR; Domain : PCHAR) : PPCHAR;
    function  GetMetadataItem(Name : PCHAR; Domain : PCHAR) : PCHAR;
    function  SetMetadataItem(Name : PCHAR; Value : PCHAR; Domain : PCHAR) : TCPLErr;
    constructor Create;//(obj : TGDALMajorObjectH);
    destructor  Destroy; override;
 end;

//----------------------------------------------------------------------------
  function  GetLastErrorMsg : PCHAR;
  function  GetConfigOption(Key, Default : PCHAR) : PCHAR;
  procedure SetConfigOption(Key, Value : PCHAR);
  procedure AllRegister;

type
  TCPLErrorHandler = procedure (TeErrClass : TCPLErr; err_no : integer; const msg : PCHAR);

const
  ObjIsNULLError = 1001;

//* -------------------------------------------------------------------- */
//*      Helper functions.                                               */
//* -------------------------------------------------------------------- */
{$IFDEF DYNAMIC_GDAL}
type
  TGDALGeneralCmdLineProcessor = function (nArgc : longint; ppapszArgv : PPPCHAR; nOption : longint) : longint; stdcall;
  TGDALSwapWords = procedure (pData : POINTER;
                          nWordSize, nWordCount, nWordSkip : longint
                          ); stdcall;
  TGDALCopyWords  = procedure (pSrcData : POINTER;
                          eSrcType : TGDALDataType;
                          nSrcPixelOffset : longint;
                          pDstData : POINTER;
                          eDstType : TGDALDataType;
                          nDstPixelOffset : longint;
                          nWordCount : longint
                          ); stdcall;
  TGDALCopyBits = procedure (const pabySrcData : PGByte;
                         nSrcOffset, nSrcStep : longint;
                         pabyDstData : PGByte;
                         nDstOffset, nDstStep : longint;
                         nBitCount, nStepCount : longint
                        ); stdcall;
  TGDALLoadWorldFile = function  (const pszFilename : PCHAR;
                              padfGeoTransform : PDOUBLE
                              ) : longint; stdcall;
  TGDALReadWorldFile = function  (const pszBaseFilename : PCHAR;
                              const pszExtension : PCHAR;
                              padfGeoTransform : PDOUBLE
                              ) : longint; stdcall;
  TGDALWriteWorldFile = function  (const pszBaseFilename : PCHAR;
                               const pszExtension : PCHAR;
                               padfGeoTransform : PDOUBLE
                              ) : longint; stdcall;
  TGDALPackedDMSToDec = function  (dfPacked : double) : double; stdcall;
  TGDALDecToPackedDMS = function  (dfDec : double) : double; stdcall;
  TGDALVersionInfo = function  (const pszRequest : PCHAR) : PCHAR; stdcall;
  TGDALCheckVersion = function  (nVersionMajor, nVersionMinor : longint;
                              const pszCallingComponentName : PCHAR
                             ) : longint; stdcall; 
var
  GDALGeneralCmdLineProcessor: TGDALGeneralCmdLineProcessor;
  GDALSwapWords: TGDALSwapWords; 
  GDALCopyWords: TGDALCopyWords;
  GDALCopyBits: TGDALCopyBits;
  GDALLoadWorldFile: TGDALLoadWorldFile;
  GDALReadWorldFile: TGDALReadWorldFile;
  GDALWriteWorldFile: TGDALWriteWorldFile;
  GDALPackedDMSToDec: TGDALPackedDMSToDec;
  GDALDecToPackedDMS: TGDALDecToPackedDMS;
  GDALVersionInfo: TGDALVersionInfo;
  GDALCheckVersion: TGDALCheckVersion;


{$ELSE}
  function  GDALGeneralCmdLineProcessor(nArgc : longint; ppapszArgv : PPPCHAR;
                                        nOption : longint
                                       ) : longint; stdcall; external External_Lib name '_GDALGeneralCmdLineProcessor@12';
  procedure GDALSwapWords(pData : POINTER;
                          nWordSize, nWordCount, nWordSkip : longint
                          ); stdcall; external External_Lib name '_GDALSwapWords@16';
  procedure GDALCopyWords(pSrcData : POINTER;
                          eSrcType : TGDALDataType;
                          nSrcPixelOffset : longint;
                          pDstData : POINTER;
                          eDstType : TGDALDataType;
                          nDstPixelOffset : longint;
                          nWordCount : longint
                          ); stdcall; external External_Lib name '_GDALCopyWords@28';
  procedure GDALCopyBits(const pabySrcData : PGByte;
                         nSrcOffset, nSrcStep : longint;
                         pabyDstData : PGByte;
                         nDstOffset, nDstStep : longint;
                         nBitCount, nStepCount : longint
                        ); stdcall; external External_Lib name 'GDALCopyBits';
  function  GDALLoadWorldFile(const pszFilename : PCHAR;
                              padfGeoTransform : PDOUBLE
                              ) : longint; stdcall; external External_Lib name '_GDALLoadWorldFile@8';
  function  GDALReadWorldFile(const pszBaseFilename : PCHAR;
                              const pszExtension : PCHAR;
                              padfGeoTransform : PDOUBLE
                              ) : longint; stdcall; external External_Lib name '_GDALReadWorldFile@12';
  function  GDALWriteWorldFile(const pszBaseFilename : PCHAR;
                               const pszExtension : PCHAR;
                               padfGeoTransform : PDOUBLE
                              ) : longint; stdcall; external External_Lib name '_GDALWriteWorldFile@12';
  function  GDALPackedDMSToDec(dfPacked : double) : double; stdcall; external External_Lib name '_GDALPackedDMSToDec@8';
  function  GDALDecToPackedDMS(dfDec : double) : double; stdcall; external External_Lib name '_GDALDecToPackedDMS@8';
  function  GDALVersionInfo(const pszRequest : PCHAR) : PCHAR; stdcall; external External_Lib name '_GDALVersionInfo@4';
  function  GDALCheckVersion(nVersionMajor, nVersionMinor : longint;
                              const pszCallingComponentName : PCHAR
                             ) : longint; stdcall; external External_Lib name '_GDALCheckVersion@12';
{$ENDIF}

//' ----------------------------------------------------------------------------
//'       CPL
//' ----------------------------------------------------------------------------
{$IFDEF DYNAMIC_GDAL}
type
  TCPLError = procedure (eErrClass : TCPLErr; err_no : integer; const fmt : PCHAR); stdcall;
  TCPLGetLastErrorMsg = function :PCHAR; stdcall;
  TCSLDestroy = procedure (const CSLList : PPCHAR);stdcall;
  TCPLGetConfigOption = function (const Key : PCHAR; const Default : PCHAR) : PCHAR; stdcall;
  TCPLSetConfigOption = procedure (const Key : PCHAR; const Value : PCHAR); stdcall;
  TCPLAssert = procedure (const pszExpression : PCHAR; const pszFile : PCHAR; iLine : integer); stdcall;
  TCPLDefaultErrorHandler = procedure (eErrClass : TCPLErr; err_no : integer; const msg : PCHAR); stdcall;
  TCPLMalloc = function  (const nsize : size_t) : POINTER; stdcall;
  TCPLErrorReset = procedure ; stdcall;
  TCPLGetLastErrorNo = procedure ; stdcall;
  TCPLGetLastErrorType = function   : TCPLErr; stdcall;
  TCPLPopErrorHandler = procedure ; stdcall;
  TCPLPushErrorHandler = procedure (pfnErrorHandlerNew : TCPLErrorHandler); stdcall;
  TCPLSetErrorHandler = function  (pfnErrorHandlerNew : TCPLErrorHandler) : TCPLErrorHandler; stdcall; 

var
  CPLError: TCPLError;
  CPLGetLastErrorMsg : TCPLGetLastErrorMsg;
  CSLDestroy : TCSLDestroy;
  CPLGetConfigOption : TCPLGetConfigOption;
  CPLSetConfigOption : TCPLSetConfigOption;
  CPLAssert : TCPLAssert;
  CPLDefaultErrorHandler : TCPLDefaultErrorHandler;
  CPLMalloc: TCPLMalloc;
  CPLErrorReset : TCPLErrorReset;
  CPLGetLastErrorNo: TCPLGetLastErrorNo;
  CPLGetLastErrorType : TCPLGetLastErrorType;
  CPLPopErrorHandler : TCPLPopErrorHandler;
  CPLPushErrorHandler: TCPLPushErrorHandler;
  CPLSetErrorHandler: TCPLSetErrorHandler;

{$ELSE}
  procedure CPLError(eErrClass : TCPLErr; err_no : integer; const fmt : PCHAR); stdcall; external External_Lib name 'CPLError';
  function  CPLGetLastErrorMsg :PCHAR; stdcall; external External_Lib name '_CPLGetLastErrorMsg@0';
  procedure CSLDestroy (const CSLList : PPCHAR);stdcall; external External_Lib name '_CSLDestroy@4';
  function  CPLGetConfigOption (const Key : PCHAR; const Default : PCHAR) : PCHAR; stdcall; external External_Lib name '_CPLGetConfigOption@8';
  procedure CPLSetConfigOption (const Key : PCHAR; const Value : PCHAR); stdcall; external External_Lib name '_CPLSetConfigOption@8';
  procedure CPLAssert(const pszExpression : PCHAR; const pszFile : PCHAR; iLine : integer); stdcall; external External_Lib name '__CPLAssert@12';
  procedure CPLDefaultErrorHandler(eErrClass : TCPLErr; err_no : integer; const msg : PCHAR); stdcall; external External_Lib name '_CPLDefaultErrorHandler@12';
  function  CPLMalloc(const nsize : size_t) : POINTER; stdcall; external External_Lib name 'CPLMalloc';
  procedure CPLErrorReset; stdcall; external External_Lib name '_CPLErrorReset@0';
  procedure CPLGetLastErrorNo; stdcall; external External_Lib name '_CPLGetLastErrorNo@0';
  function  CPLGetLastErrorType : TCPLErr; stdcall; external External_Lib name '_CPLGetLastErrorType@0';
  procedure CPLPopErrorHandler; stdcall; external External_Lib name '_CPLPopErrorHandler@0';
  procedure CPLPushErrorHandler(pfnErrorHandlerNew : TCPLErrorHandler); stdcall; external External_Lib name '_CPLPushErrorHandler@4';
  function  CPLSetErrorHandler(pfnErrorHandlerNew : TCPLErrorHandler) : TCPLErrorHandler; stdcall; external External_Lib name '_CPLSetErrorHandler@4';
  //procedure CPLDebug(const pszCategory : PCHAR; const pszFormat: PCHAR); stdcall; external External_Lib name '';
  //function  CPLFree(pData : POINTER) : longint; stdcall; external External_Lib name 'CPLFree';

{$ENDIF}

// ----------------------------------------------------------------------------
//                           Call Back Function
// ----------------------------------------------------------------------------
{$IFDEF DYNAMIC_GDAL}
type
  TGDALDummyProgress = function  (dfComplete : double; const pszMessage : PCHAR;
                              pData : POINTER) : longint; stdcall;
  TGDALTermProgress = function  (dfComplete : double;const pszMessage : PCHAR;
                             pProgressArg : POINTER) : longint; stdcall;
  TGDALScaledProgress = function  (dfComplete : double; const pszMessage : PCHAR;
        	               pData : POINTER) : longint; stdcall;
  TGDALCreateScaledProgress = procedure (dfMin : double; dfMax : double;
       	                             pfnProgress : TGDALProgressFunc; pData : POINTER); stdcall;
  TGDALDestroyScaledProgress = procedure (pData : POINTER); stdcall; 

var
  GDALDummyProgress: TGDALDummyProgress;
  GDALTermProgress: TGDALTermProgress;
  GDALScaledProgress: TGDALScaledProgress;
  GDALCreateScaledProgress: TGDALCreateScaledProgress;
  GDALDestroyScaledProgress: TGDALDestroyScaledProgress;


{$ELSE}
  function  GDALDummyProgress(dfComplete : double; const pszMessage : PCHAR;
                              pData : POINTER) : longint; stdcall; external External_Lib name '_GDALDummyProgress@16';
  function  GDALTermProgress(dfComplete : double;const pszMessage : PCHAR;
                             pProgressArg : POINTER) : longint; stdcall; external External_Lib name '_GDALTermProgress@16';
  function  GDALScaledProgress(dfComplete : double; const pszMessage : PCHAR;
        	               pData : POINTER) : longint; stdcall; external External_Lib name '_GDALScaledProgress@16';
  procedure GDALCreateScaledProgress(dfMin : double; dfMax : double;
       	                             pfnProgress : TGDALProgressFunc; pData : POINTER); stdcall; external External_Lib name '_GDALCreateScaledProgress@24';
  procedure GDALDestroyScaledProgress(pData : POINTER); stdcall; external External_Lib name '_GDALDestroyScaledProgress@4';
{$ENDIF}

  // ----------------------------------------------------------------------------
  //                           Pixel Data Type
  // ----------------------------------------------------------------------------
{$IFDEF DYNAMIC_GDAL}
type
  TGDALGetDataTypeName = function (const Datatype : TGDALDataType) : PCHAR ; stdcall;
  TGDALGetDataTypeSize = function   (const Datatype : TGDALDataType) : longint; stdcall;
  TGDALDataTypeIsComplex = function   (const Datatype : TGDALDataType) : longint; stdcall;
  TGDALDataTypeUnion = function  (eType1 : TGDALDatatype; eType2 : TGDALDataType) : TGDALDataType; stdcall;

  //*! Translate a GDALColorInterp into a user displayable string. */
  TGDALGetColorInterpretationName = function  (eInterp : TGDALColorInterp) : PCHAR; stdcall;
  //*! Translate a GDALPaletteInterp into a user displayable string. */
  TGDALGetPaletteInterpretationName = function  (eInterp : TGDALPaletteInterp) : PCHAR; stdcall;
var
  GDALGetDataTypeName : TGDALGetDataTypeName;
  GDALGetDataTypeSize : TGDALGetDataTypeSize;
  GDALDataTypeIsComplex: TGDALDataTypeIsComplex;
  GDALDataTypeUnion:TGDALDataTypeUnion;
  //*! Translate a GDALColorInterp into a user displayable string. */
  GDALGetColorInterpretationName: TGDALGetColorInterpretationName;
  //*! Translate a GDALPaletteInterp into a user displayable string. */
  GDALGetPaletteInterpretationName:TGDALGetPaletteInterpretationName;

{$ELSE}
  function  GDALGetDataTypeName (const Datatype : TGDALDataType) : PCHAR ; stdcall; external External_Lib name '_GDALGetDataTypeName@4';
  function  GDALGetDataTypeSize (const Datatype : TGDALDataType) : longint; stdcall; external External_Lib name '_GDALGetDataTypeSize@4';
  function  GDALDataTypeIsComplex (const Datatype : TGDALDataType) : longint; stdcall; external External_Lib name '_GDALDataTypeIsComplex@4';
  function  GDALDataTypeUnion(eType1 : TGDALDatatype; eType2 : TGDALDataType) : TGDALDataType; stdcall; external External_Lib name '_GDALDataTypeUnion@8';

  //*! Translate a GDALColorInterp into a user displayable string. */
  function  GDALGetColorInterpretationName(eInterp : TGDALColorInterp) : PCHAR; stdcall; external External_Lib name 'GDALGetColorInterpretationName';
  //*! Translate a GDALPaletteInterp into a user displayable string. */
  function  GDALGetPaletteInterpretationName(eInterp : TGDALPaletteInterp) : PCHAR; stdcall; external External_Lib name 'GDALGetPaletteInterpretationName';
{$ENDIF}

//* ==================================================================== */
//*      major objects (dataset, and, driver, drivermanager).            */
//* ==================================================================== */
{$IFDEF DYNAMIC_GDAL}
type
  TGDALGetMetadataItem  = function  (const Handle : TGDALMajorObjectH; const Name : PCHAR;
                                 const Domain : PCHAR) : PCHAR; stdcall;
  TGDALGetMetadata = function   (const Handle : TGDALMajorObjectH; const Domain : PCHAR) : PPCHAR; stdcall;
  TGDALSetMetadataItem = function   (const Handle : TGDALMajorObjectH; const Name : PCHAR;
                                 const Value : PCHAR; const Domain : PCHAR) : TCPLErr; stdcall;
  TGDALSetMetadata = function   (const Handle : TGDALMajorObjectH; const MetaData : PPCHAR;
                             const Domain : PCHAR) : PPCHAR; stdcall;
  TGDALGetDescription = function   (const Handle : TGDALMajorObjectH) : PCHAR; stdcall;
  TGDALSetDescription = procedure  (const Handle : TGDALMajorObjectH; const Description : PCHAR); stdcall; 
var
  GDALGetMetadataItem: TGDALGetMetadataItem;
  GDALGetMetadata : TGDALGetMetadata;
  GDALSetMetadataItem: TGDALSetMetadataItem;
  GDALSetMetadata: TGDALSetMetadata;
  GDALGetDescription : TGDALGetDescription;
  GDALSetDescription : TGDALSetDescription;

{$ELSE}
  function  GDALGetMetadataItem (const Handle : TGDALMajorObjectH; const Name : PCHAR;
                                 const Domain : PCHAR) : PCHAR; stdcall; external External_Lib name '_GDALGetMetadataItem@12';
  function  GDALGetMetadata (const Handle : TGDALMajorObjectH; const Domain : PCHAR) : PPCHAR; stdcall; external External_Lib name '_GDALGetMetadata@8';
  function  GDALSetMetadataItem (const Handle : TGDALMajorObjectH; const Name : PCHAR;
                                 const Value : PCHAR; const Domain : PCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALSetMetadataItem@16';
  function  GDALSetMetadata (const Handle : TGDALMajorObjectH; const MetaData : PPCHAR;
                             const Domain : PCHAR) : PPCHAR; stdcall; external External_Lib name '_GDALSetMetadata@12';
  function  GDALGetDescription (const Handle : TGDALMajorObjectH) : PCHAR; stdcall; external External_Lib name '_GDALGetDescription@4';
  procedure GDALSetDescription (const Handle : TGDALMajorObjectH; const Description : PCHAR); stdcall; external External_Lib name '_GDALSetDescription@8';
{$ENDIF}

//' ----------------------------------------------------------------------------
//'       GDAL Dataset
//' ----------------------------------------------------------------------------
{$IFDEF DYNAMIC_GDAL}
type
  TGDALOpen  = function  (const Filename : PCHAR; const Access : TGDALAccess) : TGDALDriverH; stdcall;
  TGDALOpenShared = function  (const Filename : PCHAR; const Access : TGDALAccess) : TGDALDriverH; stdcall;
  TGDALDumpOpenDatasets = function  (var pfp : File) : longint; stdcall;
  TGDALClose = procedure (const Handle : TGDALDatasetH); stdcall;
  TGDALGetRasterXSize = function (const Handle : TGDALDatasetH) : longint; stdcall;
  TGDALGetRasterYSize = function (const Handle : TGDALDatasetH) : longint; stdcall;
  TGDALGetRasterCount = function (const Handle : TGDALDatasetH) : longint; stdcall;
  TGDALGetRasterBand = function (const Handle : TGDALDatasetH; const BandNo : longint) : TGDALRasterBandH; stdcall;
  TGDALGetProjectionRef = function   (const Handle : TGDALDatasetH) : PCHAR; stdcall;
  TGDALSetProjection = function   (const Handle : TGDALDatasetH; const WKTProj : PCHAR) : TCPLErr; stdcall;
  TGDALGetGeoTransform = function   (const Handle : TGDALDatasetH; Geotransform : PDouble) : TCPLErr; stdcall;
  TGDALSetGeoTransform = function   (const Handle : TGDALDatasetH; Geotransform : PDouble) : TCPLErr; stdcall;
  TGDALReferenceDataset = function   (const Handle : TGDALDatasetH) : longint; stdcall;
  TGDALDereferenceDataset = function  (const Handle : TGDALDatasetH) : longint; stdcall;
  TGDALFlushCache = procedure (const Handle : TGDALDatasetH); stdcall;
  TGDALAddBand = function  (hDataset : TGDALDatasetH; eType : TGDALDataType;
		                   papszOptions : PPCHAR) : TCPLErr; stdcall;
  TGDALDatasetAdviseRead = function  (hDS : TGDALDatasetH;
		                  nXOff, nYOff, nXSize,	nYSize, nBufXSize, nBufYSize : longint;
                                  eDT : TGDALDataType; nBandCount : longint;
		                  panBandMap : PINTEGER;
		                  papszOptions : PPCHAR) : TCPLErr; stdcall;
  TGDALGetInternalHandle = procedure (hDS : TGDALDatasetH;
		                  const pszRequest : PCHAR); stdcall;
  TGDALBuildOverviews = function  (hDataset : TGDALDatasetH;
		               const pszResampling : PCHAR;
		               nOverviews : longint;
		               panOverviewList : PINTEGER;
		               nListBands : longint;
		               panBandList : PINTEGER;
		               pfnProgress : TGDALProgressFunc;
		               pProgressData : POINTER) : TCPLErr; stdcall;
  TGDALGetOpenDatasets = procedure (pahDSList : PPGDALDatasetH;
		                pnCount : PINTEGER); stdcall;
  TGDALGetAccess = function  (hDS : TGDALDatasetH): longint; stdcall;
  TGDALCreateDatasetMaskBand = function  (hDS : TGDALDatasetH;
                                      Flags : longint) : TCPLErr; stdcall;
  TGDALDatasetCopyWholeRaster = function  (hSrcDS : TGDALDatasetH;
		                       hDstDS : TGDALDatasetH;
		                       papszOptions : PPCHAR;
		                       pfnProgress : TGDALProgressFunc;
		                       pProgressData : POINTER) : TCPLErr; stdcall;
  TGDALRegenerateOverviews = function  (hSrcBand : TGDALRasterBandH;
		                    nOverviewCount : longint;
		                    pahOvrBands : PGDALRasterBandH;
		                    const pszResampling : PCHAR;
		  	            pfnProgress : TGDALProgressFunc;
		                    pProgressData : POINTER) : TCPLErr; stdcall;
  TGDALGetDatasetDriver = function  (hDataset : TGDALDatasetH) : TGDALDriverH; stdcall;
  TGDALGetFileList = function  (hDS : TGDALDatasetH) : PPCHAR; stdcall;
  TGDALDatasetRasterIO = function  (hDS : TGDALDatasetH;
                                eRWFlag : TGDALRWFlag;
                                nXOff, YOff, nXSize: longint;
                                pData : POINTER;
                                nBufXSize, nBufYSize : longint;
                                eBufType : TGDALDataType;
                                nBandCount : longint;
                                panBandMap : PINTEGER;
                                nPixelSpace, nLineSpace, nBandSpace : longint
                               ) : TCPLErr; stdcall; 
var
  GDALOpen : TGDALOpen;
  GDALOpenShared : TGDALOpenShared;
  GDALDumpOpenDatasets: TGDALDumpOpenDatasets;
  GDALClose : TGDALClose;
  GDALGetRasterXSize: TGDALGetRasterXSize;
  GDALGetRasterYSize: TGDALGetRasterYSize;
  GDALGetRasterCount: TGDALGetRasterCount;
  GDALGetRasterBand: TGDALGetRasterBand;
  GDALGetProjectionRef: TGDALGetProjectionRef;
  GDALSetProjection: TGDALSetProjection;
  GDALGetGeoTransform : TGDALGetGeoTransform;
  GDALSetGeoTransform: TGDALSetGeoTransform;
  GDALReferenceDataset: TGDALReferenceDataset;
  GDALDereferenceDataset: TGDALDereferenceDataset;
  GDALFlushCache: TGDALFlushCache;
  GDALAddBand: TGDALAddBand;
  GDALDatasetAdviseRead: TGDALDatasetAdviseRead;
  GDALGetInternalHandle: TGDALGetInternalHandle;
  GDALBuildOverviews:TGDALBuildOverviews;
  GDALGetOpenDatasets: TGDALGetOpenDatasets;
  GDALGetAccess: TGDALGetAccess;
  GDALCreateDatasetMaskBand: TGDALCreateDatasetMaskBand;
  GDALDatasetCopyWholeRaster: TGDALDatasetCopyWholeRaster;
  GDALRegenerateOverviews: TGDALRegenerateOverviews;
  GDALGetDatasetDriver: TGDALGetDatasetDriver;
  GDALGetFileList: TGDALGetFileList;
  GDALDatasetRasterIO: TGDALDatasetRasterIO;

{$ELSE}
  function  GDALOpen (const Filename : PCHAR; const Access : TGDALAccess) : TGDALDriverH; stdcall; external External_Lib name '_GDALOpen@8';
  function  GDALOpenShared (const Filename : PCHAR; const Access : TGDALAccess) : TGDALDriverH; stdcall; external External_Lib name '_GDALOpenShared@8';
  function  GDALDumpOpenDatasets(var pfp : File) : longint; stdcall; external External_Lib name '_GDALDumpOpenDatasets@4';
  procedure GDALClose (const Handle : TGDALDatasetH); stdcall; external External_Lib name '_GDALClose@4';
  function  GDALGetRasterXSize (const Handle : TGDALDatasetH) : longint; stdcall; external External_Lib name '_GDALGetRasterXSize@4';
  function  GDALGetRasterYSize (const Handle : TGDALDatasetH) : longint; stdcall; external External_Lib name '_GDALGetRasterYSize@4';
  function  GDALGetRasterCount (const Handle : TGDALDatasetH) : longint; stdcall; external External_Lib name '_GDALGetRasterCount@4';
  function  GDALGetRasterBand (const Handle : TGDALDatasetH; const BandNo : longint) : TGDALRasterBandH; stdcall; external External_Lib name '_GDALGetRasterBand@8';
  function  GDALGetProjectionRef (const Handle : TGDALDatasetH) : PCHAR; stdcall; external External_Lib name '_GDALGetProjectionRef@4';
  function  GDALSetProjection (const Handle : TGDALDatasetH; const WKTProj : PCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALSetProjection@8';
  function  GDALGetGeoTransform (const Handle : TGDALDatasetH; Geotransform : PDouble) : TCPLErr; stdcall; external External_Lib name '_GDALGetGeoTransform@8';
  function  GDALSetGeoTransform (const Handle : TGDALDatasetH; Geotransform : PDouble) : TCPLErr; stdcall; external External_Lib name '_GDALSetGeoTransform@8';
  function  GDALReferenceDataset (const Handle : TGDALDatasetH) : longint; stdcall; external External_Lib name '_GDALReferenceDataset@4';
  function  GDALDereferenceDataset (const Handle : TGDALDatasetH) : longint; stdcall; external External_Lib name '_GDALSetGeoTransform@8';
  procedure GDALFlushCache (const Handle : TGDALDatasetH); stdcall; external External_Lib name '_GDALFlushCache@4';
  function  GDALAddBand(hDataset : TGDALDatasetH; eType : TGDALDataType;
		                   papszOptions : PPCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALAddBand@12';
  function  GDALDatasetAdviseRead(hDS : TGDALDatasetH;
		                  nXOff, nYOff, nXSize,	nYSize, nBufXSize, nBufYSize : longint;
                                  eDT : TGDALDataType; nBandCount : longint;
		                  panBandMap : PINTEGER;
		                  papszOptions : PPCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALDatasetAdviseRead@44';
  procedure GDALGetInternalHandle(hDS : TGDALDatasetH;
		                  const pszRequest : PCHAR); stdcall; external External_Lib name '_GDALGetInternalHandle@8';
  function  GDALBuildOverviews(hDataset : TGDALDatasetH;
		               const pszResampling : PCHAR;
		               nOverviews : longint;
		               panOverviewList : PINTEGER;
		               nListBands : longint;
		               panBandList : PINTEGER;
		               pfnProgress : TGDALProgressFunc;
		               pProgressData : POINTER) : TCPLErr; stdcall; external External_Lib name '_GDALBuildOverviews@32';
  procedure GDALGetOpenDatasets(pahDSList : PPGDALDatasetH;
		                pnCount : PINTEGER); stdcall; external External_Lib name '_GDALGetOpenDatasets@8';
  function  GDALGetAccess(hDS : TGDALDatasetH): longint; stdcall; external External_Lib name  '_GDALGetAccess@4';
  function  GDALCreateDatasetMaskBand(hDS : TGDALDatasetH;
                                      Flags : longint) : TCPLErr; stdcall; external External_Lib name '_GDALCreateDatasetMaskBand@8';
  function  GDALDatasetCopyWholeRaster(hSrcDS : TGDALDatasetH;
		                       hDstDS : TGDALDatasetH;
		                       papszOptions : PPCHAR;
		                       pfnProgress : TGDALProgressFunc;
		                       pProgressData : POINTER) : TCPLErr; stdcall; external External_Lib name '_GDALDatasetCopyWholeRaster@20';
  function  GDALRegenerateOverviews(hSrcBand : TGDALRasterBandH;
		                    nOverviewCount : longint;
		                    pahOvrBands : PGDALRasterBandH;
		                    const pszResampling : PCHAR;
		  	            pfnProgress : TGDALProgressFunc;
		                    pProgressData : POINTER) : TCPLErr; stdcall; external External_Lib name 'GDALRegenerateOverviews';
  function  GDALGetDatasetDriver(hDataset : TGDALDatasetH) : TGDALDriverH; stdcall; external External_Lib name '_GDALGetDatasetDriver@4';
  function  GDALGetFileList(hDS : TGDALDatasetH) : PPCHAR; stdcall; external External_Lib name '_GDALGetFileList@4';
  function  GDALDatasetRasterIO(hDS : TGDALDatasetH;
                                eRWFlag : TGDALRWFlag;
                                nXOff, YOff, nXSize: longint;
                                pData : POINTER;
                                nBufXSize, nBufYSize : longint;
                                eBufType : TGDALDataType;
                                nBandCount : longint;
                                panBandMap : PINTEGER;
                                nPixelSpace, nLineSpace, nBandSpace : longint
                               ) : TCPLErr; stdcall; external External_Lib name '_GDALDatasetRasterIO@60';
{$ENDIF}

//============================ GDAL_GCP=======================================*/
{$IFDEF DYNAMIC_GDAL}
type
   TGDALGetGCPCount = function  (hDS : TGDALDatasetH) : longint; stdcall;
   TGDALGetGCPProjection = function  (hDS : TGDALDatasetH) : PCHAR; stdcall;
   TGDALGetGCPs = function  (hDS : TGDALDatasetH) : PGDAL_GCP; stdcall;
   TGDALSetGCPs = function  (hDS : TGDALDatasetH; nGCPCount : longint;
      	        const pasGCPList : PGDAL_GCP;
      	        const pszGCPProjection : PCHAR) : TCPLErr; stdcall;
var
   GDALGetGCPCount: TGDALGetGCPCount;
   GDALGetGCPProjection: TGDALGetGCPProjection;
   GDALGetGCPs: TGDALGetGCPs;
   GDALSetGCPs: TGDALSetGCPs;
{$ELSE}
function  GDALGetGCPCount(hDS : TGDALDatasetH) : longint; stdcall; external External_Lib name '_GDALGetGCPCount@4';
function  GDALGetGCPProjection(hDS : TGDALDatasetH) : PCHAR; stdcall; external External_Lib name '_GDALGetGCPProjection@4';
function  GDALGetGCPs(hDS : TGDALDatasetH) : PGDAL_GCP; stdcall; external External_Lib name '_GDALGetGCPs@4';
function  GDALSetGCPs(hDS : TGDALDatasetH; nGCPCount : longint;
      	        const pasGCPList : PGDAL_GCP;
      	        const pszGCPProjection : PCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALSetGCPs@16';
{$ENDIF}
//' ----------------------------------------------------------------------------
//'       GDAL RasterBand
//' ----------------------------------------------------------------------------
{$IFDEF DYNAMIC_GDAL}
type
  TGDALGetRasterDataType = function   (const Handle : TGDALRasterBandH) : TGDALDatatype; stdcall;
  TGDALGetRasterBandXSize = function   (const Handle : TGDALRasterBandH) : longint; stdcall;
  TGDALGetRasterBandYSize = function   (const Handle : TGDALRasterBandH) : longint; stdcall;
  TGDALGetBlockSize = procedure  (const Handle : TGDALRasterBandH; XSize : PINTEGER; YSize : PINTEGER); stdcall;
  TGDALGetRasterNoDataValue = function   (const Handle : TGDALRasterBandH; pSuccess : PINTEGER) : double; stdcall;
  TGDALSetRasterNoDataValue = function   (const Handle : TGDALRasterBandH; const NoDataValue : double) : longint; stdcall;
  TGDALGetRasterMinimum = function   (const Handle : TGDALRasterBandH; pSuccess : PINTEGER) : double; stdcall;
  TGDALGetRasterMaximum = function  (const Handle : TGDALRasterBandH; pSuccess : PINTEGER) : double; stdcall;
  TGDALGetRasterOffset = function  (const Handle : TGDALRasterBandH; pSuccess : PINTEGER) : double; stdcall;
  TGDALSetRasterOffset = function  (const Handle : TGDALRasterBandH; const Offset : double) : longint; stdcall;
  TGDALGetRasterScale = function  (const Handle : TGDALRasterBandH; pSuccess : PINTEGER) : double; stdcall;
  TGDALSetRasterScale = function   (const Handle : TGDALRasterBandH; const NewScale : double) : longint; stdcall;
  TGDALRasterIO = function   (const Handle : TGDALRasterBandH; const RWFlag : TGDALRWFlag;
                          const XOff : longint; const YOff : longint;
                          const XSize : longint; const YSize : longint;
                                pData : POINTER; const BufXSize : longint;
                          const BufYSize : longint; const DataType : TGDALDatatype;
                          const PixelSpace : longint; const LineSpace : longint
                          ) : longint; stdcall;

  TGDALRasterAdviseRead = function (hRB : TGDALRasterBandH;
		                nXOff, nYOff, nXSize, nYSize, nBufXSize, nBufYSize,
                                eDT : TGDALDataType;
		                papszOptions : PPCHAR) : TCPLErr; stdcall;
  TGDALReadBlock = function (hBand : TGDALRasterBandH;
		         nXOff, nYOff : longint;
		         pData : POINTER) : TCPLErr; stdcall;
  TGDALWriteBlock = function (hBand : TGDALRasterBandH;
                          nXOff, nYOff : longint;
		          pData : POINTER) : TCPLErr; stdcall;
  TGDALGetRasterAccess = function (hBand : TGDALRasterBandH) : TGDALAccess; stdcall;
  TGDALGetBandNumber = function (hBand : TGDALRasterBandH) : longint; stdcall;
  TGDALGetBandDataset = function (hBand : TGDALRasterBandH) : TGDALDatasetH; stdcall;
  TGDALGetRasterColorInterpretation = function (hBand : TGDALRasterBandH) : TGDALColorInterp; stdcall;
  TGDALSetRasterColorInterpretation = function (hBand : TGDALRasterBandH;
		                            eColorInterp : TGDALColorInterp) : TCPLErr; stdcall;
  TGDALHasArbitraryOverviews = function (hBand : TGDALRasterBandH) : longint; stdcall;
  TGDALGetOverviewCount = function (hBand : TGDALRasterBandH) : longint; stdcall;
  TGDALGetOverview = function (hBand : TGDALRasterBandH; i : longint) : TGDALRasterBandH; stdcall;
  TGDALGetRasterCategoryNames = function (hBand : TGDALRasterBandH) : PPCHAR; stdcall;
  TGDALSetRasterCategoryNames = function (hBand : TGDALRasterBandH;
  		                      papszNames : PPCHAR) : TCPLErr; stdcall;
  TGDALGetRasterStatistics = function (hBand : TGDALRasterBandH;
  		                   bApproxOK, bForce : longint;
  		                   pdfMin, pdfMax, pdfMean, pdfStdDev : PDOUBLE
  	                          ) : TCPLErr; stdcall;
  TGDALComputeRasterStatistics = function  (hBand : TGDALRasterBandH;
                                        bApproxOK : longint;
                                        pdfMin, pdfMax, pdfMean, pdfStdDev : PDOUBLE;
                                        pfnProgress : TGDALProgressFunc;
                                        pProgressData : POINTER) : TCPLErr; stdcall;
  TGDALSetRasterStatistics = function  (hBand : TGDALRasterBandH;
                                    bApproxOK, bForce : longint;
                                    pdfMin, pdfMax, pdfMean, pdfStdDev : double
  	                            ) : TCPLErr; stdcall;
  TGDALGetRasterUnitType = function  (hBand : TGDALRasterBandH) : PCHAR; stdcall;
  TGDALComputeRasterMinMax = procedure (hBand : TGDALRasterBandH;
                                    bApproxOK : longint;
                                    adfMinMax : TDoubleArray2); stdcall;
  TGDALFlushRasterCache = function  (hBand : TGDALRasterBandH) : TCPLErr; stdcall;
  TGDALGetRasterHistogram = function  (hBand : TGDALRasterBandH;
                                   dfMin, dfMax : double;
                                   nBuckets : longint;
                                   panHistogram : PINTEGER;
                                   bIncludeOutOfRange, bApproxOK : longint;
                                   pfnProgress : TGDALProgressFunc;
                                   pProgressData : POINTER) : TCPLErr; stdcall;
  TGDALGetDefaultHistogram = function  (hBand : TGDALRasterBandH;
                                    pdfMin, pdfMax : PDOUBLE;
                                    pnBuckets : PINTEGER;
                                    ppanHistogram : PPINTEGER;
                                    bForce : longint;
                                    pfnProgress : TGDALProgressFunc;
                                    pProgressData : POINTER) : TCPLErr; stdcall;
  TGDALSetDefaultHistogram = function  (hBand : TGDALRasterBandH;
                                    dfMin, dfMax : double;
                                    nBuckets : longint;
                                    panHistogram : PINTEGER) : TCPLErr; stdcall;
  TGDALGetRasterSampleOverview = function  (hBand : TGDALRasterBandH;
                                        nDesiredSamples : longint
                                       ) : TGDALRasterBandH; stdcall;
  TGDALFillRaster = function  (hBand : TGDALRasterBandH;
                           dfRealValue, dfImaginaryValue : double
                          ) : TCPLErr; stdcall;
  TGDALGetDefaultRAT = function  (hBand : TGDALRasterBandH) : TGDALRasterAttributeTableH;
  TGDALSetDefaultRAT = function  (hBand : TGDALRasterBandH;
                              hRAt : TGDALRasterAttributeTableH
                             ) : TCPLErr; stdcall;
  TGDALAddDerivedBandPixelFunc = function  (const pszFuncName : PCHAR;
                                        pfnNewFunction : TGDALDerivedPixelFunc
                                       ) : TCPLErr; stdcall;
  TGDALGetMaskBand = function  (hBand : TGDALRasterBandH) : TGDALRasterBandH; stdcall;
  TGDALGetMaskFlags = function  (hBand : TGDALRasterBandH) : longint; stdcall;
  TGDALCreateMaskBand = function  (hBand : TGDALRasterBandH; nFlags : longint
                              ) : TCPLErr; stdcall;


var
  GDALGetRasterDataType :TGDALGetRasterDataType;
  GDALGetRasterBandXSize:TGDALGetRasterBandXSize;
  GDALGetRasterBandYSize:TGDALGetRasterBandYSize;
  GDALGetBlockSize : TGDALGetBlockSize;
  GDALGetRasterNoDataValue:TGDALGetRasterNoDataValue;
  GDALSetRasterNoDataValue:TGDALSetRasterNoDataValue;
  GDALGetRasterMinimum:TGDALGetRasterMinimum;
  GDALGetRasterMaximum:TGDALGetRasterMaximum;
  GDALGetRasterOffset : TGDALGetRasterOffset;
  GDALSetRasterOffset: TGDALSetRasterOffset;
  GDALGetRasterScale: TGDALGetRasterScale;
  GDALSetRasterScale: TGDALSetRasterScale;
  GDALRasterIO: TGDALRasterIO;
  GDALRasterAdviseRead: TGDALRasterAdviseRead;
  GDALReadBlock: TGDALReadBlock;
  GDALWriteBlock: TGDALWriteBlock;
  GDALGetRasterAccess: TGDALGetRasterAccess;
  GDALGetBandNumber: TGDALGetBandNumber;
  GDALGetBandDataset: TGDALGetBandDataset;
  GDALGetRasterColorInterpretation: TGDALGetRasterColorInterpretation;
  GDALSetRasterColorInterpretation: TGDALSetRasterColorInterpretation;
  GDALHasArbitraryOverviews: TGDALHasArbitraryOverviews;
  GDALGetOverviewCount: TGDALGetOverviewCount;
  GDALGetOverview: TGDALGetOverview;
  GDALGetRasterCategoryNames: TGDALGetRasterCategoryNames;
  GDALSetRasterCategoryNames: TGDALSetRasterCategoryNames;
  GDALGetRasterStatistics: TGDALGetRasterStatistics;
  GDALComputeRasterStatistics: TGDALComputeRasterStatistics;
  GDALSetRasterStatistics: TGDALSetRasterStatistics;
  GDALGetRasterUnitType: TGDALGetRasterUnitType;
  GDALComputeRasterMinMax: TGDALComputeRasterMinMax;
  GDALFlushRasterCache: TGDALFlushRasterCache;
  GDALGetRasterHistogram: TGDALGetRasterHistogram;
  GDALGetDefaultHistogram: TGDALGetDefaultHistogram;
  GDALSetDefaultHistogram: TGDALSetDefaultHistogram;
  GDALGetRasterSampleOverview: TGDALGetRasterSampleOverview;
  GDALFillRaster: TGDALFillRaster;
  GDALGetDefaultRAT: TGDALGetDefaultRAT;
  GDALSetDefaultRAT: TGDALSetDefaultRAT;
  GDALAddDerivedBandPixelFunc: TGDALAddDerivedBandPixelFunc;
  GDALGetMaskBand: TGDALGetMaskBand;
  GDALGetMaskFlags: TGDALGetMaskFlags;
  GDALCreateMaskBand: TGDALCreateMaskBand;



{$ELSE}
  function  GDALGetRasterDataType (const Handle : TGDALRasterBandH) : TGDALDatatype; stdcall; external External_Lib name '_GDALGetRasterDataType@4';
  function  GDALGetRasterBandXSize (const Handle : TGDALRasterBandH) : longint; stdcall; external External_Lib name '_GDALGetRasterBandXSize@4';
  function  GDALGetRasterBandYSize (const Handle : TGDALRasterBandH) : longint; stdcall; external External_Lib name '_GDALGetRasterBandYSize@4';
  procedure GDALGetBlockSize (const Handle : TGDALRasterBandH; XSize : PINTEGER; YSize : PINTEGER); stdcall; external External_Lib name '_GDALGetBlockSize@12';
  function  GDALGetRasterNoDataValue (const Handle : TGDALRasterBandH; pSuccess : PINTEGER) : double; stdcall; external External_Lib name '_GDALGetRasterNoDataValue@8';
  function  GDALSetRasterNoDataValue (const Handle : TGDALRasterBandH; const NoDataValue : double) : longint; stdcall; external External_Lib name '_GDALSetRasterNoDataValue@12';
  function  GDALGetRasterMinimum (const Handle : TGDALRasterBandH; pSuccess : PINTEGER) : double; stdcall; external External_Lib name '_GDALGetRasterMinimum@8';
  function  GDALGetRasterMaximum (const Handle : TGDALRasterBandH; pSuccess : PINTEGER) : double; stdcall; external External_Lib name '_GDALGetRasterMaximum@8';
  function  GDALGetRasterOffset (const Handle : TGDALRasterBandH; pSuccess : PINTEGER) : double; stdcall; external External_Lib name '_GDALGetRasterOffset@8';
  function  GDALSetRasterOffset (const Handle : TGDALRasterBandH; const Offset : double) : longint; stdcall; external External_Lib name '_GDALSetRasterOffset@12';
  function  GDALGetRasterScale (const Handle : TGDALRasterBandH; pSuccess : PINTEGER) : double; stdcall; external External_Lib name '_GDALGetRasterScale@8';
  function  GDALSetRasterScale (const Handle : TGDALRasterBandH; const NewScale : double) : longint; stdcall; external External_Lib name '_GDALSetRasterScale@12';
  function  GDALRasterIO (const Handle : TGDALRasterBandH; const RWFlag : TGDALRWFlag;
                          const XOff : longint; const YOff : longint;
                          const XSize : longint; const YSize : longint;
                                pData : POINTER; const BufXSize : longint;
                          const BufYSize : longint; const DataType : TGDALDatatype;
                          const PixelSpace : longint; const LineSpace : longint
                          ) : longint; stdcall; external External_Lib name '_GDALRasterIO@48';

  function GDALRasterAdviseRead(hRB : TGDALRasterBandH;
		                nXOff, nYOff, nXSize, nYSize, nBufXSize, nBufYSize,
                                eDT : TGDALDataType;
		                papszOptions : PPCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALRasterAdviseRead@36';
  function GDALReadBlock(hBand : TGDALRasterBandH;
		         nXOff, nYOff : longint;
		         pData : POINTER) : TCPLErr; stdcall; external External_Lib name '_GDALReadBlock@16';
  function GDALWriteBlock(hBand : TGDALRasterBandH;
                          nXOff, nYOff : longint;
		          pData : POINTER) : TCPLErr; stdcall; external External_Lib name '_GDALWriteBlock@16';
  function GDALGetRasterAccess(hBand : TGDALRasterBandH) : TGDALAccess; stdcall; external External_Lib name '_GDALGetRasterAccess@4';
  function GDALGetBandNumber(hBand : TGDALRasterBandH) : longint; stdcall; external External_Lib name '_GDALGetBandNumber@4';
  function GDALGetBandDataset(hBand : TGDALRasterBandH) : TGDALDatasetH; stdcall; external External_Lib name '_GDALGetBandDataset@4';
  function GDALGetRasterColorInterpretation(hBand : TGDALRasterBandH) : TGDALColorInterp; stdcall; external External_Lib name '_GDALGetRasterColorInterpretation@4';
  function GDALSetRasterColorInterpretation(hBand : TGDALRasterBandH;
		                            eColorInterp : TGDALColorInterp) : TCPLErr; stdcall; external External_Lib name '_GDALSetRasterColorInterpretation@8';
  function GDALHasArbitraryOverviews(hBand : TGDALRasterBandH) : longint; stdcall; external External_Lib name '_GDALHasArbitraryOverviews@4';
  function GDALGetOverviewCount(hBand : TGDALRasterBandH) : longint; stdcall; external External_Lib name '_GDALGetOverviewCount@4';
  function GDALGetOverview(hBand : TGDALRasterBandH; i : longint) : TGDALRasterBandH; stdcall; external External_Lib name '_GDALGetOverview@8';
  function GDALGetRasterCategoryNames(hBand : TGDALRasterBandH) : PPCHAR; stdcall; external External_Lib name '_GDALGetRasterCategoryNames@4';
  function GDALSetRasterCategoryNames(hBand : TGDALRasterBandH;
  		                      papszNames : PPCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALSetRasterCategoryNames@8';
  function GDALGetRasterStatistics(hBand : TGDALRasterBandH;
  		                   bApproxOK, bForce : longint;
  		                   pdfMin, pdfMax, pdfMean, pdfStdDev : PDOUBLE
  	                          ) : TCPLErr; stdcall; external External_Lib name '_GDALGetRasterStatistics@28';
  function  GDALComputeRasterStatistics(hBand : TGDALRasterBandH;
                                        bApproxOK : longint;
                                        pdfMin, pdfMax, pdfMean, pdfStdDev : PDOUBLE;
                                        pfnProgress : TGDALProgressFunc;
                                        pProgressData : POINTER) : TCPLErr; stdcall; external External_Lib name '_GDALComputeRasterStatistics@32';
  function  GDALSetRasterStatistics(hBand : TGDALRasterBandH;
                                    bApproxOK, bForce : longint;
                                    pdfMin, pdfMax, pdfMean, pdfStdDev : double
  	                            ) : TCPLErr; stdcall; external External_Lib name '_GDALSetRasterStatistics@28';
  function  GDALGetRasterUnitType(hBand : TGDALRasterBandH) : PCHAR; stdcall; external External_Lib name '_GDALGetRasterUnitType@4';
  procedure GDALComputeRasterMinMax(hBand : TGDALRasterBandH;
                                    bApproxOK : longint;
                                    adfMinMax : TDoubleArray2); stdcall; external External_Lib name '_GDALComputeRasterMinMax@12';
  function  GDALFlushRasterCache(hBand : TGDALRasterBandH) : TCPLErr; stdcall; external External_Lib name '_GDALFlushRasterCache@4';
  function  GDALGetRasterHistogram(hBand : TGDALRasterBandH;
                                   dfMin, dfMax : double;
                                   nBuckets : longint;
                                   panHistogram : PINTEGER;
                                   bIncludeOutOfRange, bApproxOK : longint;
                                   pfnProgress : TGDALProgressFunc;
                                   pProgressData : POINTER) : TCPLErr; stdcall; external External_Lib name '_GDALGetRasterHistogram@44';
  function  GDALGetDefaultHistogram(hBand : TGDALRasterBandH;
                                    pdfMin, pdfMax : PDOUBLE;
                                    pnBuckets : PINTEGER;
                                    ppanHistogram : PPINTEGER;
                                    bForce : longint;
                                    pfnProgress : TGDALProgressFunc;
                                    pProgressData : POINTER) : TCPLErr; stdcall; external External_Lib name '_GDALGetDefaultHistogram@32';
  function  GDALSetDefaultHistogram(hBand : TGDALRasterBandH;
                                    dfMin, dfMax : double;
                                    nBuckets : longint;
                                    panHistogram : PINTEGER) : TCPLErr; stdcall; external External_Lib name '_GDALSetDefaultHistogram@28';
  function  GDALGetRasterSampleOverview(hBand : TGDALRasterBandH;
                                        nDesiredSamples : longint
                                       ) : TGDALRasterBandH; stdcall; external External_Lib name '_GDALGetRasterSampleOverview@8';
  function  GDALFillRaster(hBand : TGDALRasterBandH;
                           dfRealValue, dfImaginaryValue : double
                          ) : TCPLErr; stdcall; external External_Lib name '_GDALFillRaster@20';
  function  GDALGetDefaultRAT(hBand : TGDALRasterBandH) : TGDALRasterAttributeTableH; external External_Lib name '_GDALGetDefaultRAT@4';
  function  GDALSetDefaultRAT(hBand : TGDALRasterBandH;
                              hRAt : TGDALRasterAttributeTableH
                             ) : TCPLErr; stdcall; external External_Lib name '_GDALSetDefaultRAT@8';
  function  GDALAddDerivedBandPixelFunc(const pszFuncName : PCHAR;
                                        pfnNewFunction : TGDALDerivedPixelFunc
                                       ) : TCPLErr; stdcall; external External_Lib name '_GDALAddDerivedBandPixelFunc@8';
  function  GDALGetMaskBand(hBand : TGDALRasterBandH) : TGDALRasterBandH; stdcall; external External_Lib name '_GDALGetMaskBand@4';
  function  GDALGetMaskFlags(hBand : TGDALRasterBandH) : longint; stdcall; external External_Lib name '_GDALGetMaskFlags@4';
  function  GDALCreateMaskBand(hBand : TGDALRasterBandH; nFlags : longint
                              ) : TCPLErr; stdcall; external External_Lib name '_GDALCreateMaskBand@8';

{$ENDIF}

//' ----------------------------------------------------------------------------
//'       GDALDriver
//' ----------------------------------------------------------------------------
{$IFDEF DYNAMIC_GDAL}
type
  TGDALAllRegister = procedure; stdcall;
  TGDALGetDriverByName = function   (const Filename : PCHAR) : TGDALDriverH; stdcall;
  TGDALGetDriverCount = function   : longint; stdcall;
  TGDALGetDriver = function   (const DriverIndex : longint) : TGDALDriverH; stdcall;
  TGDALDestroyDriverManager = procedure  ; stdcall;
  TGDALCreate = function   (const Handle : TGDALDriverH; const Filename : PCHAR;
                        const XSize : longint; const YSize : longint; const BandCount : longint;
                        const DataType : TGDALDataType; const Options : PPCHAR
                        ) : TGDALDatasetH; stdcall;
  TGDALCreateCopy = function   (const Handle : TGDALDriverH; const Filename : PCHAR;
                            const SrcDS : TGDALDatasetH; const ApproxOK : longint; const Options : PPCHAR;
                            const ProgressFunc : TGDALDatasetH; const ProgressArg : longint) : TGDALDatasetH; stdcall;
  TGDALDestroyDriver = procedure (hDriver : TGDALDriverH); stdcall;
  TGDALRegisterDriver = function  (hDriver : TGDALDriverH) : longint; stdcall;
  TGDALDeregisterDriver = procedure (hDriver : TGDALDriverH); stdcall;
  TGDALValidateCreationOptions = function  (hDriver : TGDALDriverH;
		                        papszCreationOptions : PPCHAR) : longint; stdcall;
  TGDALGetDriverShortName = function  (hDriver : TGDALDriverH) : PCHAR; stdcall;
  TGDALGetDriverLongName = function  (hDriver : TGDALDriverH) : PCHAR; stdcall;
  TGDALGetDriverHelpTopic = function  (hDriver : TGDALDriverH) : PCHAR; stdcall;
  TGDALGetDriverCreationOptionList = function  (hDriver : TGDALDriverH) : PCHAR; stdcall;
  TGDALGCPsToGeoTransform = function  (nGCPCount : longint; const pasGCPs : PGDAL_GCP;
		                   padfGeoTransform : PDOUBLE;
                                   bApproxOK : longint) : longint; stdcall;
  TGDALInvGeoTransform = function  (gt_in : PDOUBLE; gt_out : PDOUBLE) : longint; stdcall;
  TGDALApplyGeoTransform = procedure (padfGeoTransform : PDOUBLE;
		                  dfPixel : double;
		                  dfLine : double;
		                  pdfGeoX : PDOUBLE;
		                  pdfGeoY : PDOUBLE); stdcall;

  TGDALDeleteDataset = function  (hDriver : TGDALDriverH; const pszFilename : PCHAR) : TCPLErr; stdcall;
  TGDALRenameDataset = function  (hDriver : TGDALDriverH; const pszNewName : PCHAR;
                              const pszOldName : PCHAR) : TCPLErr; stdcall;
  TGDALCopyDatasetFiles  = function  (hDriver : TGDALDriverH; const pszNewName : PCHAR;
                                 const pszOldName : PCHAR) : TCPLErr; stdcall;
  TGDALIdentifyDriver = function  (const pszFilename : PCHAR;
		               papszFileList : PPCHAR) : TGDALDriverH; stdcall;

var
  GDALAllRegister: TGDALAllRegister;
  GDALGetDriverByName: TGDALGetDriverByName;
  GDALGetDriverCount: TGDALGetDriverCount;
  GDALGetDriver: TGDALGetDriver;
  GDALDestroyDriverManager: TGDALDestroyDriverManager;
  GDALCreate : TGDALCreate ;
  GDALCreateCopy: TGDALCreateCopy;
  GDALDestroyDriver: TGDALDestroyDriver;
  GDALRegisterDriver: TGDALRegisterDriver;
  GDALDeregisterDriver: TGDALDeregisterDriver;
  GDALValidateCreationOptions: TGDALValidateCreationOptions;
  GDALGetDriverShortName: TGDALGetDriverShortName;
  GDALGetDriverLongName: TGDALGetDriverLongName;
  GDALGetDriverHelpTopic: TGDALGetDriverHelpTopic;
  GDALGetDriverCreationOptionList: TGDALGetDriverCreationOptionList;
  GDALGCPsToGeoTransform: TGDALGCPsToGeoTransform;
  GDALInvGeoTransform: TGDALInvGeoTransform;
  GDALApplyGeoTransform: TGDALApplyGeoTransform;

  GDALDeleteDataset: TGDALDeleteDataset;
  GDALRenameDataset: TGDALRenameDataset;
  GDALCopyDatasetFiles: TGDALCopyDatasetFiles;
  GDALIdentifyDriver: TGDALIdentifyDriver;


{$ELSE}
  procedure GDALAllRegister ; stdcall; external External_Lib name '_GDALAllRegister@0';
  function  GDALGetDriverByName (const Filename : PCHAR) : TGDALDriverH; stdcall; external External_Lib name '_GDALGetDriverByName@4';
  function  GDALGetDriverCount : longint; stdcall; external External_Lib name '_GDALGetDriverCount@0';
  function  GDALGetDriver (const DriverIndex : longint) : TGDALDriverH; stdcall; external External_Lib name '_GDALGetDriver@4';
  procedure GDALDestroyDriverManager ; stdcall; external External_Lib name '_GDALDestroyDriverManager@0';
  function  GDALCreate (const Handle : TGDALDriverH; const Filename : PCHAR;
                        const XSize : longint; const YSize : longint; const BandCount : longint;
                        const DataType : TGDALDataType; const Options : PPCHAR
                        ) : TGDALDatasetH; stdcall; external External_Lib name '_GDALCreate@28';
  function  GDALCreateCopy (const Handle : TGDALDriverH; const Filename : PCHAR;
                            const SrcDS : TGDALDatasetH; const ApproxOK : longint; const Options : PPCHAR;
                            const ProgressFunc : TGDALDatasetH; const ProgressArg : longint) : TGDALDatasetH; stdcall; external External_Lib name '_GDALCreateCopy@28';
  procedure GDALDestroyDriver(hDriver : TGDALDriverH); stdcall; external External_Lib name '_GDALDestroyDriver@4';
  function  GDALRegisterDriver(hDriver : TGDALDriverH) : longint; stdcall; external External_Lib name '_GDALRegisterDriver@4';
  procedure GDALDeregisterDriver(hDriver : TGDALDriverH); stdcall; external External_Lib name '_GDALDeregisterDriver@4';
  function  GDALValidateCreationOptions(hDriver : TGDALDriverH;
		                        papszCreationOptions : PPCHAR) : longint; stdcall; external External_Lib name '_GDALValidateCreationOptions@8';
  function  GDALGetDriverShortName(hDriver : TGDALDriverH) : PCHAR; stdcall; external External_Lib name '_GDALGetDriverShortName@4';
  function  GDALGetDriverLongName(hDriver : TGDALDriverH) : PCHAR; stdcall; external External_Lib name '_GDALGetDriverLongName@4';
  function  GDALGetDriverHelpTopic(hDriver : TGDALDriverH) : PCHAR; stdcall; external External_Lib name '_GDALGetDriverHelpTopic@4';
  function  GDALGetDriverCreationOptionList(hDriver : TGDALDriverH) : PCHAR; stdcall; external External_Lib name '_GDALGetDriverCreationOptionList@4';
  function  GDALGCPsToGeoTransform(nGCPCount : longint; const pasGCPs : PGDAL_GCP;
		                   padfGeoTransform : PDOUBLE;
                                   bApproxOK : longint) : longint; stdcall; external External_Lib name '_GDALGCPsToGeoTransform@16';
  function  GDALInvGeoTransform(gt_in : PDOUBLE; gt_out : PDOUBLE) : longint; stdcall; external External_Lib name '_GDALInvGeoTransform@8';
  procedure GDALApplyGeoTransform(padfGeoTransform : PDOUBLE;
		                  dfPixel : double;
		                  dfLine : double;
		                  pdfGeoX : PDOUBLE;
		                  pdfGeoY : PDOUBLE); stdcall; external External_Lib name '_GDALApplyGeoTransform@28';

  function  GDALDeleteDataset(hDriver : TGDALDriverH; const pszFilename : PCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALDeleteDataset@8';
  function  GDALRenameDataset(hDriver : TGDALDriverH; const pszNewName : PCHAR;
                              const pszOldName : PCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALRenameDataset@12';
  function  GDALCopyDatasetFiles(hDriver : TGDALDriverH; const pszNewName : PCHAR;
                                 const pszOldName : PCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALCopyDatasetFiles@12';
  function  GDALIdentifyDriver(const pszFilename : PCHAR;
		               papszFileList : PPCHAR) : TGDALDriverH; stdcall; external External_Lib name '_GDALIdentifyDriver@8';


{$ENDIF}

//' ----------------------------------------------------------------------------
//'       GDALColorTable
//' ----------------------------------------------------------------------------
{$IFDEF DYNAMIC_GDAL}
type
  TGDALCreateColorTable = function   (const PaletteInterp : TGDALPaletteInterp) : TGDALColorTableH; stdcall;
  TGDALDestroyColorTable = procedure  (const Handle : TGDALColorTableH); stdcall;
  TGDALCloneColorTable = function   (const Handle : TGDALColorTableH) : TGDALColorTableH; stdcall;
  TGDALGetPaletteInterpretation = function   (const Handle : TGDALColorTableH) : longint; stdcall;
  TGDALGetColorEntryCount = function   (const Handle : TGDALColorTableH) : longint; stdcall;
  TGDALGetColorEntryAsRGB = function   (const Handle : TGDALColorTableH; const ColorIndex : longint;
                                          ColorEntry : Integer) : longint; stdcall;
  TGDALGetColorEntry = function   (const Handle : TGDALColorTableH; const ColorIndex : longint;
                                     ColorEntry : Integer) : longint; stdcall;
  TGDALSetColorEntry = procedure  (const Handle : TGDALColorTableH; const ColorIndex : longint;
                                     ColorEntry : Integer); stdcall;
  TGDALCreateColorRamp = procedure (hTable : TGDALColorTableH;
                                nStartIndex : longint;
                                const psStartColor : PGDALColorEntry;
                                nEndIndex : longint;
                                const psEndColor : PGDALColorEntry
                               ); stdcall;
  TGDALGetRasterColorTable = function  (const Handle : TGDALRasterBandH) : TGDALColorTableH; stdcall;
  TGDALSetRasterColorTable = function  (const BandHandle : TGDALRasterBandH; const TableHandle : TGDALColorTableH) : TCPLErr; stdcall; 

var
  GDALCreateColorTable:  TGDALCreateColorTable;
  GDALDestroyColorTable:  TGDALDestroyColorTable;
  GDALCloneColorTable: TGDALCloneColorTable;
  GDALGetPaletteInterpretation: TGDALGetPaletteInterpretation;
  GDALGetColorEntryCount: TGDALGetColorEntryCount;
  GDALGetColorEntryAsRGB: TGDALGetColorEntryAsRGB;
  GDALGetColorEntry: TGDALGetColorEntry;
  GDALSetColorEntry: TGDALSetColorEntry;
  GDALCreateColorRamp: TGDALCreateColorRamp;
  GDALGetRasterColorTable: TGDALGetRasterColorTable;
  GDALSetRasterColorTable: TGDALSetRasterColorTable;


{$ELSE}
  function  GDALCreateColorTable (const PaletteInterp : TGDALPaletteInterp) : TGDALColorTableH; stdcall; external External_Lib name '_GDALCreateColorTable@4';
  procedure GDALDestroyColorTable (const Handle : TGDALColorTableH); stdcall; external External_Lib name '_GDALDestroyColorTable@4';
  function  GDALCloneColorTable (const Handle : TGDALColorTableH) : TGDALColorTableH; stdcall; external External_Lib name '_GDALCloneColorTable@4';
  function  GDALGetPaletteInterpretation (const Handle : TGDALColorTableH) : longint; stdcall; external External_Lib name '_GDALGetPaletteInterpretation@4';
  function  GDALGetColorEntryCount (const Handle : TGDALColorTableH) : longint; stdcall; external External_Lib name '_GDALGetColorEntryCount@4';
  function  GDALGetColorEntryAsRGB (const Handle : TGDALColorTableH; const ColorIndex : longint;
                                          ColorEntry : Integer) : longint; stdcall; external External_Lib name '_GDALGetColorEntryAsRGB@12';
  function  GDALGetColorEntry (const Handle : TGDALColorTableH; const ColorIndex : longint;
                                     ColorEntry : Integer) : longint; stdcall; external External_Lib name '_GDALGetColorEntry@12';
  procedure GDALSetColorEntry (const Handle : TGDALColorTableH; const ColorIndex : longint;
                                     ColorEntry : Integer); stdcall; external External_Lib name '_GDALSetColorEntry@12';
  procedure GDALCreateColorRamp(hTable : TGDALColorTableH;
                                nStartIndex : longint;
                                const psStartColor : PGDALColorEntry;
                                nEndIndex : longint;
                                const psEndColor : PGDALColorEntry
                               ); stdcall; external External_Lib name '_GDALCreateColorRamp@20';
  function GDALGetRasterColorTable (const Handle : TGDALRasterBandH) : TGDALColorTableH; stdcall; external External_Lib name '_GDALGetRasterColorTable@4';
  function GDALSetRasterColorTable (const BandHandle : TGDALRasterBandH; const TableHandle : TGDALColorTableH) : TCPLErr; stdcall; external External_Lib name '_GDALSetRasterColorTable@8';
{$ENDIF}

//* ==================================================================== */
//*      Raster Attribute Table                                          */
//* ==================================================================== */
{$IFDEF DYNAMIC_GDAL}
type
  TGDALCreateRasterAttributeTable = function   : TGDALRasterAttributeTableH; stdcall;
  TGDALDestroyRasterAttributeTable = procedure (hRAT : TGDALRasterAttributeTableH ); stdcall;
  TGDALRATGetColumnCount = function  (hRAT : TGDALRasterAttributeTableH ) : longint; stdcall;
  TGDALRATGetNameOfCol = function  (hRAT : TGDALRasterAttributeTableH;
                                iCol : longint ) : PCHAR; stdcall;
  TGDALRATGetUsageOfCol = function  (hRAT : TGDALRasterAttributeTableH;
                                iCol : longint) : TGDALRATFieldUsage; stdcall;
  TGDALRATGetTypeOfCol = function  (hRAT : TGDALRasterAttributeTableH;
                                iCol : longint) : TGDALRATFieldType; stdcall;
  TGDALRATGetColOfUsage = function  (hRAT : TGDALRasterAttributeTableH;
                                 eUsage : TGDALRATFieldUsage ) : longint; stdcall;
  TGDALRATGetRowCount = function  ( hRAT : TGDALRasterAttributeTableH) : longint; stdcall;
  TGDALRATGetValueAsString = function  (hRAT : TGDALRasterAttributeTableH;
                                 iRow, iField : longint) : PCHAR; stdcall;
  TGDALRATGetValueAsInt = function  (hRAT : TGDALRasterAttributeTableH;
                                 iRow, iField, nValue : longint) : longint; stdcall;
  TGDALRATGetValueAsDouble = function  (hRAT : TGDALRasterAttributeTableH;
                                 iRow, iField : longint) : double; stdcall;
  TGDALRATSetValueAsString = procedure (hRAT : TGDALRasterAttributeTableH;
                                    iRow, iField : longint;
                                    const pszValue : PCHAR); stdcall;
  TGDALRATSetValueAsInt = procedure (hRAT : TGDALRasterAttributeTableH;
                                 iRow, iField, nValue : longint); stdcall;
  TGDALRATSetValueAsDouble = procedure (hRAT : TGDALRasterAttributeTableH;
                                    iRow, iField : longint;
                                    dfValue : double ); stdcall;
  TGDALRATSetRowCount = procedure (hRAT : TGDALRasterAttributeTableH;
                               nNewCount : longint); stdcall;
  TGDALRATCreateColumn = function  (hRAT : TGDALRasterAttributeTableH;
            pszFieldName : PCHAR;
            eFieldType : TGDALRATFieldType;
            eFieldUsage : TGDALRATFieldUsage) : TCPLErr; stdcall;
  TGDALRATSetLinearBinning = function  (hRAT : TGDALRasterAttributeTableH;
                                    dfRow0Min, dfBinSize : double ) : TCPLErr; stdcall;
  TGDALRATGetLinearBinning = function  (hRAT : TGDALRasterAttributeTableH;
                                    pdfRow0Min, pdfBinSize :PDOUBLE) : longint; stdcall;
  TGDALRATInitializeFromColorTable = function  (hRAT : TGDALRasterAttributeTableH;
                                            hCT : TGDALColorTableH ) : TCPLErr; stdcall;
  TGDALRATTranslateToColorTable = function  (hRAT : TGDALRasterAttributeTableH;
                                         nEntryCount : longint) : TGDALColorTableH; stdcall;
  TGDALRATDumpReadable = procedure (hRAT : TGDALRasterAttributeTableH; var fp : FILE ); stdcall;
  TGDALRATClone = function  (hRAT : TGDALRasterAttributeTableH ) : TGDALRasterAttributeTableH; stdcall;
  TGDALRATGetRowOfValue = function  (hRAT : TGDALRasterAttributeTableH;
                                 dfValue : double ) : longint; stdcall;

var
    GDALCreateRasterAttributeTable:  TGDALCreateRasterAttributeTable;
    GDALDestroyRasterAttributeTable:  TGDALDestroyRasterAttributeTable;
    GDALRATGetColumnCount:  TGDALRATGetColumnCount;
    GDALRATGetNameOfCol:  TGDALRATGetNameOfCol;
    GDALRATGetUsageOfCol:  TGDALRATGetUsageOfCol;
    GDALRATGetTypeOfCol:  TGDALRATGetTypeOfCol;
    GDALRATGetColOfUsage:  TGDALRATGetColOfUsage;
    GDALRATGetRowCount:  TGDALRATGetRowCount;
    GDALRATGetValueAsString:  TGDALRATGetValueAsString;
    GDALRATGetValueAsInt:  TGDALRATGetValueAsInt;
    GDALRATGetValueAsDouble:  TGDALRATGetValueAsDouble;
    GDALRATSetValueAsString:  TGDALRATSetValueAsString;
    GDALRATSetValueAsInt:  TGDALRATSetValueAsInt;
    GDALRATSetValueAsDouble:  TGDALRATSetValueAsDouble;
    GDALRATSetRowCount:  TGDALRATSetRowCount;
    GDALRATCreateColumn:  TGDALRATCreateColumn;
    GDALRATSetLinearBinning:  TGDALRATSetLinearBinning;
    GDALRATGetLinearBinning:  TGDALRATGetLinearBinning;
    GDALRATInitializeFromColorTable:  TGDALRATInitializeFromColorTable;
    GDALRATTranslateToColorTable:  TGDALRATTranslateToColorTable;
    GDALRATDumpReadable:  TGDALRATDumpReadable;
    GDALRATClone:  TGDALRATClone;
    GDALRATGetRowOfValue:  TGDALRATGetRowOfValue;


{$ELSE}
  function  GDALCreateRasterAttributeTable : TGDALRasterAttributeTableH; stdcall; external External_Lib name '_GDALCreateRasterAttributeTable@0';
  procedure GDALDestroyRasterAttributeTable(hRAT : TGDALRasterAttributeTableH ); stdcall; external External_Lib name '_GDALDestroyRasterAttributeTable@4';
  function  GDALRATGetColumnCount(hRAT : TGDALRasterAttributeTableH ) : longint; stdcall; external External_Lib name '_GDALRATGetColumnCount@4';
  function  GDALRATGetNameOfCol(hRAT : TGDALRasterAttributeTableH;
                                iCol : longint ) : PCHAR; stdcall; external External_Lib name '_GDALRATGetNameOfCol@8';
  function  GDALRATGetUsageOfCol(hRAT : TGDALRasterAttributeTableH;
                                iCol : longint) : TGDALRATFieldUsage; stdcall; external External_Lib name '_GDALRATGetUsageOfCol@8';
  function  GDALRATGetTypeOfCol(hRAT : TGDALRasterAttributeTableH;
                                iCol : longint) : TGDALRATFieldType; stdcall; external External_Lib name '_GDALRATGetTypeOfCol@8';
  function  GDALRATGetColOfUsage(hRAT : TGDALRasterAttributeTableH;
                                 eUsage : TGDALRATFieldUsage ) : longint; stdcall; external External_Lib name '_GDALRATGetColOfUsage@8';
  function  GDALRATGetRowCount( hRAT : TGDALRasterAttributeTableH) : longint; stdcall; external External_Lib name '_GDALRATGetRowCount@4';
  function  GDALRATGetValueAsString(hRAT : TGDALRasterAttributeTableH;
                                 iRow, iField : longint) : PCHAR; stdcall; external External_Lib name '_GDALRATGetValueAsString@12';
  function  GDALRATGetValueAsInt(hRAT : TGDALRasterAttributeTableH;
                                 iRow, iField, nValue : longint) : longint; stdcall; external External_Lib name '_GDALRATGetValueAsInt@12';
  function  GDALRATGetValueAsDouble(hRAT : TGDALRasterAttributeTableH;
                                 iRow, iField : longint) : double; stdcall; external External_Lib name '_GDALRATGetValueAsDouble@12';
  procedure GDALRATSetValueAsString(hRAT : TGDALRasterAttributeTableH;
                                    iRow, iField : longint;
                                    const pszValue : PCHAR); stdcall; external External_Lib name '_GDALRATSetValueAsString@16';
  procedure GDALRATSetValueAsInt(hRAT : TGDALRasterAttributeTableH;
                                 iRow, iField, nValue : longint); stdcall; external External_Lib name '_GDALRATSetValueAsInt@16';
  procedure GDALRATSetValueAsDouble(hRAT : TGDALRasterAttributeTableH;
                                    iRow, iField : longint;
                                    dfValue : double ); stdcall; external External_Lib name '_GDALRATSetValueAsDouble@20';
  procedure GDALRATSetRowCount(hRAT : TGDALRasterAttributeTableH;
                               nNewCount : longint); stdcall; external External_Lib name '_GDALRATSetRowCount@8';
  function  GDALRATCreateColumn(hRAT : TGDALRasterAttributeTableH;
            pszFieldName : PCHAR;
            eFieldType : TGDALRATFieldType;
            eFieldUsage : TGDALRATFieldUsage) : TCPLErr; stdcall; external External_Lib name '_GDALRATCreateColumn@16';
  function  GDALRATSetLinearBinning(hRAT : TGDALRasterAttributeTableH;
                                    dfRow0Min, dfBinSize : double ) : TCPLErr; stdcall; external External_Lib name '_GDALRATSetLinearBinning@20';
  function  GDALRATGetLinearBinning(hRAT : TGDALRasterAttributeTableH;
                                    pdfRow0Min, pdfBinSize :PDOUBLE) : longint; stdcall; external External_Lib name '_GDALRATGetLinearBinning@12';
  function  GDALRATInitializeFromColorTable(hRAT : TGDALRasterAttributeTableH;
                                            hCT : TGDALColorTableH ) : TCPLErr; stdcall; external External_Lib name '_GDALRATInitializeFromColorTable@8';
  function  GDALRATTranslateToColorTable(hRAT : TGDALRasterAttributeTableH;
                                         nEntryCount : longint) : TGDALColorTableH; stdcall; external External_Lib name '_GDALRATInitializeFromColorTable@8';
  procedure GDALRATDumpReadable(hRAT : TGDALRasterAttributeTableH; var fp : FILE ); stdcall; external External_Lib name '_GDALRATDumpReadable@8';
  function  GDALRATClone(hRAT : TGDALRasterAttributeTableH ) : TGDALRasterAttributeTableH; stdcall; external External_Lib name '_GDALRATClone@4';
  function  GDALRATGetRowOfValue(hRAT : TGDALRasterAttributeTableH;
                                 dfValue : double ) : longint; stdcall; external External_Lib name '_GDALRATGetRowOfValue@12';
{$ENDIF}

{$IFDEF DYNAMIC_GDAL}
var
   GDALDllHandle: THandle = 0;
function IsGDALLoaded: Boolean;
function GDALInit: Boolean;
procedure GDALDeinit;
{$ENDIF}
implementation

uses Windows, Ogr, ogrcore, cpl_csv;

{$IFDEF DYNAMIC_GDAL}
function IsGDALLoaded: Boolean;
begin
  Result := GDALDllHandle <> 0;
end;

function GDALInit: Boolean;
begin
  if not IsGDALLoaded then
  begin
   try
     GDALDllHandle := SafeLoadLibrary(GDALDLL);
     Result := IsGDALLoaded;
     if Result then
      begin
         @GDALGeneralCmdLineProcessor := GetProcAddress(GDALDllHandle, '_GDALGeneralCmdLineProcessor@12');
         @GDALSwapWords := GetProcAddress(GDALDllHandle, '_GDALSwapWords@16');
         @GDALCopyWords := GetProcAddress(GDALDllHandle, '_GDALCopyWords@28');
         @GDALCopyBits := GetProcAddress(GDALDllHandle, 'GDALCopyBits');
         @GDALLoadWorldFile := GetProcAddress(GDALDllHandle, '_GDALLoadWorldFile@8');
         @GDALReadWorldFile := GetProcAddress(GDALDllHandle, '_GDALReadWorldFile@12');
         @GDALWriteWorldFile := GetProcAddress(GDALDllHandle, '_GDALWriteWorldFile@12');
         @GDALPackedDMSToDec := GetProcAddress(GDALDllHandle, '_GDALPackedDMSToDec@8');
         @GDALDecToPackedDMS := GetProcAddress(GDALDllHandle, '_GDALDecToPackedDMS@8');
         @GDALVersionInfo := GetProcAddress(GDALDllHandle, '_GDALVersionInfo@4');
         @GDALCheckVersion := GetProcAddress(GDALDllHandle, '_GDALCheckVersion@12');
         @CPLError := GetProcAddress(GDALDllHandle, 'CPLError');
         @CPLGetLastErrorMsg := GetProcAddress(GDALDllHandle, '_CPLGetLastErrorMsg@0');
         @CSLDestroy := GetProcAddress(GDALDllHandle, '_CSLDestroy@4');
         @CPLGetConfigOption  := GetProcAddress(GDALDllHandle, '_CPLGetConfigOption@8');
         @CPLSetConfigOption  := GetProcAddress(GDALDllHandle, '_CPLSetConfigOption@8');
         @CPLAssert := GetProcAddress(GDALDllHandle, '__CPLAssert@12');
         @CPLDefaultErrorHandler := GetProcAddress(GDALDllHandle, '_CPLDefaultErrorHandler@12');
         @CPLMalloc := GetProcAddress(GDALDllHandle, 'CPLMalloc');
         @CPLErrorReset := GetProcAddress(GDALDllHandle, '_CPLErrorReset@0');
         @CPLGetLastErrorNo := GetProcAddress(GDALDllHandle, '_CPLGetLastErrorNo@0');
         @CPLGetLastErrorType  := GetProcAddress(GDALDllHandle, '_CPLGetLastErrorType@0');
         @CPLPopErrorHandler := GetProcAddress(GDALDllHandle, '_CPLPopErrorHandler@0');
         @CPLPushErrorHandler := GetProcAddress(GDALDllHandle, '_CPLPushErrorHandler@4');
         @CPLSetErrorHandler := GetProcAddress(GDALDllHandle, '_CPLSetErrorHandler@4');
         @GDALDummyProgress := GetProcAddress(GDALDllHandle, '_GDALDummyProgress@16');
         @GDALTermProgress := GetProcAddress(GDALDllHandle, '_GDALTermProgress@16');
         @GDALScaledProgress := GetProcAddress(GDALDllHandle, '_GDALScaledProgress@16');
         @GDALCreateScaledProgress := GetProcAddress(GDALDllHandle, '_GDALCreateScaledProgress@24');
         @GDALDestroyScaledProgress := GetProcAddress(GDALDllHandle, '_GDALDestroyScaledProgress@4');
         @GDALGetDataTypeName  := GetProcAddress(GDALDllHandle, '_GDALGetDataTypeName@4');
         @GDALGetDataTypeSize  := GetProcAddress(GDALDllHandle, '_GDALGetDataTypeSize@4');
         @GDALDataTypeIsComplex  := GetProcAddress(GDALDllHandle, '_GDALDataTypeIsComplex@4');
         @GDALDataTypeUnion := GetProcAddress(GDALDllHandle, '_GDALDataTypeUnion@8');
         @GDALGetColorInterpretationName := GetProcAddress(GDALDllHandle, 'GDALGetColorInterpretationName');
         @GDALGetPaletteInterpretationName := GetProcAddress(GDALDllHandle, 'GDALGetPaletteInterpretationName');
         @GDALGetMetadataItem  := GetProcAddress(GDALDllHandle, '_GDALGetMetadataItem@12');
         @GDALGetMetadata := GetProcAddress(GDALDllHandle, '_GDALGetMetadata@8');
         @GDALSetMetadataItem := GetProcAddress(GDALDllHandle, '_GDALSetMetadataItem@16');
         @GDALSetMetadata := GetProcAddress(GDALDllHandle, '_GDALSetMetadata@12');
         @GDALGetDescription := GetProcAddress(GDALDllHandle, '_GDALGetDescription@4');
         @GDALSetDescription := GetProcAddress(GDALDllHandle, '_GDALSetDescription@8');
         @GDALOpen := GetProcAddress(GDALDllHandle, '_GDALOpen@8');
         @GDALOpenShared := GetProcAddress(GDALDllHandle, '_GDALOpenShared@8');
         @GDALDumpOpenDatasets := GetProcAddress(GDALDllHandle, '_GDALDumpOpenDatasets@4');
         @GDALClose := GetProcAddress(GDALDllHandle, '_GDALClose@4');
         @GDALGetRasterXSize := GetProcAddress(GDALDllHandle, '_GDALGetRasterXSize@4');
         @GDALGetRasterYSize := GetProcAddress(GDALDllHandle, '_GDALGetRasterYSize@4');
         @GDALGetRasterCount := GetProcAddress(GDALDllHandle, '_GDALGetRasterCount@4');
         @GDALGetRasterBand := GetProcAddress(GDALDllHandle, '_GDALGetRasterBand@8');
         @GDALGetProjectionRef := GetProcAddress(GDALDllHandle, '_GDALGetProjectionRef@4');
         @GDALSetProjection := GetProcAddress(GDALDllHandle, '_GDALSetProjection@8');
         @GDALGetGeoTransform := GetProcAddress(GDALDllHandle, '_GDALGetGeoTransform@8');
         @GDALSetGeoTransform := GetProcAddress(GDALDllHandle, '_GDALSetGeoTransform@8');
         @GDALReferenceDataset := GetProcAddress(GDALDllHandle, '_GDALReferenceDataset@4');
         @GDALDereferenceDataset := GetProcAddress(GDALDllHandle, '_GDALSetGeoTransform@8');
         @GDALFlushCache := GetProcAddress(GDALDllHandle, '_GDALFlushCache@4');
         @GDALAddBand := GetProcAddress(GDALDllHandle, '_GDALAddBand@12');
         @GDALDatasetAdviseRead := GetProcAddress(GDALDllHandle, '_GDALDatasetAdviseRead@44');
         @GDALGetInternalHandle := GetProcAddress(GDALDllHandle, '_GDALGetInternalHandle@8');
         @GDALBuildOverviews := GetProcAddress(GDALDllHandle, '_GDALBuildOverviews@32');
         @GDALGetOpenDatasets := GetProcAddress(GDALDllHandle, '_GDALGetOpenDatasets@8');
         @GDALGetAccess := GetProcAddress(GDALDllHandle, '_GDALGetAccess@4');
         @GDALCreateDatasetMaskBand := GetProcAddress(GDALDllHandle, '_GDALCreateDatasetMaskBand@8');
         @GDALDatasetCopyWholeRaster := GetProcAddress(GDALDllHandle, '_GDALDatasetCopyWholeRaster@20');
         @GDALRegenerateOverviews := GetProcAddress(GDALDllHandle, 'GDALRegenerateOverviews');
         @GDALGetDatasetDriver := GetProcAddress(GDALDllHandle, '_GDALGetDatasetDriver@4');
         @GDALGetFileList := GetProcAddress(GDALDllHandle, '_GDALGetFileList@4');
         @GDALDatasetRasterIO := GetProcAddress(GDALDllHandle, '_GDALDatasetRasterIO@60');
         @GDALGetGCPCount := GetProcAddress(GDALDllHandle, '_GDALGetGCPCount@4');
         @GDALGetGCPProjection := GetProcAddress(GDALDllHandle, '_GDALGetGCPProjection@4');
         @GDALGetGCPs := GetProcAddress(GDALDllHandle, '_GDALGetGCPs@4');
         @GDALSetGCPs := GetProcAddress(GDALDllHandle, '_GDALSetGCPs@16');

         @GDALGetRasterDataType := GetProcAddress(GDALDllHandle, '_GDALGetRasterDataType@4');
         @GDALGetRasterBandXSize := GetProcAddress(GDALDllHandle, '_GDALGetRasterBandXSize@4');
         @GDALGetRasterBandYSize := GetProcAddress(GDALDllHandle, '_GDALGetRasterBandYSize@4');
         @GDALGetBlockSize := GetProcAddress(GDALDllHandle, '_GDALGetBlockSize@12');
         @GDALGetRasterNoDataValue := GetProcAddress(GDALDllHandle, '_GDALGetRasterNoDataValue@8');
         @GDALSetRasterNoDataValue := GetProcAddress(GDALDllHandle, '_GDALSetRasterNoDataValue@12');
         @GDALGetRasterMinimum := GetProcAddress(GDALDllHandle, '_GDALGetRasterMinimum@8');
         @GDALGetRasterMaximum := GetProcAddress(GDALDllHandle, '_GDALGetRasterMaximum@8');
         @GDALGetRasterOffset := GetProcAddress(GDALDllHandle, '_GDALGetRasterOffset@8');
         @GDALSetRasterOffset := GetProcAddress(GDALDllHandle, '_GDALSetRasterOffset@12');
         @GDALGetRasterScale := GetProcAddress(GDALDllHandle, '_GDALGetRasterScale@8');
         @GDALSetRasterScale := GetProcAddress(GDALDllHandle, '_GDALSetRasterScale@12');
         @GDALRasterIO := GetProcAddress(GDALDllHandle, '_GDALRasterIO@48');

         @GDALRasterAdviseRead := GetProcAddress(GDALDllHandle, '_GDALRasterAdviseRead@36');
         @GDALReadBlock := GetProcAddress(GDALDllHandle, '_GDALReadBlock@16');
         @GDALWriteBlock := GetProcAddress(GDALDllHandle, '_GDALWriteBlock@16');
         @GDALGetRasterAccess := GetProcAddress(GDALDllHandle, '_GDALGetRasterAccess@4');
         @GDALGetBandNumber := GetProcAddress(GDALDllHandle, '_GDALGetBandNumber@4');
         @GDALGetBandDataset := GetProcAddress(GDALDllHandle, '_GDALGetBandDataset@4');
         @GDALGetRasterColorInterpretation := GetProcAddress(GDALDllHandle, '_GDALGetRasterColorInterpretation@4');
         @GDALSetRasterColorInterpretation := GetProcAddress(GDALDllHandle, '_GDALSetRasterColorInterpretation@8');
         @GDALHasArbitraryOverviews := GetProcAddress(GDALDllHandle, '_GDALHasArbitraryOverviews@4');
         @GDALGetOverviewCount := GetProcAddress(GDALDllHandle, '_GDALGetOverviewCount@4');
         @GDALGetOverview := GetProcAddress(GDALDllHandle, '_GDALGetOverview@8');
         @GDALGetRasterCategoryNames := GetProcAddress(GDALDllHandle, '_GDALGetRasterCategoryNames@4');
         @GDALSetRasterCategoryNames := GetProcAddress(GDALDllHandle, '_GDALSetRasterCategoryNames@8');
         @GDALGetRasterStatistics := GetProcAddress(GDALDllHandle, '_GDALGetRasterStatistics@28');
         @GDALComputeRasterStatistics := GetProcAddress(GDALDllHandle, '_GDALComputeRasterStatistics@32');
         @GDALSetRasterStatistics := GetProcAddress(GDALDllHandle, '_GDALSetRasterStatistics@28');
         @GDALGetRasterUnitType := GetProcAddress(GDALDllHandle, '_GDALGetRasterUnitType@4');
         @GDALComputeRasterMinMax := GetProcAddress(GDALDllHandle, '_GDALComputeRasterMinMax@12');
         @GDALFlushRasterCache := GetProcAddress(GDALDllHandle, '_GDALFlushRasterCache@4');
         @GDALGetRasterHistogram := GetProcAddress(GDALDllHandle, '_GDALGetRasterHistogram@44');
         @GDALGetDefaultHistogram := GetProcAddress(GDALDllHandle, '_GDALGetDefaultHistogram@32');
         @GDALSetDefaultHistogram := GetProcAddress(GDALDllHandle, '_GDALSetDefaultHistogram@28');
         @GDALGetRasterSampleOverview := GetProcAddress(GDALDllHandle, '_GDALGetRasterSampleOverview@8');
         @GDALFillRaster := GetProcAddress(GDALDllHandle, '_GDALFillRaster@20');
         @GDALGetDefaultRAT := GetProcAddress(GDALDllHandle, '_GDALGetDefaultRAT@4');
         @GDALSetDefaultRAT := GetProcAddress(GDALDllHandle, '_GDALSetDefaultRAT@8');
         @GDALAddDerivedBandPixelFunc := GetProcAddress(GDALDllHandle, '_GDALAddDerivedBandPixelFunc@8');
         @GDALGetMaskBand := GetProcAddress(GDALDllHandle, '_GDALGetMaskBand@4');
         @GDALGetMaskFlags := GetProcAddress(GDALDllHandle, '_GDALGetMaskFlags@4');
         @GDALCreateMaskBand := GetProcAddress(GDALDllHandle, '_GDALCreateMaskBand@8');

         @GDALAllRegister := GetProcAddress(GDALDllHandle, '_GDALAllRegister@0');
         @GDALGetDriverByName := GetProcAddress(GDALDllHandle, '_GDALGetDriverByName@4');
         @GDALGetDriverCount := GetProcAddress(GDALDllHandle, '_GDALGetDriverCount@0');
         @GDALGetDriver := GetProcAddress(GDALDllHandle, '_GDALGetDriver@4');
         @GDALDestroyDriverManager := GetProcAddress(GDALDllHandle, '_GDALDestroyDriverManager@0');
         @GDALCreate := GetProcAddress(GDALDllHandle, '_GDALCreate@28');
         @GDALCreateCopy := GetProcAddress(GDALDllHandle, '_GDALCreateCopy@28');
         @GDALDestroyDriver := GetProcAddress(GDALDllHandle, '_GDALDestroyDriver@4');
         @GDALRegisterDriver := GetProcAddress(GDALDllHandle, '_GDALRegisterDriver@4');
         @GDALDeregisterDriver := GetProcAddress(GDALDllHandle, '_GDALDeregisterDriver@4');
         @GDALValidateCreationOptions := GetProcAddress(GDALDllHandle, '_GDALValidateCreationOptions@8');
         @GDALGetDriverShortName := GetProcAddress(GDALDllHandle, '_GDALGetDriverShortName@4');
         @GDALGetDriverLongName := GetProcAddress(GDALDllHandle, '_GDALGetDriverLongName@4');
         @GDALGetDriverHelpTopic := GetProcAddress(GDALDllHandle, '_GDALGetDriverHelpTopic@4');
         @GDALGetDriverCreationOptionList := GetProcAddress(GDALDllHandle, '_GDALGetDriverCreationOptionList@4');
         @GDALGCPsToGeoTransform := GetProcAddress(GDALDllHandle, '_GDALGCPsToGeoTransform@16');
         @GDALInvGeoTransform := GetProcAddress(GDALDllHandle, '_GDALInvGeoTransform@8');
         @GDALApplyGeoTransform := GetProcAddress(GDALDllHandle, '_GDALApplyGeoTransform@28');

         @GDALDeleteDataset := GetProcAddress(GDALDllHandle, '_GDALDeleteDataset@8');
         @GDALRenameDataset := GetProcAddress(GDALDllHandle, '_GDALRenameDataset@12');
         @GDALCopyDatasetFiles := GetProcAddress(GDALDllHandle, '_GDALCopyDatasetFiles@12');
         @GDALIdentifyDriver := GetProcAddress(GDALDllHandle, '_GDALIdentifyDriver@8');

         @GDALCreateColorTable := GetProcAddress(GDALDllHandle, '_GDALCreateColorTable@4');
         @GDALDestroyColorTable := GetProcAddress(GDALDllHandle, '_GDALDestroyColorTable@4');
         @GDALCloneColorTable := GetProcAddress(GDALDllHandle, '_GDALCloneColorTable@4');
         @GDALGetPaletteInterpretation := GetProcAddress(GDALDllHandle, '_GDALGetPaletteInterpretation@4');
         @GDALGetColorEntryCount := GetProcAddress(GDALDllHandle, '_GDALGetColorEntryCount@4');
         @GDALGetColorEntryAsRGB := GetProcAddress(GDALDllHandle, '_GDALGetColorEntryAsRGB@12');
         @GDALGetColorEntry := GetProcAddress(GDALDllHandle, '_GDALGetColorEntry@12');
         @GDALSetColorEntry := GetProcAddress(GDALDllHandle, '_GDALSetColorEntry@12');
         @GDALCreateColorRamp := GetProcAddress(GDALDllHandle, '_GDALCreateColorRamp@20');
         @GDALGetRasterColorTable := GetProcAddress(GDALDllHandle, '_GDALGetRasterColorTable@4');
         @GDALSetRasterColorTable := GetProcAddress(GDALDllHandle, '_GDALSetRasterColorTable@8');

         @GDALCreateRasterAttributeTable := GetProcAddress(GDALDllHandle, '_GDALCreateRasterAttributeTable@0');
         @GDALDestroyRasterAttributeTable := GetProcAddress(GDALDllHandle, '_GDALDestroyRasterAttributeTable@4');
         @GDALRATGetColumnCount := GetProcAddress(GDALDllHandle, '_GDALRATGetColumnCount@4');
         @GDALRATGetNameOfCol := GetProcAddress(GDALDllHandle, '_GDALRATGetNameOfCol@8');
         @GDALRATGetUsageOfCol := GetProcAddress(GDALDllHandle, '_GDALRATGetUsageOfCol@8');
         @GDALRATGetTypeOfCol := GetProcAddress(GDALDllHandle, '_GDALRATGetTypeOfCol@8');
         @GDALRATGetColOfUsage := GetProcAddress(GDALDllHandle, '_GDALRATGetColOfUsage@8');
         @GDALRATGetRowCount := GetProcAddress(GDALDllHandle, '_GDALRATGetRowCount@4');
         @GDALRATGetValueAsString := GetProcAddress(GDALDllHandle, '_GDALRATGetValueAsString@12');
         @GDALRATGetValueAsInt := GetProcAddress(GDALDllHandle, '_GDALRATGetValueAsInt@12');
         @GDALRATGetValueAsDouble := GetProcAddress(GDALDllHandle, '_GDALRATGetValueAsDouble@12');
         @GDALRATSetValueAsString := GetProcAddress(GDALDllHandle, '_GDALRATSetValueAsString@16');
         @GDALRATSetValueAsInt := GetProcAddress(GDALDllHandle, '_GDALRATSetValueAsInt@16');
         @GDALRATSetValueAsDouble := GetProcAddress(GDALDllHandle, '_GDALRATSetValueAsDouble@20');
         @GDALRATSetRowCount := GetProcAddress(GDALDllHandle, '_GDALRATSetRowCount@8');
         @GDALRATCreateColumn := GetProcAddress(GDALDllHandle, '_GDALRATCreateColumn@16');
         @GDALRATSetLinearBinning := GetProcAddress(GDALDllHandle, '_GDALRATSetLinearBinning@20');
         @GDALRATGetLinearBinning := GetProcAddress(GDALDllHandle, '_GDALRATGetLinearBinning@12');
         @GDALRATInitializeFromColorTable := GetProcAddress(GDALDllHandle, '_GDALRATInitializeFromColorTable@8');
         @GDALRATTranslateToColorTable := GetProcAddress(GDALDllHandle, '_GDALRATInitializeFromColorTable@8');
         @GDALRATDumpReadable := GetProcAddress(GDALDllHandle, '_GDALRATDumpReadable@8');
         @GDALRATClone := GetProcAddress(GDALDllHandle, '_GDALRATClone@4');
         @GDALRATGetRowOfValue := GetProcAddress(GDALDllHandle, '_GDALRATGetRowOfValue@12');

         @OGRGeometryTypeToName := GetProcAddress(GDALDllHandle, 'OGRGeometryTypeToName');
         @OGRMergeGeometryTypes := GetProcAddress(GDALDllHandle, 'OGRMergeGeometryTypes');

         @OGRParseDate := GetProcAddress(GDALDllHandle, 'OGRParseDate');

         @OSRAxisEnumToName := GetProcAddress(GDALDllHandle, 'OSRAxisEnumToName');
         @OGR_G_CreateFromWkb := GetProcAddress(GDALDllHandle, 'OGR_G_CreateFromWkb');
         @OGR_G_CreateFromWkt := GetProcAddress(GDALDllHandle, 'OGR_G_CreateFromWkt');
         @OGR_G_CreateFromFgf := GetProcAddress(GDALDllHandle, 'OGR_G_CreateFromFgf');
         @OGR_G_DestroyGeometry := GetProcAddress(GDALDllHandle, 'OGR_G_DestroyGeometry');

         @OGR_G_CreateGeometry := GetProcAddress(GDALDllHandle, 'OGR_G_CreateGeometry');
         @OGR_G_GetDimension := GetProcAddress(GDALDllHandle, 'OGR_G_GetDimension');
         @OGR_G_GetCoordinateDimension := GetProcAddress(GDALDllHandle, 'OGR_G_GetCoordinateDimension');
         @OGR_G_SetCoordinateDimension := GetProcAddress(GDALDllHandle, 'OGR_G_SetCoordinateDimension');
         @OGR_G_Clone := GetProcAddress(GDALDllHandle, 'OGR_G_Clone');
         @OGR_G_GetEnvelope := GetProcAddress(GDALDllHandle, 'OGR_G_GetEnvelope');
         @OGR_G_ImportFromWkb := GetProcAddress(GDALDllHandle, 'OGR_G_ImportFromWkb');
         @OGR_G_ExportToWkb := GetProcAddress(GDALDllHandle, 'OGR_G_ExportToWkb');
         @OGR_G_WkbSize := GetProcAddress(GDALDllHandle, 'OGR_G_WkbSize');
         @OGR_G_ImportFromWkt := GetProcAddress(GDALDllHandle, 'OGR_G_ImportFromWkt');
         @OGR_G_ExportToWkt := GetProcAddress(GDALDllHandle, 'OGR_G_ExportToWkt');
         @OGR_G_GetGeometryType := GetProcAddress(GDALDllHandle, 'OGR_G_GetGeometryType');
         @OGR_G_GetGeometryName := GetProcAddress(GDALDllHandle, 'OGR_G_GetGeometryName');
         @OGR_G_DumpReadable := GetProcAddress(GDALDllHandle, 'OGR_G_DumpReadable');
         @OGR_G_FlattenTo2D := GetProcAddress(GDALDllHandle, 'OGR_G_FlattenTo2D');
         @OGR_G_CloseRings := GetProcAddress(GDALDllHandle, 'OGR_G_CloseRings');

         @OGR_G_CreateFromGML := GetProcAddress(GDALDllHandle, 'OGR_G_CreateFromGML');
         @OGR_G_ExportToGML := GetProcAddress(GDALDllHandle, 'OGR_G_ExportToGML');

         @OGR_G_ExportToKML := GetProcAddress(GDALDllHandle, 'OGR_G_ExportToKML');

         @OGR_G_ExportToJson := GetProcAddress(GDALDllHandle, 'OGR_G_ExportToJson');
         @OGR_G_CreateGeometryFromJson := GetProcAddress(GDALDllHandle, 'OGR_G_CreateGeometryFromJson');
         @OGR_G_AssignSpatialReference := GetProcAddress(GDALDllHandle, 'OGR_G_AssignSpatialReference');
         @OGR_G_GetSpatialReference := GetProcAddress(GDALDllHandle, 'OGR_G_GetSpatialReference');
         @OGR_G_Transform := GetProcAddress(GDALDllHandle, 'OGR_G_Transform');
         @OGR_G_TransformTo := GetProcAddress(GDALDllHandle, 'OGR_G_TransformTo');
         @OGR_G_Segmentize := GetProcAddress(GDALDllHandle, 'OGR_G_Segmentize');
         @OGR_G_Intersects := GetProcAddress(GDALDllHandle, 'OGR_G_Intersects');
         @OGR_G_Equals := GetProcAddress(GDALDllHandle, 'OGR_G_Equals');
         @OGR_G_Disjoint := GetProcAddress(GDALDllHandle, 'OGR_G_Disjoint');
         @OGR_G_Touches := GetProcAddress(GDALDllHandle, 'OGR_G_Touches');
         @OGR_G_Crosses := GetProcAddress(GDALDllHandle, 'OGR_G_Crosses');
         @OGR_G_Within := GetProcAddress(GDALDllHandle, 'OGR_G_Within');
         @OGR_G_Contains := GetProcAddress(GDALDllHandle, 'OGR_G_Contains');
         @OGR_G_Overlaps := GetProcAddress(GDALDllHandle, 'OGR_G_Overlaps');
         @OGR_G_GetBoundary := GetProcAddress(GDALDllHandle, 'OGR_G_GetBoundary');
         @OGR_G_ConvexHull := GetProcAddress(GDALDllHandle, 'OGR_G_ConvexHull');
         @OGR_G_Buffer := GetProcAddress(GDALDllHandle, 'OGR_G_Buffer');
         @OGR_G_Intersection := GetProcAddress(GDALDllHandle, 'OGR_G_Intersection');
         @OGR_G_Union := GetProcAddress(GDALDllHandle, 'OGR_G_Union');
         @OGR_G_Difference := GetProcAddress(GDALDllHandle, 'OGR_G_Difference');
         @OGR_G_SymmetricDifference := GetProcAddress(GDALDllHandle, 'OGR_G_SymmetricDifference');
         @OGR_G_Distance := GetProcAddress(GDALDllHandle, 'OGR_G_Distance');
         @OGR_G_GetArea := GetProcAddress(GDALDllHandle, 'OGR_G_GetArea');
         @OGR_G_Centroid := GetProcAddress(GDALDllHandle, 'OGR_G_Centroid');
         @OGR_G_Empty := GetProcAddress(GDALDllHandle, 'OGR_G_Empty');
         @OGR_G_IsEmpty  := GetProcAddress(GDALDllHandle, 'OGR_G_IsEmpty');
         @OGR_G_IsValid  := GetProcAddress(GDALDllHandle, 'OGR_G_IsValid');
         @OGR_G_IsSimple   := GetProcAddress(GDALDllHandle, 'OGR_G_IsSimple');
         @OGR_G_IsRing   := GetProcAddress(GDALDllHandle, 'OGR_G_IsRing');

         @OGR_G_Intersect := GetProcAddress(GDALDllHandle, 'OGR_G_Intersect');
         @OGR_G_Equal := GetProcAddress(GDALDllHandle, 'OGR_G_Equal');
         @OGR_G_GetPointCount := GetProcAddress(GDALDllHandle, 'OGR_G_GetPointCount');
         @OGR_G_GetX := GetProcAddress(GDALDllHandle, 'OGR_G_GetX');
         @OGR_G_GetY := GetProcAddress(GDALDllHandle, 'OGR_G_GetY');
         @OGR_G_GetZ := GetProcAddress(GDALDllHandle, 'OGR_G_GetZ');
         @OGR_G_GetPoint := GetProcAddress(GDALDllHandle, 'OGR_G_GetPoint');
         @OGR_G_SetPoint := GetProcAddress(GDALDllHandle, 'OGR_G_SetPoint');
         @OGR_G_SetPoint_2D := GetProcAddress(GDALDllHandle, 'OGR_G_SetPoint_2D');
         @OGR_G_AddPoint := GetProcAddress(GDALDllHandle, 'OGR_G_AddPoint');
         @OGR_G_AddPoint_2D := GetProcAddress(GDALDllHandle, 'OGR_G_AddPoint_2D');


         @OGR_G_GetGeometryCount := GetProcAddress(GDALDllHandle, 'OGR_G_GetGeometryCount');
         @OGR_G_GetGeometryRef := GetProcAddress(GDALDllHandle, 'OGR_G_GetGeometryRef');
         @OGR_G_AddGeometry := GetProcAddress(GDALDllHandle, 'OGR_G_AddGeometry');
         @OGR_G_AddGeometryDirectly := GetProcAddress(GDALDllHandle, 'OGR_G_AddGeometryDirectly');
         @OGR_G_RemoveGeometry := GetProcAddress(GDALDllHandle, 'OGR_G_RemoveGeometry');
         @OGRBuildPolygonFromEdges := GetProcAddress(GDALDllHandle, 'OGRBuildPolygonFromEdges');
         @OGRSetGenerate_DB2_V72_BYTE_ORDER := GetProcAddress(GDALDllHandle, 'OGRSetGenerate_DB2_V72_BYTE_ORDER');
         @OGRGetGenerate_DB2_V72_BYTE_ORDER := GetProcAddress(GDALDllHandle, 'OGRGetGenerate_DB2_V72_BYTE_ORDER');

         @OGRFree := GetProcAddress(GDALDllHandle, 'OGRFree');
         @OGR_Fld_Create := GetProcAddress(GDALDllHandle, 'OGR_Fld_Create');
         @OGR_Fld_Destroy := GetProcAddress(GDALDllHandle, 'OGR_Fld_Destroy');
         @OGR_Fld_SetName := GetProcAddress(GDALDllHandle, 'OGR_Fld_SetName');
         @OGR_Fld_GetNameRef := GetProcAddress(GDALDllHandle, 'OGR_Fld_GetNameRef');
         @OGR_Fld_GetType := GetProcAddress(GDALDllHandle, 'OGR_Fld_GetType');
         @OGR_Fld_SetType := GetProcAddress(GDALDllHandle, 'OGR_Fld_SetType');
         @OGR_Fld_GetJustify := GetProcAddress(GDALDllHandle, 'OGR_Fld_GetJustify');
         @OGR_Fld_SetJustify := GetProcAddress(GDALDllHandle, 'OGR_Fld_SetJustify');
         @OGR_Fld_GetWidth := GetProcAddress(GDALDllHandle, 'OGR_Fld_GetWidth');
         @OGR_Fld_SetWidth := GetProcAddress(GDALDllHandle, 'OGR_Fld_SetWidth');
         @OGR_Fld_GetPrecision := GetProcAddress(GDALDllHandle, 'OGR_Fld_GetPrecision');
         @OGR_Fld_SetPrecision := GetProcAddress(GDALDllHandle, 'OGR_Fld_SetPrecision');
         @OGR_Fld_Set := GetProcAddress(GDALDllHandle, 'OGR_Fld_Set');

         @OGR_GetFieldTypeName := GetProcAddress(GDALDllHandle, 'OGR_GetFieldTypeName');

         @OGR_F_Create := GetProcAddress(GDALDllHandle, 'OGR_F_Create');
         @OGR_F_Destroy := GetProcAddress(GDALDllHandle, 'OGR_F_Destroy');
         @OGR_F_GetDefnRef := GetProcAddress(GDALDllHandle, 'OGR_F_GetDefnRef');
         @OGR_F_SetGeometryDirectly := GetProcAddress(GDALDllHandle, 'OGR_F_SetGeometryDirectly');
         @OGR_F_SetGeometry := GetProcAddress(GDALDllHandle, 'OGR_F_SetGeometry');
         @OGR_F_GetGeometryRef := GetProcAddress(GDALDllHandle, 'OGR_F_GetGeometryRef');
         @OGR_F_Clone := GetProcAddress(GDALDllHandle, 'OGR_F_Clone');
         @OGR_F_Equal := GetProcAddress(GDALDllHandle, 'OGR_F_Equal');
         @OGR_F_GetFieldCount := GetProcAddress(GDALDllHandle, 'OGR_F_GetFieldCount');
         @OGR_F_GetFieldDefnRef := GetProcAddress(GDALDllHandle, 'OGR_F_GetFieldDefnRef');
         @OGR_F_GetFieldIndex := GetProcAddress(GDALDllHandle, 'OGR_F_GetFieldIndex');
         @OGR_F_IsFieldSet := GetProcAddress(GDALDllHandle, 'OGR_F_IsFieldSet');
         @OGR_F_UnsetField := GetProcAddress(GDALDllHandle, 'OGR_F_UnsetField');
         @OGR_F_GetRawFieldRef := GetProcAddress(GDALDllHandle, 'OGR_F_GetRawFieldRef');
         @OGR_F_GetFieldAsInteger := GetProcAddress(GDALDllHandle, 'OGR_F_GetFieldAsInteger');
         @OGR_F_GetFieldAsDouble := GetProcAddress(GDALDllHandle, 'OGR_F_GetFieldAsDouble');
         @OGR_F_GetFieldAsString := GetProcAddress(GDALDllHandle, 'OGR_F_GetFieldAsString');
         @OGR_F_GetFieldAsIntegerList := GetProcAddress(GDALDllHandle, 'OGR_F_GetFieldAsIntegerList');
         @OGR_F_GetFieldAsDoubleList := GetProcAddress(GDALDllHandle, 'OGR_F_GetFieldAsDoubleList');
         @OGR_F_GetFieldAsStringList := GetProcAddress(GDALDllHandle, 'OGR_F_GetFieldAsStringList');
         @OGR_F_GetFieldAsBinary := GetProcAddress(GDALDllHandle, 'OGR_F_GetFieldAsBinary');
         @OGR_F_GetFieldAsDateTime := GetProcAddress(GDALDllHandle, 'OGR_F_GetFieldAsDateTime');
         @OGR_F_SetFieldInteger := GetProcAddress(GDALDllHandle, 'OGR_F_SetFieldInteger');
         @OGR_F_SetFieldDouble := GetProcAddress(GDALDllHandle, 'OGR_F_SetFieldDouble');
         @OGR_F_SetFieldString := GetProcAddress(GDALDllHandle, 'OGR_F_SetFieldString');
         @OGR_F_SetFieldIntegerList := GetProcAddress(GDALDllHandle, 'OGR_F_SetFieldIntegerList');
         @OGR_F_SetFieldDoubleList := GetProcAddress(GDALDllHandle, 'OGR_F_SetFieldDoubleList');
         @OGR_F_SetFieldStringList := GetProcAddress(GDALDllHandle, 'OGR_F_SetFieldStringList');
         @OGR_F_SetFieldRaw := GetProcAddress(GDALDllHandle, 'OGR_F_SetFieldRaw');
         @OGR_F_SetFieldBinary := GetProcAddress(GDALDllHandle, 'OGR_F_SetFieldBinary');
         @OGR_F_SetFieldDateTime := GetProcAddress(GDALDllHandle, 'OGR_F_SetFieldDateTime');
         @OGR_F_GetFID := GetProcAddress(GDALDllHandle, 'OGR_F_GetFID');
         @OGR_F_SetFID := GetProcAddress(GDALDllHandle, 'OGR_F_SetFID');
         @OGR_F_DumpReadable := GetProcAddress(GDALDllHandle, 'OGR_F_DumpReadable');
         @OGR_F_SetFrom := GetProcAddress(GDALDllHandle, 'OGR_F_SetFrom');
         @OGR_F_GetStyleString := GetProcAddress(GDALDllHandle, 'OGR_F_GetStyleString');
         @OGR_F_SetStyleString := GetProcAddress(GDALDllHandle, 'OGR_F_SetStyleString');
         @OGR_F_SetStyleStringDirectly := GetProcAddress(GDALDllHandle, 'OGR_F_SetStyleStringDirectly');

         @OGR_L_GetSpatialFilter := GetProcAddress(GDALDllHandle, 'OGR_L_GetSpatialFilter');
         @OGR_L_SetSpatialFilter := GetProcAddress(GDALDllHandle, 'OGR_L_SetSpatialFilter');
         @OGR_L_SetSpatialFilterRect := GetProcAddress(GDALDllHandle, 'OGR_L_SetSpatialFilterRect');
         @OGR_L_SetAttributeFilter := GetProcAddress(GDALDllHandle, 'OGR_L_SetAttributeFilter');
         @OGR_L_ResetReading := GetProcAddress(GDALDllHandle, 'OGR_L_ResetReading');
         @OGR_L_GetNextFeature := GetProcAddress(GDALDllHandle, 'OGR_L_GetNextFeature');
         @OGR_L_SetNextByIndex := GetProcAddress(GDALDllHandle, 'OGR_L_SetNextByIndex');
         @OGR_L_GetFeature := GetProcAddress(GDALDllHandle, 'OGR_L_GetFeature');
         @OGR_L_SetFeature := GetProcAddress(GDALDllHandle, 'OGR_L_SetFeature');
         @OGR_L_CreateFeature := GetProcAddress(GDALDllHandle, 'OGR_L_CreateFeature');
         @OGR_L_DeleteFeature := GetProcAddress(GDALDllHandle, 'OGR_L_DeleteFeature');
         @OGR_L_GetLayerDefn := GetProcAddress(GDALDllHandle, 'OGR_L_GetLayerDefn');
         @OGR_L_GetSpatialRef := GetProcAddress(GDALDllHandle, 'OGR_L_GetSpatialRef');
         @OGR_L_GetFeatureCount := GetProcAddress(GDALDllHandle, 'OGR_L_GetFeatureCount');
         @OGR_L_GetExtent := GetProcAddress(GDALDllHandle, 'OGR_L_GetExtent');
         @OGR_L_TestCapability := GetProcAddress(GDALDllHandle, 'OGR_L_TestCapability');
         @OGR_L_CreateField := GetProcAddress(GDALDllHandle, 'OGR_L_CreateField');
         @OGR_L_StartTransaction := GetProcAddress(GDALDllHandle, 'OGR_L_StartTransaction');
         @OGR_L_CommitTransaction := GetProcAddress(GDALDllHandle, 'OGR_L_CommitTransaction');
         @OGR_L_RollbackTransaction := GetProcAddress(GDALDllHandle, 'OGR_L_RollbackTransaction');
         @OGR_L_Reference := GetProcAddress(GDALDllHandle, 'OGR_L_Reference');
         @OGR_L_Dereference := GetProcAddress(GDALDllHandle, 'OGR_L_Dereference');
         @OGR_L_GetRefCount := GetProcAddress(GDALDllHandle, 'OGR_L_GetRefCount');
         @OGR_L_SyncToDisk := GetProcAddress(GDALDllHandle, 'OGR_L_SyncToDisk');
         @OGR_L_GetFeaturesRead := GetProcAddress(GDALDllHandle, 'OGR_L_GetFeaturesRead');
         @OGR_L_GetFIDColumn := GetProcAddress(GDALDllHandle, 'OGR_L_GetFIDColumn');
         @OGR_L_GetGeometryColumn := GetProcAddress(GDALDllHandle, 'OGR_L_GetGeometryColumn');

         @OGR_DS_Destroy := GetProcAddress(GDALDllHandle, 'OGR_DS_Destroy');
         @OGR_DS_GetName := GetProcAddress(GDALDllHandle, 'OGR_DS_GetName');
         @OGR_DS_GetLayerCount := GetProcAddress(GDALDllHandle, 'OGR_DS_GetLayerCount');
         @OGR_DS_GetLayer := GetProcAddress(GDALDllHandle, 'OGR_DS_GetLayer');
         @OGR_DS_GetLayerByName := GetProcAddress(GDALDllHandle, 'OGR_DS_GetLayerByName');
         @OGR_DS_DeleteLayer := GetProcAddress(GDALDllHandle, 'OGR_DS_DeleteLayer');
         @OGR_DS_GetDriver := GetProcAddress(GDALDllHandle, 'OGR_DS_GetDriver');
         @OGR_DS_CreateLayer := GetProcAddress(GDALDllHandle, 'OGR_DS_CreateLayer');
         @OGR_DS_CopyLayer := GetProcAddress(GDALDllHandle, 'OGR_DS_CopyLayer');

         @OGR_DS_TestCapability := GetProcAddress(GDALDllHandle, 'OGR_DS_TestCapability');
         @OGR_DS_ExecuteSQL := GetProcAddress(GDALDllHandle, 'OGR_DS_ExecuteSQL');
         @OGR_DS_ReleaseResultSet := GetProcAddress(GDALDllHandle, 'OGR_DS_ReleaseResultSet');
         @OGR_DS_Reference := GetProcAddress(GDALDllHandle, 'OGR_DS_Reference');
         @OGR_DS_Dereference := GetProcAddress(GDALDllHandle, 'OGR_DS_Dereference');
         @OGR_DS_GetRefCount := GetProcAddress(GDALDllHandle, 'OGR_DS_GetRefCount');
         @OGR_DS_GetSummaryRefCount := GetProcAddress(GDALDllHandle, 'OGR_DS_GetSummaryRefCount');
         @OGR_DS_SyncToDisk := GetProcAddress(GDALDllHandle, 'OGR_DS_SyncToDisk');
         @OGR_Dr_GetName := GetProcAddress(GDALDllHandle, 'OGR_Dr_GetName');
         @OGR_Dr_Open := GetProcAddress(GDALDllHandle, 'OGR_Dr_Open');
         @OGR_Dr_TestCapability := GetProcAddress(GDALDllHandle, 'OGR_Dr_TestCapability');
         @OGR_Dr_CreateDataSource := GetProcAddress(GDALDllHandle, 'OGR_Dr_CreateDataSource');
         @OGR_Dr_CopyDataSource := GetProcAddress(GDALDllHandle, 'OGR_Dr_CopyDataSource');
         @OGR_Dr_DeleteDataSource := GetProcAddress(GDALDllHandle, 'OGR_Dr_DeleteDataSource');


         @OGROpen := GetProcAddress(GDALDllHandle, 'OGROpen');
         @OGROpenShared := GetProcAddress(GDALDllHandle, 'OGROpenShared');
         @OGRReleaseDataSource := GetProcAddress(GDALDllHandle, 'OGRReleaseDataSource');
         @OGRRegisterDriver := GetProcAddress(GDALDllHandle, 'OGRRegisterDriver');
         @OGRGetDriverCount := GetProcAddress(GDALDllHandle, 'OGRGetDriverCount');
         @OGRGetDriver := GetProcAddress(GDALDllHandle, 'OGRGetDriver');
         @OGRGetDriverByName := GetProcAddress(GDALDllHandle, 'OGRGetDriverByName');
         @OGRGetOpenDSCount := GetProcAddress(GDALDllHandle, 'OGRGetOpenDSCount');
         @OGRGetOpenDS := GetProcAddress(GDALDllHandle, 'OGRGetOpenDS');

         @OGRRegisterAll := GetProcAddress(GDALDllHandle, 'OGRRegisterAll');
         @OGRCleanupAll := GetProcAddress(GDALDllHandle, 'OGRCleanupAll');




         @OSRNewSpatialReference := GetProcAddress(GDALDllHandle, '_OSRNewSpatialReference@4');
         @OSRCloneGeogCS := GetProcAddress(GDALDllHandle, '_OSRCloneGeogCS@4');

         @OSRClone := GetProcAddress(GDALDllHandle, '_OSRClone@4');
         @OSRDestroySpatialReference  := GetProcAddress(GDALDllHandle, '_OSRDestroySpatialReference@4');

         @OSRReference := GetProcAddress(GDALDllHandle, 'OSRReference');
         @OSRDereference := GetProcAddress(GDALDllHandle, 'OSRDereference');
         @OSRRelease := GetProcAddress(GDALDllHandle, 'OSRRelease');
         @OSRValidate := GetProcAddress(GDALDllHandle, 'OSRValidate');
         @OSRFixupOrdering := GetProcAddress(GDALDllHandle, 'OSRFixupOrdering');
         @OSRFixup := GetProcAddress(GDALDllHandle, 'OSRFixup');
         @OSRStripCTParms := GetProcAddress(GDALDllHandle, 'OSRStripCTParms');
         @OSRImportFromEPSG := GetProcAddress(GDALDllHandle, '_OSRImportFromEPSG@8');
         @OSRImportFromEPSGA := GetProcAddress(GDALDllHandle, '_OSRImportFromEPSGA@8');
         @OSRImportFromWkt := GetProcAddress(GDALDllHandle, 'OSRImportFromWkt');
         @OSRImportFromProj4 := GetProcAddress(GDALDllHandle, 'OSRImportFromProj4');
         @OSRImportFromESRI := GetProcAddress(GDALDllHandle, 'OSRImportFromESRI');
         @OSRImportFromPCI := GetProcAddress(GDALDllHandle, 'OSRImportFromPCI');
         @OSRImportFromUSGS := GetProcAddress(GDALDllHandle, 'OSRImportFromUSGS');
         @OSRImportFromXML := GetProcAddress(GDALDllHandle, 'OSRImportFromXML');

         @OSRImportFromDict := GetProcAddress(GDALDllHandle, 'OSRImportFromDict');

         @OSRImportFromPanorama := GetProcAddress(GDALDllHandle, 'OSRImportFromPanorama');

         @OSRImportFromMICoordSys := GetProcAddress(GDALDllHandle, 'OSRImportFromMICoordSys');
         @OSRImportFromUrl := GetProcAddress(GDALDllHandle, 'OSRImportFromUrl');

         @OSRExportToWkt := GetProcAddress(GDALDllHandle, '_OSRExportToWkt@8');
         @OSRExportToPrettyWkt := GetProcAddress(GDALDllHandle, '_OSRExportToPrettyWkt@12');

         @OSRExportToProj4 := GetProcAddress(GDALDllHandle, '_OSRExportToProj4@8');
         @OSRExportToPCI := GetProcAddress(GDALDllHandle, 'OSRExportToPCI');
         @OSRExportToUSGS := GetProcAddress(GDALDllHandle, 'OSRExportToUSGS');
         @OSRExportToXML := GetProcAddress(GDALDllHandle, 'OSRExportToXML');
         @OSRExportToPanorama := GetProcAddress(GDALDllHandle, 'OSRExportToPanorama');

         @OSRExportToMICoordSys := GetProcAddress(GDALDllHandle, 'OSRExportToMICoordSys');

         @OSRMorphToESRI := GetProcAddress(GDALDllHandle, 'OSRMorphToESRI');
         @OSRMorphFromESRI := GetProcAddress(GDALDllHandle, 'OSRMorphFromESRI');
         @OSRSetAttrValue := GetProcAddress(GDALDllHandle, '_OSRSetAttrValue@12');
         @OSRGetAttrValue := GetProcAddress(GDALDllHandle, '_OSRGetAttrValue@12');
         @OSRSetAngularUnits := GetProcAddress(GDALDllHandle, 'OSRSetAngularUnits');
         @OSRGetAngularUnits := GetProcAddress(GDALDllHandle, 'OSRGetAngularUnits');
         @OSRSetLinearUnits := GetProcAddress(GDALDllHandle, 'OSRSetLinearUnits');
         @OSRSetLinearUnitsAndUpdateParameters := GetProcAddress(GDALDllHandle, 'OSRSetLinearUnitsAndUpdateParameters');
         @OSRGetLinearUnits := GetProcAddress(GDALDllHandle, 'OSRGetLinearUnits');
         @OSRGetPrimeMeridian := GetProcAddress(GDALDllHandle, 'OSRGetPrimeMeridian');
         @OSRIsGeographic := GetProcAddress(GDALDllHandle, 'OSRIsGeographic');
         @OSRIsLocal := GetProcAddress(GDALDllHandle, 'OSRIsLocal');
         @OSRIsProjected := GetProcAddress(GDALDllHandle, 'OSRIsProjected');
         @OSRIsSameGeogCS := GetProcAddress(GDALDllHandle, 'OSRIsSameGeogCS');
         @OSRIsSame := GetProcAddress(GDALDllHandle, 'OSRIsSame');
         @OSRSetLocalCS := GetProcAddress(GDALDllHandle, 'OSRSetLocalCS');
         @OSRSetProjCS := GetProcAddress(GDALDllHandle, 'OSRSetProjCS');
         @OSRSetWellKnownGeogCS := GetProcAddress(GDALDllHandle, 'OSRSetWellKnownGeogCS');
         @OSRSetFromUserInput := GetProcAddress(GDALDllHandle, '_OSRSetFromUserInput@8');
         @OSRCopyGeogCSFrom := GetProcAddress(GDALDllHandle, 'OSRCopyGeogCSFrom');
         @OSRSetTOWGS84 := GetProcAddress(GDALDllHandle, 'OSRSetTOWGS84');
         @OSRGetTOWGS84 := GetProcAddress(GDALDllHandle, 'OSRGetTOWGS84');
         @OSRSetGeogCS := GetProcAddress(GDALDllHandle, 'OSRSetGeogCS');

         @OSRGetSemiMajor := GetProcAddress(GDALDllHandle, 'OSRGetSemiMajor');
         @OSRGetSemiMinor := GetProcAddress(GDALDllHandle, 'OSRGetSemiMinor');
         @OSRGetInvFlattening := GetProcAddress(GDALDllHandle, 'OSRGetInvFlattening');
         @OSRSetAuthority := GetProcAddress(GDALDllHandle, 'OSRSetAuthority');
         @OSRGetAuthorityCode := GetProcAddress(GDALDllHandle, 'OSRGetAuthorityCode');
         @OSRGetAuthorityName := GetProcAddress(GDALDllHandle, 'OSRGetAuthorityName');
         @OSRSetProjection := GetProcAddress(GDALDllHandle, 'OSRSetProjection');
         @OSRSetProjParm := GetProcAddress(GDALDllHandle, 'OSRSetProjParm');
         @OSRGetProjParm := GetProcAddress(GDALDllHandle, 'OSRGetProjParm');
         @OSRSetNormProjParm := GetProcAddress(GDALDllHandle, 'OSRSetNormProjParm');
         @OSRGetNormProjParm := GetProcAddress(GDALDllHandle, 'OSRGetNormProjParm');
         @OSRSetUTM := GetProcAddress(GDALDllHandle, 'OSRSetUTM');
         @OSRGetUTMZone := GetProcAddress(GDALDllHandle, 'OSRGetUTMZone');
         @OSRSetStatePlane := GetProcAddress(GDALDllHandle, 'OSRSetStatePlane');
         @OSRSetStatePlaneWithUnits := GetProcAddress(GDALDllHandle, 'OSRSetStatePlaneWithUnits');
         @OSRAutoIdentifyEPSG := GetProcAddress(GDALDllHandle, 'OSRAutoIdentifyEPSG');
         @OSRGetAxis := GetProcAddress(GDALDllHandle, 'OSRGetAxis');
         @OSRSetACEA := GetProcAddress(GDALDllHandle, 'OSRSetACEA');
         @OSRSetAE := GetProcAddress(GDALDllHandle, 'OSRSetACEA');
         @OSRSetBonne := GetProcAddress(GDALDllHandle, 'OSRSetBonne');
         @OSRSetCEA := GetProcAddress(GDALDllHandle, 'OSRSetCEA');
         @OSRSetCS := GetProcAddress(GDALDllHandle, 'OSRSetCS');
         @OSRSetEC := GetProcAddress(GDALDllHandle, 'OSRSetEC');
         @OSRSetEckert := GetProcAddress(GDALDllHandle, 'OSRSetEckert');
         @OSRSetEckertIV := GetProcAddress(GDALDllHandle, 'OSRSetEckertIV');
         @OSRSetEckertVI := GetProcAddress(GDALDllHandle, 'OSRSetEckertVI');
         @OSRSetEquirectangular := GetProcAddress(GDALDllHandle, 'OSRSetEquirectangular');
         @OSRSetEquirectangular2 := GetProcAddress(GDALDllHandle, 'OSRSetEquirectangular2');
         @OSRSetGS := GetProcAddress(GDALDllHandle, 'OSRSetGS');
         @OSRSetGH := GetProcAddress(GDALDllHandle, 'OSRSetGH');
         @OSRSetGEOS := GetProcAddress(GDALDllHandle, 'OSRSetGEOS');
         @OSRSetGaussSchreiberTMercator := GetProcAddress(GDALDllHandle, 'OSRSetGaussSchreiberTMercator');
         @OSRSetGnomonic := GetProcAddress(GDALDllHandle, 'OSRSetGnomonic');
         @OSRSetHOM := GetProcAddress(GDALDllHandle, 'OSRSetHOM');
         @OSRSetHOM2PNO := GetProcAddress(GDALDllHandle, 'OSRSetHOM2PNO');
         @OSRSetIWMPolyconic := GetProcAddress(GDALDllHandle, 'OSRSetIWMPolyconic');
         @OSRSetKrovak := GetProcAddress(GDALDllHandle, 'OSRSetKrovak');
         @OSRSetLAEA := GetProcAddress(GDALDllHandle, 'OSRSetLAEA');
         @OSRSetLCC := GetProcAddress(GDALDllHandle, 'OSRSetLCC');
         @OSRSetLCC1SP := GetProcAddress(GDALDllHandle, 'OSRSetLCC1SP');
         @OSRSetLCCB := GetProcAddress(GDALDllHandle, 'OSRSetLCCB');
         @OSRSetMC := GetProcAddress(GDALDllHandle, 'OSRSetMC');
         @OSRSetMercator := GetProcAddress(GDALDllHandle, 'OSRSetMercator');
         @OSRSetMollweide := GetProcAddress(GDALDllHandle, 'OSRSetMollweide');
         @OSRSetNZMG := GetProcAddress(GDALDllHandle, 'OSRSetNZMG');
         @OSRSetOS := GetProcAddress(GDALDllHandle, 'OSRSetOS');
         @OSRSetOrthographic := GetProcAddress(GDALDllHandle, 'OSRSetOrthographic');
         @OSRSetPolyconic := GetProcAddress(GDALDllHandle, 'OSRSetPolyconic');
         @OSRSetPS := GetProcAddress(GDALDllHandle, 'OSRSetPS');
         @OSRSetRobinson := GetProcAddress(GDALDllHandle, 'OSRSetRobinson');
         @OSRSetSinusoidal := GetProcAddress(GDALDllHandle, 'OSRSetSinusoidal');
         @OSRSetStereographic := GetProcAddress(GDALDllHandle, 'OSRSetStereographic');
         @OSRSetSOC := GetProcAddress(GDALDllHandle, 'OSRSetSOC');
         @OSRSetTM := GetProcAddress(GDALDllHandle, 'OSRSetTM');
         @OSRSetTMVariant := GetProcAddress(GDALDllHandle, 'OSRSetTMVariant');
         @OSRSetTMG := GetProcAddress(GDALDllHandle, 'OSRSetTMG');
         @OSRSetTMSO := GetProcAddress(GDALDllHandle, 'OSRSetTMSO');
         @OSRSetVDG := GetProcAddress(GDALDllHandle, 'OSRSetVDG');
         @OSRSetWagner := GetProcAddress(GDALDllHandle, 'OSRSetWagner');
         @OSRCleanup := GetProcAddress(GDALDllHandle, 'OSRCleanup');

         @OPTGetProjectionMethods := GetProcAddress(GDALDllHandle, 'OPTGetProjectionMethods');
         @OPTGetParameterList := GetProcAddress(GDALDllHandle, 'OPTGetParameterList');
         @OPTGetParameterInfo := GetProcAddress(GDALDllHandle, 'OPTGetParameterInfo');

         @OCTNewCoordinateTransformation  := GetProcAddress(GDALDllHandle, '_OCTNewCoordinateTransformation@8');
         @OCTDestroyCoordinateTransformation  := GetProcAddress(GDALDllHandle, '_OCTDestroyCoordinateTransformation@4');
         @OCTTransformEx  := GetProcAddress(GDALDllHandle, '_OCTTransformEx@24');
         @OCTTransform  := GetProcAddress(GDALDllHandle, '_OCTTransform@20');

         @CSVFileName  := GetProcAddress(GDALDllHandle, 'CSVFilename');
         @CSVDetectSeperator  := GetProcAddress(GDALDllHandle, 'CSVDetectSeperator');
         @CSVReadParseLine  := GetProcAddress(GDALDllHandle, 'CSVReadParseLine');
         @CSVReadParseLine2  := GetProcAddress(GDALDllHandle, 'CSVReadParseLine2');
         @CSVScanLines  := GetProcAddress(GDALDllHandle, 'CSVScanLines');
         @CSVScanFile  := GetProcAddress(GDALDllHandle, 'CSVScanFile');

         @CSVScanFileByName  := GetProcAddress(GDALDllHandle, 'CSVScanFileByName');
         @CSVGetNextLine  := GetProcAddress(GDALDllHandle, 'CSVGetNextLine');
         @CSVGetFieldId  := GetProcAddress(GDALDllHandle, 'CSVGetFieldId');
         @CSVGetFileFieldId  := GetProcAddress(GDALDllHandle, 'CSVGetFileFieldId');
         @CSVDeaccess  := GetProcAddress(GDALDllHandle, 'CSVDeaccess');
         @CSVGetField  := GetProcAddress(GDALDllHandle, 'CSVGetField');

      end;
   except
     GDALDllHandle := 0;
     Result := False;
   end;
  end
   else
    Result := True;
end;

procedure GDALDeinit;
begin
  if GDALDllHandle <> 0 then
  begin
   FreeLibrary(GDALDllHandle);
   GDALDllHandle := 0;
  end;
end;
{$ENDIF}



{
function SrcVal(papoSource : PPOINTER;
		eSrcType : TGDALDatatype;
		ii : longint) : variant;

var
  val :variant;
begin
  case eSrcType of
    GDT_Byte    : val.vbyte := GByte(papoSource[ii]);
    GDT_Float32 : val.vsingle := GFloat32(papoSource[ii]);
    GDT_Float64 : val.vdouble := GFloat64(papoSource[ii]);
    GDT_Int32   : val.vinteger := GInt32(papoSource[ii]);
    GDT_UInt16  : val.vword := GUInt16(papoSource[ii]);
    GDT_Int16   : val.vsmallint := GInt16(papoSource[ii]);
    GDT_UInt32  : val.vlongword := GUInt32(papoSource[ii]);
    GDT_CInt16  : val.vword := GInt16(papoSource[ii * 2]);
    GDT_CInt32  : val.vlongword := GCint32(papoSource[ii * 2]);
    GDT_CFloat32: val.vsingle := GCFloat32(papoSource[ii * 2]);
    GDT_CFloat64: val.vdouble := GCFloat64(papoSource[ii * 2]);
  else
    val := 0;
  end;
end;
}

//----------------------------------------------------------------------------//
//                       TGDALMajorObject                                     //
//----------------------------------------------------------------------------//

constructor TGDALMajorObject.Create;
begin
  inherited create;
  FObj := NIL;
end;

destructor TGDALMajorObject.Destroy;
begin
  inherited Destroy;
end;

function  TGDALMajorObject.GetDescription : PCHAR;
begin
  {$IFDEF DYNAMIC_GDAL}
   if IsGDALLoaded then
    begin
  {$ENDIF}
      result := GDALCore.GDALGetDescription(FObj);
  {$IFDEF DYNAMIC_GDAL}
    end else
    Result := '';
  {$ENDIF}
end;

procedure TGDALMajorObject.SetDescription (const Description : PCHAR);
begin
  {$IFDEF DYNAMIC_GDAL}
   if IsGDALLoaded then
  {$ENDIF}
    GDALCore.GDALSetDescription(FObj, Description);
end;

//' ----------------------------------------------------------------------------
function TGDALMajorObject.GetMetadata(Domain : PCHAR) : PPCHAR;
begin
  if (FObj <> NIL) {$IFDEF DYNAMIC_GDAL} and IsGDALLoaded{$ENDIF} then
    result := GDALCore.GDALGetMetadata(FObj, Domain);
end;

//' ----------------------------------------------------------------------------
function TGDALMajorObject.SetMetadata(MetaData : PPCHAR; Domain : PCHAR) : PPCHAR;
begin
  if (FObj <> NIL) {$IFDEF DYNAMIC_GDAL} and IsGDALLoaded{$ENDIF} then
    result := GDALCore.GDALSetMetadata(FObj, MetaData, Domain)
  else
    result := PPCHAR(GDALCore.ObjIsNULLError);
end;

//' ----------------------------------------------------------------------------
function TGDALMajorObject.GetMetadataItem(Name : PCHAR; Domain : PCHAR) : PCHAR;
begin
  if (FObj <> NIL) {$IFDEF DYNAMIC_GDAL} and IsGDALLoaded{$ENDIF} then
    result := GDALCore.GDALGetMetadataItem(FObj, Name, Domain)
  else
    result := NIL;

end;

//' -------------------------ECW SDK---------------------------------------------------
function TGDALMajorObject.SetMetadataItem(Name : PCHAR; Value : PCHAR; Domain : PCHAR) : TCPLErr;
begin
  if (FObj <> NIL) {$IFDEF DYNAMIC_GDAL} and IsGDALLoaded{$ENDIF} then
    result := GDALCore.GDALSetMetadataItem(FObj, Name, Value, Domain)
  else
    result := CE_None;
end;


//----------------------------------------------------------------------------
function GetLastErrorMsg : PCHAR;
begin
  {$IFDEF DYNAMIC_GDAL}
   if IsGDALLoaded and Assigned(@CPLGetLastErrorMsg) then
    Result := CPLGetLastErrorMsg
   else
    Result := 'GDAL not loaded.';
  {$ELSE}
   result := GDALCore.CPLGetLastErrorMsg;
  {$ENDIF}
end;

//' ----------------------------------------------------------------------------
function GetConfigOption(Key : PCHAR; Default : PCHAR) : PCHAR;
begin
{$IFDEF DYNAMIC_GDAL}
   if IsGDALLoaded then
   begin
{$ENDIF}
    result := GDALCore.CPLGetConfigOption(Key, Default);
{$IFDEF DYNAMIC_GDAL}
   end
    else
      Result := '';
{$ENDIF}      
end;

//' ----------------------------------------------------------------------------
procedure SetConfigOption(Key : PCHAR; Value : PCHAR);
begin
{$IFDEF DYNAMIC_GDAL}
   if IsGDALLoaded then
{$ENDIF}
    GDALCore.CPLSetConfigOption(Key, Value);
end;

//' ----------------------------------------------------------------------------
procedure AllRegister;
begin
{$IFDEF DYNAMIC_GDAL}
   if IsGDALLoaded then
{$ENDIF}
    GDALCore.GDALAllRegister;
end;

initialization
 {$IFDEF DYNAMIC_GDAL}
   GDALInit;
  {$ENDIF}
finalization
  {$IFDEF DYNAMIC_GDAL}
    GDALDeinit;
  {$ENDIF}  

end.

