{'*****************************************************************************
' Build 0135
'
' Project:  GDAL Lazarus Bindings
' Purpose:  Lazarus GDALDriver Shadow Class.
' Author:   Frank Warmerdam, warmerdam@pobox.com
' Lazarus Wrapper : Prajuab Riabroy
'*****************************************************************************
' Copyright (c) 2005, Frank Warmerdam <warmerdam@pobox.com>
'
' Permission is hereby granted, free of charge, to any person obtaining a
' copy of this software and associated documentation files (the "Software"),
' to deal in the Software without restriction, including without limitation
' the rights to use, copy, modify, merge, publish, distribute, sublicense,
' and/or sell copies of the Software, and to permit persons to whom the
' Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included
' in all copies or substantial portions of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
' OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
' FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
' DEALINGS IN THE SOFTWARE.
'*****************************************************************************
'}
unit gdaldriver;

{$IFDEF FPC}
{$mode objfpc}{$H+}
{$ENDIF}

interface

uses
  Classes, SysUtils, gdal, gdaldataset, gdalcore;

type
   TGDALDriver = class(TGDALMajorObject)
   private
     //Handle : TGDALDriverH;
   public
     function  GDalCreate(const Filename : PCHAR; const XSize : longint;
                          const YSize : longint; const BandCount : longint;
                          const DataType : TGDALDataType; Options : PPCHAR) : TGDALDataset;
     procedure CInit(objA : TGDALDriverH);
     function  CreateCopy(const Filename : PCHAR; const SrcDS : TGDALDataset;
                          ApproxOK : Boolean; Options : PPCHAR) : TGDALDataset;
     function  Delete(const pszFilename : PCHAR) : TCPLErr;
     function  Rename(const pszNewName : PCHAR;
                      const pszOldName : PCHAR) : TCPLErr;
     constructor Create;
     destructor  Destroy; override;
  end;

implementation

//' ----------------------------------------------------------------------------
procedure TGDALDriver.CInit(objA : TGDALDriverH);
begin
  Handle := objA;
end;

//' ----------------------------------------------------------------------------


function TGDALDriver.GDalCreate(const Filename : PCHAR; const XSize : longint;
                    const YSize : longint; const BandCount : longint;
                    const DataType : TGDALDataType; Options : PPCHAR) : TGDALDataset;
var
  hds : TGDALDriverH;
  ds : TGDALDataset;
begin
  if (Handle <> NIL) then
  begin
    hds := gdalcore.GDALCreate(Handle, Filename, XSize, YSize, BandCount,
                               DataType, Options);
    GDALCore.CSLDestroy(Options);
    if (hds <> NIL) then
    begin
      ds := TGDALDataset.Create;
      ds.CInit(hds, true);
      result := ds
    end;
  end;
end;

//' ----------------------------------------------------------------------------
function TGDALDriver.CreateCopy(const Filename : PCHAR; const SrcDS : TGDALDataset;
             ApproxOK : boolean; Options : PPCHAR) : TGDALDataset;
var
  ds_obj : TGDALDatasetH;
  ApproxOK_Int : longint;
  ds : TGDALDataset;
begin
  if (Handle <> NIL) and (SrcDS.GetObjPtr <> NIL) then
  begin
    if (ApproxOK) then
      ApproxOK_Int := 1
    else
      ApproxOK_Int := 0;

    //CSLOptions := GDALCore.VariantToCSL(Options);
    ds_obj := GDALCore.GDALCreateCopy(Handle, Filename, SrcDS.GetObjPtr,
                                      ApproxOK_Int, Options, NIL, 0);
    GDALCore.CSLDestroy(Options);
    if (ds_obj <> NIL) then
    begin
      ds := TGDALDataset.Create;
      ds.CInit(ds_obj, true);
      result := ds;
    end;
  end;
end;

function TGDALDriver.Delete(const pszFilename : PCHAR) : TCPLErr;
begin
  result := GDALCore.GDALDeleteDataset(Handle, pszFilename);
end;

function TGDALDriver.Rename(const pszNewName : PCHAR; const pszOldName : PCHAR) : TCPLErr;
begin
  result := GDALCore.GDALRenameDataset(Handle, pszNewName, pszOldName);
end;

constructor TGDALDriver.Create;
begin
  inherited create;
  Handle := NIL;
end;

destructor TGDALDriver.Destroy;
begin
  inherited Destroy;
end;


end.

