{'*****************************************************************************
' Build 0135
'
' Project:  GDAL Lazarus Bindings
' Purpose:  Lazarus GDALRasterBand Shadow Class.
' Author:   Frank Warmerdam, warmerdam@pobox.com
' Lazarus Wrapper : Prajuab Riabroy
'*****************************************************************************
' Copyright (c) 2005, Frank Warmerdam <warmerdam@pobox.com>
'
' Permission is hereby granted, free of charge, to any person obtaining a
' copy of this software and associated documentation files (the "Software"),
' to deal in the Software without restriction, including without limitation
' the rights to use, copy, modify, merge, publish, distribute, sublicense,
' and/or sell copies of the Software, and to permit persons to whom the
' Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included
' in all copies or substantial portions of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
' OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
' FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
' DEALINGS IN THE SOFTWARE.
'*****************************************************************************
}

unit gdalrasterband;

{$IFDEF FPC}
{$mode objfpc}{$H+}
{$ENDIF}

interface

uses
  Classes, SysUtils, gdal, gdalcore, gdalcolortable;

type
  TGDALRasterband = class(TGDALMajorObject)
  private
    //FObj : TGDALRasterBandH;
    FOwned : longint;
    FXSize : longint;
    FYSize : longint;
    FBlockXSize : longint;
    FBlockYSize : longint;
    FDataType : TGDALDatatype;
  public
    property XSize : longint read FXSize write FXSize;
    property YSize : longint read FYSize write FYSize;
    property BlockXSize : longint read FBlockXSize write FBlockXSize;
    property BlockYSize : longint read FBlockYSize write FBlockYSize;
    property DataType : TGDALDatatype read FDataType write FDataType;

    function  GetXSize(hBand : TGDALRasterBandH) : longint;
    function  GetYSize(hBand : TGDALRasterBandH) : longint;
    function  GetBand(hBand : TGDALRasterBandH) : longint;
    function  GetRasterDataType(const hBand : TGDALRasterBandH) : TGDALDatatype;
    function  GetDataTypeSize(const DatatypeA : TGDALDataType) : longint;
    function  GetDataTypeName (const DatatypeA : TGDALDataType) : PCHAR ;

    procedure CInit(objA : TGDALRasterBandH);
    function  GetMinimum : double;
    function  GetMaximum : double;
    function  GetOffset : double;
    function  SetOffset(const Offset : double) : longint;
    function  GetScale : double;
    function  SetScale(const NewScale : double) : longint;
    function  GetNoDataValue(pSuccess : PINTEGER) : double;
    function  SetNoDataValue(const NoDataValue : double) : double;
    function  RasterIO (const RWFlag : TGDALRWFlag; const XOff : longint;
                        const YOff : longint; const XSizeA : longint;
                        const YSizeA : longint; pDataArray : POINTER;
                        DataTypeA : TGDALDatatype) : longint;
    {function  GetMetadata(Domain : PCHAR) : PPCHAR;
    function  SetMetadata(MetaData : PPCHAR; Domain : PCHAR) : PPCHAR;
    function  GetMetadataItem(Name : PCHAR; Domain : PCHAR) : PCHAR;
    function  SetMetadataItem(Name : PCHAR; Value : PCHAR; Domain : PCHAR) : longint;
    function  GetDescription : PCHAR;}
    function  GetColorTable : TGDALColorTable;
    function  SetColorTable(ColorTable : TGDALColorTable) : TCPLErr;
    constructor Create;
    destructor Destroy; override;
  end;

implementation

constructor TGDALRasterband.Create;
begin
  inherited create;
    Handle := NIL;
    FOwned := 0;
    FXSize := 0;
    FYSize := 0;
    FBlockXSize := 0;
    FBlockYSize := 0;
    FDataType := GDT_Unknown;
end;

destructor TGDALRasterband.Destroy;
begin
  //  if (Handle <> 0) and (FOwned <> 0) then
  //      CloseDS;
  inherited Destroy;
end;

//' ----------------------------------------------------------------------------
procedure TGDALRasterband.CInit(objA : TGDALRasterBandH);
begin
  Handle := objA;
  FXSize := GDALCore.GDALGetRasterBandXSize(Handle);
  FYSize := GDALCore.GDALGetRasterBandYSize(Handle);
  GDALCore.GDALGetBlockSize(Handle, @FBlockXSize, @FBlockYSize);
  FDataType := GDALCore.GDALGetRasterDataType(Handle);
end;

function  TGDALRasterband.GetXSize(hBand : TGDALRasterBandH) : longint;
begin
  result := GDALCore.GDALGetRasterBandXSize(hBand);
end;

function  TGDALRasterband.GetYSize(hBand : TGDALRasterBandH) : longint;
begin
  result := GDALCore.GDALGetRasterBandYSize(hBand);
end;

function TGDALRasterband.GetBand(hBand : TGDALRasterBandH) : longint;
begin
  result := GDALCore.GDALGetBandNumber(hBand);
end;

function  TGDALRasterband.GetRasterDataType (const hBand : TGDALRasterBandH) : TGDALDatatype;
begin
  result := GDALCore.GDALGetRasterDataType(hBand);
end;

function  TGDALRasterband.GetDataTypeSize(const DatatypeA : TGDALDataType) : longint;
begin
  result := GDALCore.GDALGetDataTypeSize(Datatype);
end;

function  TGDALRasterband.GetDataTypeName(const DatatypeA : TGDALDataType) : PCHAR ;
begin
  result := GDALCore.GDALGetDataTypeName(DatatypeA);
end;

//' ----------------------------------------------------------------------------
function TGDALRasterband.GetMinimum : double;
var
  Success : longint;
begin
  if (Handle <> NIL) then
      result := GDALCore.GDALGetRasterMinimum(Handle, @Success);
end;

//' ----------------------------------------------------------------------------
function TGDALRasterband.GetMaximum : double;
var
  Success : longint;//PINTEGER;
begin
  if (Handle <> NIL) then
      result := GDALCore.GDALGetRasterMaximum(Handle, @Success);
end;

//' ----------------------------------------------------------------------------
function TGDALRasterband.GetOffset : double;
var
  Success : longint;
begin
  Success := 0;
  if (Handle <> NIL) then
      result := GDALCore.GDALGetRasterOffset(Handle, @Success);
end;

//' ----------------------------------------------------------------------------
function TGDALRasterband.SetOffset(const Offset : double) : longint;
begin
  if (Handle <> NIL) then
    result := GDALCore.GDALSetRasterOffset(Handle, Offset)
  else
    result := GDALCore.ObjIsNULLError;
end;

//' ----------------------------------------------------------------------------
function TGDALRasterband.GetScale : double;
var
  Success : longint;
begin
  if (Handle <> NIL) then
      result := GDALCore.GDALGetRasterScale(Handle, @Success);
end;

//' ----------------------------------------------------------------------------
function TGDALRasterband.SetScale(const NewScale : double) : longint;
begin
  if (Handle <> NIL) then
      result := GDALCore.GDALSetRasterScale(Handle, NewScale)
  else
      result := GDALCore.ObjIsNULLError;
end;


//' ----------------------------------------------------------------------------
function TGDALRasterband.GetNoDataValue(pSuccess : PINTEGER) : double;
begin
  if (Handle <> NIL) then
    result := GDALCore.GDALGetRasterNoDataValue(Handle, pSuccess)
  else
    pSuccess := NIL;
end;


//' ----------------------------------------------------------------------------
function TGDALRasterband.SetNoDataValue(const NoDataValue : double) : double;
begin
  if (Handle <> NIL) then
      result := GDALCore.GDALSetRasterNoDataValue(Handle, NoDataValue)
  else
      result := GDALCore.ObjIsNULLError;

end;

//' ----------------------------------------------------------------------------
function TGDALRasterband.RasterIO (const RWFlag : TGDALRWFlag; const XOff : longint;
                                   const YOff : longint; const XSizeA : longint;
                                   const YSizeA : longint; pDataArray : POINTER;
                                   DataTypeA : TGDALDatatype) : longint;
var
  BufXSize, BufYSize : longint;
begin
  if (Handle <> NIL) then
  begin
    //RawPtr := GDALCore.SafeArrayToPtr(DataArray, DataTypeA, BufXSize, BufYSize);
    BufXSize := XSizeA + 1;
    BufYSize := YSizeA;
    result := GDALCore.GDALRasterIO(Handle, RWFlag, XOff, YOff, XSizeA, YSizeA
                                        , pDataArray, BufXSize, BufYSize, DataTypeA, 0, 0);
  end
  else
    result := GDALCore.ObjIsNULLError
end;

{
//' ----------------------------------------------------------------------------
function TGDALRasterband.GetMetadata(Domain : PCHAR) : PPCHAR;
begin
  if (Handle <> NIL) then
      result :=  gdalcore.GDALGetMetadata(Handle, Domain);
      //result := GDALCore.GetMetadata(Handle, Domain);

end;

//' ----------------------------------------------------------------------------
function TGDALRasterband.SetMetadata(MetaData : PPCHAR; Domain : PCHAR) : PPCHAR;
begin
  if (Handle <> NIL) then
     result := GDALCore.GDALSetMetadata(Handle, MetaData, Domain)
     //result := GDALCore.SetMetadata(Handle, MetaData, Domain)
  else
     result := PPCHAR(GDALCore.ObjIsNULLError);
end;

//' ----------------------------------------------------------------------------
function TGDALRasterband.GetMetadataItem(Name : PCHAR; Domain : PCHAR) : PCHAR;
begin
  if (Handle <> NIL) then
      //result := GDALCore.CStr2VB(GDALCore.GDALGetMetadataItem(Handle, Name, Domain))
      result := GDALCore.GDALGetMetadataItem(Handle, Name, Domain)
  else
      result := NIL;//vbNullPCHAR
end;

//' ----------------------------------------------------------------------------
function TGDALRasterband.SetMetadataItem(Name : PCHAR;
                                Value : PCHAR; Domain : PCHAR) : longint;
begin
  if (Handle <> NIL) then
      result := GDALCore.GDALSetMetadataItem(Handle, Name, Value, Domain)
  else
      result := GDALCore.ObjIsNULLError;
end;

//' ----------------------------------------------------------------------------
function TGDALRasterband.GetDescription : PCHAR;
begin
  if (Handle <> NIL) then
     //result := GDALCore.CStr2VB(GDALCore.GDALGetDescription(FObj));
     result := GDALCore.GDALGetDescription(FObj);
end;
}
//' ----------------------------------------------------------------------------
function TGDALRasterband.GetColorTable : TGDALColorTable;
var
  ct : TGDALColorTable;
  CTHandle : TGDALColorTableH;
begin
  ct := TGDALColorTable.Create;
  if (Handle <> NIL) then
  begin
    CTHandle := GDALCore.GDALGetRasterColorTable(Handle);
    if (CTHandle <> NIL) then
      ct.CInit(CTHandle, 0);
  end;
  result := ct;
end;

//' ----------------------------------------------------------------------------
function TGDALRasterband.SetColorTable(ColorTable : TGDALColorTable) : TCPLErr;
begin
  if (Handle <> NIL) and (ColorTable.GetObjPtr <> NIL) then
    result := GDALCore.GDALSetRasterColorTable(Handle, ColorTable.GetObjPtr);
  //else
  //  result := ;//GDALCore.ObjIsNULLError;
end;


end.

