{'*****************************************************************************
' Build 0135
'
' Project:  GDAL Lazarus Bindings
' Purpose:  Lazarus OGRCoordinateTransformation Shadow Class.
' Author:   Frank Warmerdam, warmerdam@pobox.com
' Lazarus Wrapper : Prajuab Riabroy
'*****************************************************************************
' Copyright (c) 2005, Frank Warmerdam <warmerdam@pobox.com>
'
' Permission is hereby granted, free of charge, to any person obtaining a
' copy of this software and associated documentation files (the "Software"),
' to deal in the Software without restriction, including without limitation
' the rights to use, copy, modify, merge, publish, distribute, sublicense,
' and/or sell copies of the Software, and to permit persons to whom the
' Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included
' in all copies or substantial portions of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
' OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
' FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
' DEALINGS IN THE SOFTWARE.
'*****************************************************************************
}
unit ogrcoordinatetransformation;

{$IFDEF FPC}
{$mode objfpc}{$H+}
{$ENDIF}

interface

uses
  Classes, SysUtils, ogr, ogrcore;
type
  TOGRCoordinateTransformation = class
  private
    FObj : TOGRCoordinateTransformationH;
    FOwned : longint;
  public
    procedure   CInit(ObjA : TOGRCoordinateTransformationH; ownedA : longint);
    function    IsValid : boolean;
    function    TransformEx(X, Y, Z : double) : boolean;
    constructor Create;
    destructor  Destroy; override;
  end;

implementation

constructor TOGRCoordinateTransformation.Create;
begin
  inherited create;
    FObj := NIL;
    FOwned := 0;
end;

destructor TOGRCoordinateTransformation.Destroy;
begin
  if (FObj <> NIL) And (FOwned <> 0) then
  begin
    ogrcore.OCTDestroyCoordinateTransformation(FObj);
//' Call MsgBox("Destroying OGRCoordinateTransformation")
    FObj := NIL;
    FOwned := 0;
  end;
  inherited Destroy;
end;

//' ----------------------------------------------------------------------------
procedure TOGRCoordinateTransformation.CInit(ObjA : TOGRCoordinateTransformationH; ownedA : longint);
begin
  if (FObj <> NIL) then
    ogrcore.OCTDestroyCoordinateTransformation(FObj);

  FObj := ObjA;
  FOwned := ownedA;
end;

//' ----------------------------------------------------------------------------
function TOGRCoordinateTransformation.IsValid : boolean;
begin
  if (FObj <> NIL) then
    IsValid := true
  else
    IsValid := false;
end;

//' ----------------------------------------------------------------------------
//' Eventually we might add a multi-point version but this is simple
//' to do for now, and quite useful.
function TOGRCoordinateTransformation.TransformEx(X, Y, Z : double) : boolean;
var
  Success : longint;
  RetValue : longint;
begin
  Success := 0;
  if (FObj <> NIL) then
  begin
    RetValue := ogrcore.OCTTransformEx(FObj, 1, @X, @Y, @Z, @Success);
    if (Success = 0) Or (RetValue = 0) then
      result := false
    else
      result := true;
    end
    else
      result := false;

end;

end.

