{'*****************************************************************************
' Build 0135
'
' Project:  GDAL Lazarus Bindings
' Purpose:  Lazarus OGRSpatialReference Shadow Class.
' Author:   Frank Warmerdam, warmerdam@pobox.com
' Lazarus Wrapper : Prajuab Riabroy
'*****************************************************************************
' Copyright (c) 2005, Frank Warmerdam <warmerdam@pobox.com>
' Permission is hereby granted, free of charge, to any person obtaining a
' copy of this software and associated documentation files (the "Software"),
' to deal in the Software without restriction, including without limitation
' the rights to use, copy, modify, merge, publish, distribute, sublicense,
' and/or sell copies of the Software, and to permit persons to whom the
' Software is furnished to do so, subject to the following conditions:

' The above copyright notice and this permission notice shall be included
' in all copies or substantial portions of the Software.

' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
' OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
' FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
' DEALINGS IN THE SOFTWARE.
'*****************************************************************************
}
unit ogrspatialreference;

{$IFDEF FPC}
{$mode objfpc}{$H+}
{$ENDIF}

interface

uses
  Classes, SysUtils, gdalcore, ogr, ogrcore;
type
  TOGRSpatialReference = class
  private
    FObj : TOGRSpatialReferenceH;
    FOwned : longint;
  public
    procedure   CInit(ObjA : TOGRSpatialReferenceH; ownedA : longint);
    function    GetObjPtr : TOGRSpatialReferenceH;
    function    SetFromUserInput(InputText : PCHAR) : TOGRErr;
    function    Clone : TOGRSpatialReference;
    function    CloneGeogCS : TOGRSpatialReference;
    function    ImportFromEPSG(EPSGCode : longint) : TOGRErr;
    function    ExportToWkt : PCHAR;
    function    ExportToPrettyWkt(bSimplify : longint) : PPCHAR;
    function    SetAttrValue(NodePath : PCHAR; NodeValue : PCHAR) : TOGRErr;
    function    GetAttrValue(NodePath : PCHAR; iChild : longint) : PCHAR;
    constructor Create;
    destructor  Destroy; override;
  end;

implementation

constructor TOGRSpatialReference.Create;
begin
  inherited create;
  FObj := ogrcore.OSRNewSpatialReference(NIL);
  FOwned := 1;
end;

destructor TOGRSpatialReference.Destroy;
begin
  if (FObj <> NIL) and (FOwned <> 0) then
  begin
    //Call MsgBox("Destroy OGRSpatialReference: " & ExportToWkt())
    ogrcore.OSRDestroySpatialReference(FObj);
    FObj := NIL;
    FOwned := 0;
  end;
  inherited Destroy;
end;

//' ----------------------------------------------------------------------------
procedure TOGRSpatialReference.CInit(ObjA : TOGRSpatialReferenceH; ownedA : longint);
begin
  if (FObj <> NIL) And (FOwned <> 0) then
  begin
    ogrcore.OSRDestroySpatialReference(FObj);
    FObj := NIL;
    FOwned := 0;
    ogrcore.OSRDestroySpatialReference(FObj);
  end;
  FObj := ObjA;
  FOwned := ownedA;
end;

//' ----------------------------------------------------------------------------
function TOGRSpatialReference.GetObjPtr : TOGRSpatialReferenceH;
begin
  result := FObj;
end;

//' ----------------------------------------------------------------------------
function TOGRSpatialReference.SetFromUserInput(InputText : PCHAR) : TOGRErr;
begin
  if (FObj <> NIL) then
    result := ogrcore.OSRSetFromUserInput(FObj, InputText)
  else
   Result := OGRERR_INVALID_HANDLE;  
//  else
//    SetFromUserInput := ogrcore.ObjIsNULLError;
end;

//' ----------------------------------------------------------------------------
function TOGRSpatialReference.Clone : TOGRSpatialReference;
var
  srs : TOGRSpatialReference;
  Obj_new : TOGRSpatialReferenceH;
begin
  srs := TOGRSpatialReference.Create;
  if (FObj <> NIL) then
  begin
    Obj_new := ogrcore.OSRClone(FObj);
    srs.CInit(Obj_new, 1);
  end;
  result := srs;
end;

//' ----------------------------------------------------------------------------
function TOGRSpatialReference.CloneGeogCS : TOGRSpatialReference;
var
  srs : TOGRSpatialReference;
  Obj_new : TOGRSpatialReferenceH;
begin
  srs := TOGRSpatialReference.Create;
  if (FObj <> NIL) then
  begin
    Obj_new := ogrcore.OSRCloneGeogCS(FObj);
    srs.CInit(Obj_new, 1);
  end;
  result := srs;
end;

//' ----------------------------------------------------------------------------
function TOGRSpatialReference.ImportFromEPSG(EPSGCode : longint) : TOGRErr;
begin
  if (FObj <> NIL) then
    result := ogrcore.OSRImportFromEPSG(FObj, EPSGCode)
  else
   Result := OGRERR_INVALID_HANDLE;  
  //else
  //  ImportFromEPSG := GDALCore.ObjIsNULLError;
end;

//' ----------------------------------------------------------------------------
function TOGRSpatialReference.ExportToWkt : PCHAR;
var
  wktptr : PPCHAR;
  err : TOGRErr;
begin
  if (FObj <> NIL) then
  begin
    err := ogrcore.OSRExportToWkt(FObj, wktptr);
    result := PCHAR(wktptr);//GDALCore.CStr2VB(wktptr);
    // We really should free wktptr here.
  end
  else
    result := '';
end;

//' ----------------------------------------------------------------------------
function TOGRSpatialReference.ExportToPrettyWkt(bSimplify : longint) : PPCHAR;
var
  wktptr : PPCHAR;
  err : TOGRErr;
begin
  if (FObj <> NIL) then
  begin
    err := ogrcore.OSRExportToPrettyWkt(FObj, wktptr, bSimplify);
    result := wktptr;
    //We really should free wktptr here.
  end
  else
    result := NIL;
end;

//' ----------------------------------------------------------------------------
function TOGRSpatialReference.SetAttrValue(NodePath : PCHAR; NodeValue : PCHAR) : TOGRErr;
begin
  if (FObj <> NIL) then
    result := ogrcore.OSRSetAttrValue(FObj, NodePath, NodeValue)
  else
   Result := OGRERR_INVALID_HANDLE;  
  //else
  //  result := GDALCore.ObjIsNULLError;
end;

//' ----------------------------------------------------------------------------
function TOGRSpatialReference.GetAttrValue(NodePath : PCHAR; iChild : longint) : PChar;
var
  c_str : PChar;
begin
  if (FObj <> NIL) then
  begin
    c_str := ogrcore.OSRGetAttrValue(FObj, NodePath, iChild);
    if (c_str = NIL) then
      result := ''
    else
      result := c_str;//GDALCore.CStr2VB(c_str);
  end
  else
    result := '';
end;


end.

