
{ Summary
  <b>Project:</b> GDAL Lazarus/Delphi Bindings
  
  <b>Purpose</b>: OGR Lazarus/Delphi internal support
  functions.
  
  <b>Author:</b> Frank Warmerdam, warmerdam@pobox.com
  
  <b>Lazarus Wrapper:</b> Prajuab Riabroy
  
  <b>Delphi Wrapper:</b>
  Source Description
  Copyright (c) 2005, Frank Warmerdam \<warmerdam@pobox.com\>
  
  
  
  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated
  documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation
  
  the rights to use, copy, modify, merge, publish, distribute,
  sublicense, and/or sell copies of the Software, and to permit
  persons to whom the Software is furnished to do so, subject
  to the following conditions:
  
  
  
  The above copyright notice and this permission notice shall
  be included in all copies or substantial portions of the
  Software.
  
  
  
  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
  KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
  PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
  OTHERWISE, ARISING

  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
  OTHER DEALINGS IN THE SOFTWARE.
  Version
  Build 0135                                                    }
unit ogrcore;

{$I gdal.inc}

interface

uses
  Classes, SysUtils, gdal, ogr;
{$IFNDEF DYNAMIC_GDAL}  
{ OGR_G_CreateFromWkb() *
  
  
  
  Create a geometry object of the appropriate type from it's
  well known binary representation.
  
  
  
  \Note that if nBytes is passed as zero, no checking can be
  done on whether the pabyData is sufficient. This can result
  in a crash if the input data is corrupt. This function
  Parameters
  pabyData :    pointer to the input BLOB data.
  hSRS :        handle to the spatial reference to be assigned
                to the created geometry object. This may be
                NULL.
  phGeometry :  the newly created geometry object will be
                assigned to the indicated handle on return. This
                will be NULL in case of failure.
  nBytes :      the number of bytes of data available in
                pabyData, or \-1 if it is not known, but assumed
                to be sufficient.<p />*
  Returns
  no indication of the number of bytes from the data source
  actually used to represent the returned geometry object. Use
  OGR_G_WkbSize() on the returned geometry to establish the
  number of bytes it required in WKB format.

  OGRERR_NONE if all goes well, otherwise any of
  OGRERR_NOT_ENOUGH_DATA, OGRERR_UNSUPPORTED_GEOMETRY_TYPE, or
  OGRERR_CORRUPT_DATA may be returned.                           }

  function OGR_G_CreateFromWkb(pabyData: PChar; hSRS: TOGRSpatialReferenceH; phGeometry: TOGRGeometryH; nBytes: Integer): TOGRErr; cdecl; external external_lib name 'OGR_G_CreateFromWkb';

  { createFromWkt()
    
    
    
    Create a geometry object of the appropriate type from it's
    well known text representation.
    
    The C function OGR_G_CreateFromWkt() is the same as this
    method.
    Parameters
    ppszData :   \Input zero terminated string containing well
                 known text representation of the geometry to be
                 created. The pointer is updated to point just
                 beyond that last character consumed.
    poSR :       pointer to the spatial reference to be assigned
                 to the created geometry object. This may be NULL.
    ppoReturn :  the newly created geometry object will be
                 assigned to the indicated pointer on return. This
                 will be NULL if the method fails.
    Returns
    OGRERR_NONE if all goes well, otherwise any of
    OGRERR_NOT_ENOUGH_DATA,OGRERR_UNSUPPORTED_GEOMETRY_TYPE, or
    OGRERR_CORRUPT_DATA may be returned.
    Example
    <code lang="c++">
    const char* wkt = "POINT(0 0)";
    char* pszWkt = (char*) wkt.c_str();
    OGRSpatialRefernceH ref = OSRNewSpatialReference(NULL);
    OGRGeometryH new_geom;
    OGRErr err = OGR_G_CreateFromWkt(&amp;pszWkt, ref, &amp;new_geom);
    </code>
    
    <code lang="delphi">
    var Wkt: String;
        Ref: OGRSpatialReferenceH;
        Geom: OGRGeometryH;
        Err: OGRErr;
    begin
      Wkt := 'POINT(0 0)';
      Ref := OSRNewSpatialReference(nil);
      Err := OGR_G_CreateFromWkt(PChar(Wkt), Ref, Geom);
    end;
    </code>                                                            }

  function OGR_G_CreateFromWkt(ppszData: PPChar; poSR: TOGRSpatialReferenceH; ppoReturn: TOGRGeometryH): TOGRErr; cdecl; external external_lib name 'OGR_G_CreateFromWkt';
  function OGR_G_CreateFromFgf( Fgf: PChar; Ref: TOGRSpatialReferenceH; hGeom: TOGRGeometryH; nBytes: integer; var nBytesConsumed: Integer): TOGRErr; cdecl; external external_lib name 'OGR_G_CreateFromFgf';
  { Destroy geometry object.

    Equivalent to invoking delete on a geometry, but it
    guaranteed to take place within the context of the GDAL/OGR
    heap.

    This function is the same as the CPP method
    OGRGeometryFactory::destroyGeometry.

    Parameters
    
    hGeom : handle to the geometry to delete.                   }

  procedure OGR_G_DestroyGeometry(hGeom: TOGRGeometryH); cdecl; external external_lib name 'OGR_G_DestroyGeometry';
  { reate an empty geometry of desired type.



    This is equivelent to allocating the desired geometry with
    new, but the allocation is guaranteed to take place in the
    context of the GDAL/OGR heap.



    This function is the same as the CPP method
    OGRGeometryFactory::createGeometry.



    Parameters

    eGeometryType : the type code of the geometry to be created.

    
    
    Returns
    
    handle to the newly create geometry or NULL on failure.      }
  function OGR_G_CreateGeometry(GeomType: TOGRwkbGeometryType): TOGRGeometryH; cdecl; external external_lib name 'OGR_G_CreateGeometry';
  { Get the dimension of this geometry.
    
    This function corresponds to the SFCOM
    IGeometry::GetDimension() method. It indicates the dimension
    of the geometry, but does not indicate the dimension of the
    underlying space (as indicated by
    OGR_G_GetCoordinateDimension() function).
    
    This function is the same as the CPP method
    OGRGeometry::getDimension().
    
    
    
    Parameters
    
    hGeom : handle on the geometry to get the dimension from.
    
    
    
    Returns
    
    0 for points, 1 for lines and 2 for surfaces.                }
  function OGR_G_GetDimension(hGeom: TOGRGeometryH ): integer; cdecl; external external_lib name 'OGR_G_GetDimension';

  { Get the dimension of the coordinates in this geometry.
    
    This function corresponds to the SFCOM
    IGeometry::GetDimension() method.
    
    This function is the same as the CPP method
    OGRGeometry::getCoordinateDimension().
    
    
    
    Parameters
    
    hGeom : handle on the geometry to get the dimension of the
    coordinates from.

    
    
    Returns
    
    in practice this always returns 2 indicating that coordinates
    are specified within a two dimensional space.                 }
  function OGR_G_GetCoordinateDimension(hGeom: TOGRGeometryH ): Integer; cdecl; external external_lib name 'OGR_G_GetCoordinateDimension';
  procedure OGR_G_SetCoordinateDimension(hGeom: TOGRGeometryH; nNewDimension: Integer ); cdecl; external external_lib name 'OGR_G_SetCoordinateDimension';
  { Make a copy of this object.
    
    
    
    This function relates to the SFCOM IGeometry::clone() method.
    
    
    
    This function is the same as the CPP method
    OGRGeometry::clone().
    
    
    
    Parameters
    
    hGeom : handle on the geometry to clone from.
    
    
    
    Returns
    
    an handle on the copy of the geometry with the spatial
    reference system as the original.                             }
  function OGR_G_Clone(hGeom: TOGRGeometryH ): TOGRGeometryH; cdecl; external external_lib name 'OGR_G_Clone';
  { Computes and returns the bounding envelope for this geometry
    in the passed psEnvelope structure.
    
    This function is the same as the CPP method
    OGRGeometry::getEnvelope().
    

    
    Parameters
    
    hGeom : handle of the geometry to get envelope from.
    
    psEnvelope : the structure in which to place the results.    }
  procedure OGR_G_GetEnvelope(hGeom: TOGRGeometryH; var psEnvelope: TOGREnvelope); cdecl; external external_lib name 'OGR_G_GetEnvelope';
  { Assign geometry from well known binary data.
    
    The object must have already been instantiated as the correct
    derived type of geometry object to match the binaries type.
    
    This function relates to the SFCOM IWks::ImportFromWKB()
    method.
    
    This function is the same as the CPP method
    OGRGeometry::importFromWkb().
    
    Parameters
    
    hGeom : handle on the geometry to assign the well know binary
    data to.
    
    pabyData : the binary input data.
    
    nSize : the size of pabyData in bytes, or zero if not known.
    
    Returns
    
    OGRERR_NONE if all goes well, otherwise any of
    OGRERR_NOT_ENOUGH_DATA, OGRERR_UNSUPPORTED_GEOMETRY_TYPE, or
    OGRERR_CORRUPT_DATA may be returned.                          }
  function OGR_G_ImportFromWkb(hGeom: TOGRGeometryH; pabyData: PChar; nSize: Integer): TOGRErr; cdecl; external external_lib name 'OGR_G_ImportFromWkb';
  { Convert a geometry into well known binary format.
    
    
    
    This function relates to the SFCOM IWks::ExportToWKB()
    method.
    
    
    
    This function is the same as the CPP method
    OGRGeometry::exportToWkb().
    
    
    
    Parameters

    hGeom : handle on the geometry to convert to a well know
    binary data from.
    
    eOrder : One of wkbXDR or wkbNDR indicating MSB or LSB byte
    order respectively.
    
    pabyDstBuffer : a buffer into which the binary representation
    is written. This buffer must be at least OGR_G_WkbSize() byte
    in size.
    
    
    
    Returns
    
    Currently OGRERR_NONE is always returned.                     }
  function OGR_G_ExportToWkb(hGeom: TOGRGeometryH; ByteOrder: TOGRwkbByteOrder; Wkb: PChar): TOGRErr; cdecl; external external_lib name 'OGR_G_ExportToWkb';
  { Returns size of related binary representation.
    
    This function returns the exact number of bytes required to
    hold the well known binary representation of this geometry
    object. Its computation may be slightly expensive for complex
    geometries.
    
    This function relates to the SFCOM IWks::WkbSize() method.
    
    This function is the same as the CPP method
    OGRGeometry::WkbSize().
    
    Parameters
    
    hGeom : handle on the geometry to get the binary size from.
    
    Returns
    
    size of binary representation in bytes.                       }
  function OGR_G_WkbSize(hGeom: TOGRGeometryH): Integer; cdecl; external external_lib name 'OGR_G_WkbSize';
  { Assign geometry from well known text data.
    
    The object must have already been instantiated as the correct
    derived type of geometry object to match the text type.
    
    This function relates to the SFCOM IWks::ImportFromWKT()
    method.
    
    This function is the same as the CPP method
    OGRGeometry::importFromWkt().
    
    Parameters
    
    hGeom : handle on the geometry to assign well know text data
    to.
    
    ppszSrcText : pointer to a pointer to the source text. The
    pointer is updated to pointer after the consumed text.
    
    Returns
    
    OGRERR_NONE if all goes well, otherwise any of
    OGRERR_NOT_ENOUGH_DATA, OGRERR_UNSUPPORTED_GEOMETRY_TYPE, or
    OGRERR_CORRUPT_DATA may be returned.                          }
  function OGR_G_ImportFromWkt(hGeom: TOGRGeometryH; ppszSrcText: PPChar): TOGRErr; cdecl; external external_lib name 'OGR_G_ImportFromWkt';
  { Convert a geometry into well known text format.
    
    This function relates to the SFCOM IWks::ExportToWKT()
    method.
    
    This function is the same as the CPP method
    OGRGeometry::exportToWkt().
    
    
    
    Parameters
    
    hGeom : handle on the geometry to convert to a text format
    from.
    
    ppszSrcText : a text buffer is allocated by the program, and
    assigned to the passed pointer.
    
    
    
    Returns
    
    Currently OGRERR_NONE is always returned.                    }
  function OGR_G_ExportToWkt(hGeom: TOGRGeometryH; Wkt: PPChar ): TOGRErr; cdecl; external external_lib name 'OGR_G_ExportToWkt';
  { Fetch geometry type.
    
    \Note that the geometry type may include the 2.5D flag. To
    get a 2D flattened version of the geometry type apply the
    wkbFlatten() macro to the return result.
    
    This function is the same as the CPP method
    OGRGeometry::getGeometryType().
    
    
    
    Parameters
    
    hGeom : handle on the geometry to get type from.
    
    
    
    Returns
    
    the geometry type code.                                    }
  function OGR_G_GetGeometryType(hGeom: TOGRGeometryH): TOGRwkbGeometryType; cdecl; external external_lib name 'OGR_G_GetGeometryType';
  { Fetch WKT name for geometry type.
    
    There is no SFCOM analog to this function.
    
    This function is the same as the CPP method
    OGRGeometry::getGeometryName().
    
    
    
    Parameters
    
    hGeom : handle on the geometry to get name from.
    
    
    
    Returns
    
    name used for this geometry type in well known text format.  }
  function OGR_G_GetGeometryName(hGeom: TOGRGeometryH): PChar; cdecl; external external_lib name 'OGR_G_GetGeometryName';
  { Dump geometry in well known text format to indicated output
    \file.
    
    
    
    This method is the same as the CPP method
    OGRGeometry::dumpReadable.
    
    
    
    Parameters
    
    hGeom : handle on the geometry to dump.
    
    fp : the text file to write the geometry to.
    
    pszPrefix : the prefix to put on each line of output.       }
  procedure OGR_G_DumpReadable(hGeom: TOGRGeometryH; var pfp : file; pszPrefix: pChar ); cdecl; external external_lib name 'OGR_G_DumpReadable';
  { Convert geometry to strictly 2D. In a sense this converts all
    Z coordinates to 0.0.

    This function is the same as the CPP method
    OGRGeometry::flattenTo2D().
    
    
    
    Parameters
    
    hGeom : handle on the geometry to convert.                    }
  procedure OGR_G_FlattenTo2D(hGeom: TOGRGeometryH ); cdecl; external external_lib name 'OGR_G_FlattenTo2D';
  procedure OGR_G_CloseRings(hGeom: TOGRGeometryH ); cdecl; external external_lib name 'OGR_G_CloseRings';

  function OGR_G_CreateFromGML(GML: PChar): TOGRGeometryH; cdecl; external external_lib name 'OGR_G_CreateFromGML';
  function OGR_G_ExportToGML(hGeom: TOGRGeometryH ): PChar; cdecl; external external_lib name 'OGR_G_ExportToGML';

  function OGR_G_ExportToKML(hGeom: TOGRGeometryH; pszAltitudeMode: PChar): PChar; cdecl; external external_lib name 'OGR_G_ExportToKML';

  function OGR_G_ExportToJson(hGeom: TOGRGeometryH ): PChar; cdecl; external external_lib name 'OGR_G_ExportToJson';
  function OGR_G_CreateGeometryFromJson(Json: PChar): TOGRGeometryH; cdecl; external external_lib name 'OGR_G_CreateGeometryFromJson';

  { Assign spatial reference to this object.
    
    
    
    Any existing spatial reference is replaced, but under no
    circumstances does this result in the object being
    reprojected. It is just changing the interpretation of the
    existing geometry. Note that assigning a spatial reference
    increments the reference count on the OGRSpatialReference,
    but does not copy it.
    
    
    
    This is similar to the SFCOM
    IGeometry::put_SpatialReference() method.
    
    
    
    This function is the same as the CPP method
    OGRGeometry::assignSpatialReference.
    
    
    
    Parameters
    
    hGeom : handle on the geometry to apply the new spatial
    reference system.
    
    hSRS : handle on the new spatial reference system to apply.  }
  procedure OGR_G_AssignSpatialReference(hGeom: TOGRGeometryH;
                                             hRef: TOGRSpatialReferenceH ); cdecl; external external_lib name 'OGR_G_AssignSpatialReference';
  { Returns spatial reference system for geometry.
    
    This function relates to the SFCOM
    IGeometry::get_SpatialReference() method.
    
    This function is the same as the CPP method
    OGRGeometry::getSpatialReference().
    
    
    
    Parameters
    
    hGeom : handle on the geometry to get spatial reference from.
    
    
    
    Returns
    
    a reference to the spatial reference geometry.                }
  function OGR_G_GetSpatialReference(hGeom: TOGRGeometryH): TOGRSpatialReferenceH; cdecl; external external_lib name 'OGR_G_GetSpatialReference';
  { Apply arbitrary coordinate transformation to geometry.
    
    This function will transform the coordinates of a geometry
    from their current spatial reference system to a new target
    spatial reference system. Normally this means reprojecting
    the vectors, but it could include datum shifts, and changes
    of units.

    This function is the same as the CPP method
    OGRGeometry::transform.
    
    Parameters
    
    hGeom : handle on the geometry to apply the transform to.
    
    hTransform : handle on the transformation to apply.
    
    Returns
    
    OGRERR_NONE on success or an error code.
    Note
    \Note that this function does not require that the geometry
    already have a spatial reference system. It will be assumed
    that they can be treated as having the source spatial
    reference system of the OGRCoordinateTransformation object,
    and the actual SRS of the geometry will be ignored. On
    successful completion the output OGRSpatialReference of the
    OGRCoordinateTransformation will be assigned to the geometry.
                                                                  }
  function OGR_G_Transform(hGeom: TOGRGeometryH; hTransform: TOGRCoordinateTransformationH ): TOGRErr; cdecl; external external_lib name 'OGR_G_Transform';
  { Transform geometry to new spatial reference system.
    
    This function will transform the coordinates of a geometry
    from their current spatial reference system to a new target
    spatial reference system. Normally this means reprojecting
    the vectors, but it could include datum shifts, and changes
    of units.
    
    This function will only work if the geometry already has an
    assigned spatial reference system, and if it is transformable
    to the target coordinate system.
    
    Because this function requires internal creation and
    initialization of an OGRCoordinateTransformation object it is
    significantly more expensive to use this function to
    transform many geometries than it is to create the
    OGRCoordinateTransformation in advance, and call transform()
    with that transformation. This function exists primarily for
    convenience when only transforming a single geometry.
    
    This function is the same as the CPP method
    OGRGeometry::transformTo.
    
    Parameters
    
    hGeom : handle on the geometry to apply the transform to.
    
    hSRS : handle on the spatial reference system to apply.
    
    Returns
    
    OGRERR_NONE on success, or an error code.                     }
  function OGR_G_TransformTo(hGeom: TOGRGeometryH; hRef: TOGRSpatialReferenceH ): TOGRErr; cdecl; external external_lib name 'OGR_G_TransformTo';
  { Modify the geometry such it has no segment longer then the
    given distance.
    
    Interpolated points will have Z and M values (if needed) set
    to 0. Distance computation is performed in 2d only
    
    This function is the same as the CPP method
    OGRGeometry::segmentize().
    
    Parameters
    
    hGeom : handle on the geometry to segmentize
    
    dfMaxLength : the maximum distance between 2 points after
    segmentization                                               }
  procedure OGR_G_Segmentize(hGeom: TOGRGeometryH; dfMaxLength: Double ); cdecl; external external_lib name 'OGR_G_Segmentize';
  
  { Do these features intersect?
    
    Currently this is not implemented in a rigerous fashion, and
    generally just tests whether the envelopes of the two
    features intersect. Eventually this will be made rigerous.

    This function is the same as the CPP method
    OGRGeometry::Intersects.
    
    Parameters
    
    hGeom : handle on the first geometry.
    
    hOtherGeom : handle on the other geometry to test against.
    
    Returns
    
    TRUE if the geometries intersect, otherwise FALSE.           }
  function OGR_G_Intersects(hGeom: TOGRGeometryH; hOtherGeom: TOGRGeometryH ): Integer; cdecl; external external_lib name 'OGR_G_Intersects';
  { Returns TRUE if two geometries are equivalent.
    
    
    
    This function is the same as the CPP method
    OGRGeometry::Equals() method.
    
    
    
    Parameters
    
    hGeom : handle on the first geometry.
    
    hOther : handle on the other geometry to test against.
    
    
    
    Returns
    
    TRUE if equivalent or FALSE otherwise.                 }
  function OGR_G_Equals(hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl; external external_lib name 'OGR_G_Equals';
  { Test for disjointness.
    
    
    
    Tests if this geometry and the other geometry are disjoint.
    
    
    
    This function is the same as the C++ method
    OGRGeometry::Disjoint().
    
    
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always fail, issuing a
    CPLE_NotSupported error.
    
    
    
    Parameters
    
    hThis : the geometry to compare.
    
    hOther : the other geometry to compare.
    
    
    
    Returns
    
    TRUE if they are disjoint, otherwise FALSE.                   }
  function OGR_G_Disjoint(hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl; external external_lib name 'OGR_G_Disjoint';
  { Test for touching.
    
    Tests if this geometry and the other geometry are touching.
    
    This function is the same as the C++ method
    OGRGeometry::Touches().
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always fail, issuing a
    CPLE_NotSupported error.

    Parameters :
    
    hThis : the geometry to compare.
    
    hOther : the other geometry to compare.                       }
  function OGR_G_Touches(hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl; external external_lib name 'OGR_G_Touches';
  { Test for crossing.
    
    
    
    Tests if this geometry and the other geometry are crossing.
    
    
    
    This function is the same as the C++ method
    OGRGeometry::Crosses().
    
    
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always fail, issuing a
    CPLE_NotSupported error.
    
    
    
    Parameters
    
    hThis : the geometry to compare.
    
    hOther : the other geometry to compare.
    
    
    
    Returns
    
    TRUE if they are crossing, otherwise FALSE.                   }
  function OGR_G_Crosses(hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl; external external_lib name 'OGR_G_Crosses';
  { Test for containment.
    
    Tests if this geometry is within the other geometry.
    
    This function is the same as the C++ method
    OGRGeometry::Within().
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always fail, issuing a
    CPLE_NotSupported error.
    
    Parameters:
    
    hThis : the geometry to compare.
    
    hOther : the other geometry to compare.
    
    Returns
    
    TRUE if hThis is within hOther, otherwise FALSE.              }
  function OGR_G_Within(hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl; external external_lib name 'OGR_G_Within';
  { Test for containment.
    
    
    
    Tests if this geometry contains the other geometry.
    
    
    
    This function is the same as the C++ method
    OGRGeometry::Contains().
    
    
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always fail, issuing a
    CPLE_NotSupported error.

    
    
    Parameters
    
    hThis : the geometry to compare.
    
    hOther: the other geometry to compare.
    
    
    
    Returns
    
    TRUE if hThis contains hOther geometry, otherwise FALSE.      }
  function OGR_G_Contains(hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl; external external_lib name 'OGR_G_Contains';
  { Test for overlap.
    
    Tests if this geometry and the other geometry overlap, that
    is their intersection has a non-zero area.
    
    This function is the same as the C++ method
    OGRGeometry::Overlaps().
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always fail, issuing a
    CPLE_NotSupported error.
    
    Parameters
    
    hThis : the geometry to compare.
    
    hOther : the other geometry to compare.
    
    Returns
    
    TRUE if they are overlapping, otherwise FALSE.                }
  function OGR_G_Overlaps(hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl; external external_lib name 'OGR_G_Overlaps';

  { Compute boundary.
    
    
    
    A new geometry object is created and returned containing the
    boundary of the geometry on which the method is invoked.
    
    
    
    This function is the same as the C++ method OGR_G_Boundary().
    
    
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always fail, issuing a
    CPLE_NotSupported error.
    
    
    
    Parameters
    
    hTarget : The Geometry to calculate the boundary of.
    
    
    
    Returns
    
    a handle to a newly allocated geometry now owned by the
    caller, or NULL on failure.                                   }
  function OGR_G_GetBoundary(hGeom: TOGRGeometryH): TOGRGeometryH; cdecl; external external_lib name 'OGR_G_GetBoundary';
  { Compute convex hull.
    
    
    
    A new geometry object is created and returned containing the
    convex hull of the geometry on which the method is invoked.
    
    

    This function is the same as the C++ method
    OGRGeometry::ConvexHull().
    
    
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always fail, issuing a
    CPLE_NotSupported error.
    
    
    
    Parameters
    
    hTarget : The Geometry to calculate the convex hull of.
    
    
    
    Returns
    
    a handle to a newly allocated geometry now owned by the
    caller, or NULL on failure.
    
    
                                                                  }
  function OGR_G_ConvexHull(hTarget: TOGRGeometryH): TOGRGeometryH; cdecl; external external_lib name 'OGR_G_ConvexHull';
  { Compute buffer of geometry.
    
    
    
    Builds a new geometry containing the buffer region around the
    geometry on which it is invoked. The buffer is a polygon
    containing the region within the buffer distance of the
    original geometry.
    
    
    
    Some buffer sections are properly described as curves, but
    are converted to approximate polygons. The nQuadSegs
    parameter can be used to control how many segements should be
    used to define a 90 degree curve - a quadrant of a circle. A
    value of 30 is a reasonable default. Large values result in
    large numbers of vertices in the resulting buffer geometry
    while small numbers reduce the accuracy of the result.
    
    
    
    This function is the same as the C++ method
    OGRGeometry::Buffer().
    
    
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always fail, issuing a
    CPLE_NotSupported error.
    
    
    
    Parameters
    
    hTarget : the geometry.
    
    dfDist : the buffer distance to be applied.
    
    nQuadSegs : the number of segments used to approximate a 90
    degree (quadrant) of curvature.
    
    
    
    Returns
    
    the newly created geometry, or NULL if an error occurs.       }
  function OGR_G_Buffer(hGeom: TOGRGeometryH; dfDist: double; nQuadSegs: integer): TOGRGeometryH; cdecl; external external_lib name 'OGR_G_Buffer';
  { Compute intersection.
    
    Generates a new geometry which is the region of intersection
    of the two geometries operated on. The OGR_G_Intersects()
    function can be used to test if two geometries intersect.
    
    This function is the same as the C++ method
    OGRGeometry::Intersection().
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always fail, issuing a
    CPLE_NotSupported error.
    
    Parameters
    
    hThis : the geometry.
    
    hOther : the other geometry.
    
    Returns
    
    a new geometry representing the intersection or NULL if there
    is no intersection or an error occurs.                        }
  function OGR_G_Intersection(hGeom: TOGRGeometryH; hOther: TOGRGeometryH): TOGRGeometryH; cdecl; external external_lib name 'OGR_G_Intersection';
  { Compute union.
    
    Generates a new geometry which is the region of union of the
    two geometries operated on.
    
    This function is the same as the C++ method
    OGRGeometry::Union().
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always fail, issuing a
    CPLE_NotSupported error.
    
    Parameters
    
    hThis : the geometry.
    
    hOther: the other geometry.
    
    Returns
    
    a new geometry representing the union or NULL if an error
    occurs.                                                       }
  function OGR_G_Union(hGeom1: TOGRGeometryH; hGeom2: TOGRGeometryH): TOGRGeometryH; cdecl; external external_lib name 'OGR_G_Union';
  { Compute difference.
    
    
    
    Generates a new geometry which is the region of this geometry
    with the region of the other geometry removed.
    
    
    
    This function is the same as the C++ method
    OGRGeometry::Difference().
    
    
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always fail, issuing a
    CPLE_NotSupported error.
    
    
    
    Parameters
    
    hThis : the geometry.
    
    hOther : the other geometry.
    
    
    
    Returns
    
    a new geometry representing the difference or NULL if the
    difference is empty or an error occurs.                       }
  function OGR_G_Difference(hGeom1: TOGRGeometryH; hGeom2: TOGRGeometryH): TOGRGeometryH; cdecl; external external_lib name 'OGR_G_Difference';
  { Compute symmetric difference.
    
    Generates a new geometry which is the symmetric difference of
    this geometry and the other geometry.
    
    This function is the same as the C++ method
    OGRGeometry::SymmetricDifference().
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always fail, issuing a
    CPLE_NotSupported error.
    
    Parameters
    
    hThis : the geometry.
    
    hOther : the other geometry.
    
    Returns
    
    a new geometry representing the symmetric difference or NULL
    if the difference is empty or an error occurs.                }
  function OGR_G_SymmetricDifference(hGeom1: TOGRGeometryH; hGeom2: TOGRGeometryH): TOGRGeometryH; cdecl; external external_lib name 'OGR_G_SymmetricDifference';
  { Compute distance between two geometries.
    
    
    
    Returns the shortest distance between the two geometries.
    
    
    
    This function is the same as the C++ method
    OGRGeometry::Distance().
    
    
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always fail, issuing a
    CPLE_NotSupported error.
    
    
    
    Parameters
    
    hFirst : the first geometry to compare against.
    
    hOther : the other geometry to compare against.
    
    
    
    Returns
    
    the distance between the geometries or -1 if an error occurs. }
  function OGR_G_Distance(hGeom1: TOGRGeometryH; hGeom2: TOGRGeometryH): Double; cdecl; external external_lib name 'OGR_G_Distance';

  { Compute geometry area.
    
    
    
    Computes the area for an OGRLinearRing, OGRPolygon or
    OGRMultiPolygon. Undefined for all other geometry types
    (returns zero).
    
    
    
    This function utilizes the C++ get_Area() methods such as
    OGRPolygon::get_Area().
    
    
    
    Parameters
    
    hGeom : the geometry to operate on.
    

    
    Returns
    
    the area or 0.0 for unsupported geometry types.
    
    
    Note
    OGR 1.8.0                                                 }
  function OGR_G_GetArea(hGeom: TOGRGeometryH ): Double;cdecl; external external_lib name 'OGR_G_GetArea';
  { Compute the geometry centroid.
    
    
    
    The centroid location is applied to the passed in OGRPoint
    object. The centroid is not necessarily within the geometry.
    
    
    
    This method relates to the SFCOM ISurface::get_Centroid()
    method however the current implementation based on GEOS can
    operate on other geometry types such as multipoint,
    linestring, geometrycollection such as multipolygons. OGC SF
    SQL 1.1 defines the operation for surfaces (polygons).
    SQL/MM-Part 3 defines the operation for surfaces and
    multisurfaces (multipolygons).
    
    
    
    This function is the same as the C++ method
    OGRGeometry::Centroid().
    
    
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always fail, issuing a
    CPLE_NotSupported error.
    

    
    Returns
    
    OGRERR_NONE on success or OGRERR_FAILURE on error.
    Parameters
    hPolygon :        _nt_
    hCentroidPoint :  _nt_                                        }
  function OGR_G_Centroid(hPolygon: TOGRGeometryH; hCentroidPoint: TOGRGeometryH): Integer;cdecl; external external_lib name 'OGR_G_Centroid';

  { Clear geometry information. This restores the geometry to
    it's initial state after construction, and before assignment
    of actual geometry.
    
    
    
    This function relates to the SFCOM IGeometry::Empty() method.
    
    
    
    This function is the same as the CPP method
    OGRGeometry::empty().
    
    
    
    Parameters
    
    hGeom : handle on the geometry to empty.                      }
  procedure OGR_G_Empty(hGeom: TOGRGeometryH); cdecl; external external_lib name 'OGR_G_Empty';
  { Test if the geometry is empty.
    
    This method is the same as the CPP method
    OGRGeometry::IsEmpty().
    
    Parameters
    
    hGeom : The Geometry to test.
    
    Returns

    TRUE if the geometry has no points, otherwise FALSE.  }
  function OGR_G_IsEmpty (hGeom: TOGRGeometryH): Integer; cdecl; external external_lib name 'OGR_G_IsEmpty';
  { Test if the geometry is valid.
    
    This function is the same as the C++ method
    OGRGeometry::IsValid().
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always return FALSE.
    
    Parameters
    
    hGeom : The Geometry to test.
    
    Returns
    
    TRUE if the geometry has no points, otherwise FALSE.          }
  function OGR_G_IsValid (hGeom: TOGRGeometryH): Integer; cdecl; external external_lib name 'OGR_G_IsValid';
  { Returns TRUE if the geometry is simple.
    
    Returns TRUE if the geometry has no anomalous geometric
    points, such as self intersection or self tangency. The
    \description of each instantiable geometric class will
    include the specific conditions that cause an instance of
    that class to be classified as not simple.
    
    This function is the same as the c++ method
    OGRGeometry::IsSimple() method.
    
    If OGR is built without the GEOS library, this function will
    always return FALSE.
    
    Parameters
    
    hGeom : The Geometry to test.
    
    Returns

    TRUE if object is simple, otherwise FALSE.                   }
  function OGR_G_IsSimple  (hGeom: TOGRGeometryH): Integer; cdecl; external external_lib name 'OGR_G_IsSimple';
  { Test if the geometry is a ring.
    
    This function is the same as the C++ method
    OGRGeometry::IsRing().
    
    This function is built on the GEOS library, check it for the
    definition of the geometry operation. If OGR is built without
    the GEOS library, this function will always return FALSE.
    
    Parameters
    
    hGeom : The Geometry to test.
    
    Returns
    
    TRUE if the geometry has no points, otherwise FALSE.          }
  function OGR_G_IsRing  (hGeom: TOGRGeometryH): Integer; cdecl; external external_lib name 'OGR_G_IsRing';



  function OGR_G_Intersect(hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl; external external_lib name 'OGR_G_Intersect';
  { Returns TRUE if two geometries are equivalent.
    
    
    
    This function is the same as the CPP method
    OGRGeometry::Equals() method.
    
    
    
    Parameters
    
    hGeom : handle on the first geometry.
    
    hOther : handle on the other geometry to test against.
    

    
    Returns
    
    TRUE if equivalent or FALSE otherwise.                 }
  function OGR_G_Equal(hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl; external external_lib name 'OGR_G_Equal';

{ Fetch number of points from a geometry.
  
  Only wkbPoint[25D] or wkbLineString[25D] may return a valid
  value. Other geometry types will silently return 0.
  
  Parameters
  
  hGeom : handle to the geometry from which to get the number
  of points.
  
  
  
  Returns
  
  the number of points.                                       }
  function OGR_G_GetPointCount(hGeom: TOGRGeometryH ): Integer; cdecl; external external_lib name 'OGR_G_GetPointCount';
  { Fetch the x coordinate of a point from a geometry.

    Parameters

    hGeom : handle to the geometry from which to get the x
    coordinate.
    i : point to get the x coordinate.

    Returns
    
    the X coordinate of this point.                        }
  function OGR_G_GetX(hGeom: TOGRGeometryH; i: Integer): Double; cdecl; external external_lib name 'OGR_G_GetX';
  { Fetch the y coordinate of a point from a geometry.
    
    Parameters
    
    hGeom : handle to the geometry from which to get the y
    coordinate.
    
    i : point to get the Y coordinate.
    
    Returns
    
    the Y coordinate of this point.                        }
  function OGR_G_GetY(hGeom: TOGRGeometryH; i: Integer): Double; cdecl; external external_lib name 'OGR_G_GetY';
  { Fetch the z coordinate of a point from a geometry.
    
    Parameters
    
    hGeom : handle to the geometry from which to get the Z
    coordinate.
    
    i : point to get the Z coordinate.
    
    Returns
    
    the Z coordinate of this point.                        }
  function OGR_G_GetZ(hGeom: TOGRGeometryH; i: Integer): Double; cdecl; external external_lib name 'OGR_G_GetZ';
  { Fetch a point in line string or a point geometry.
    
    
    
    Parameters
    
    hGeom: handle to the geometry from which to get the
    coordinates.
    
    i : the vertex to fetch, from 0 to getNumPoints()-1, zero for
    a point.
    
    pdfX : value of x coordinate.
    
    pdfY : value of y coordinate.
    
    pdfZ : value of z coordinate.                                 }
  procedure OGR_G_GetPoint(hGeom: TOGRGeometryH; iPoint: Integer;  var pdfX: double; var pdfY:double;  var pdfZ: double); cdecl; external external_lib name 'OGR_G_GetPoint';
  { et the location of a vertex in a point or linestring
    geometry.
    
    
    
    If iPoint is larger than the number of existing points in the
    linestring, the point count will be increased to accomodate
    the request.
    
    
    
    Parameters
    
    hGeom : handle to the geometry to add a vertex to.
    
    i : the index of the vertex to assign (zero based) or zero
    for a point.
    
    dfX : input X coordinate to assign.
    
    dfY : input Y coordinate to assign.
    
    dfZ : input Z coordinate to assign (defaults to zero).        }
  procedure OGR_G_SetPoint(hGeom: TOGRGeometryH; iPoint: Integer;  dfX, dfY, dfZ: double); cdecl; external external_lib name 'OGR_G_SetPoint';
  { Set the location of a vertex in a point or linestring
    geometry.
    
    
    
    If iPoint is larger than the number of existing points in the
    linestring, the point count will be increased to accomodate
    the request.
    
    
    
    Parameters
    
    hGeom : handle to the geometry to add a vertex to.

    i : the index of the vertex to assign (zero based) or zero
    for a point.
    
    dfX : input X coordinate to assign.
    
    dfY : input Y coordinate to assign.                           }
  procedure OGR_G_SetPoint_2D(hGeom: TOGRGeometryH; iPoint: Integer;  dfX, dfY :double); cdecl; external external_lib name 'OGR_G_SetPoint_2D';
  { Add a point to a geometry (line string or point).
    
    
    
    The vertex count of the line string is increased by one, and
    assigned from the passed location value.
    
    
    
    Parameters
    
    hGeom : handle to the geometry to add a point to.
    
    dfX : x coordinate of point to add.

    dfY : y coordinate of point to add.

    dfZ : z coordinate of point to add.                            }
  procedure OGR_G_AddPoint(hGeom: TOGRGeometryH; dfX, dfY, dfZ: Double ); cdecl; external external_lib name 'OGR_G_AddPoint';
  { Add a point to a geometry (line string or point).
    

    
    The vertex count of the line string is increased by one, and
    assigned from the passed location value.
    
    
    
    Parameters
    
    hGeom : handle to the geometry to add a point to.

    dfX : x coordinate of point to add.
    
    dfY : y coordinate of point to add.                          }
  procedure OGR_G_AddPoint_2D(hGeom: TOGRGeometryH; dfX, dfY: double ); cdecl; external external_lib name 'OGR_G_AddPoint_2D';

{ Methods for getting/setting rings and members collections }

  function OGR_G_GetGeometryCount(hGeom: TOGRGeometryH ): Integer; cdecl; external external_lib name 'OGR_G_GetGeometryCount';
  { Fetch geometry from a geometry container.
    
    This function returns an handle to a geometry within the
    container. The returned geometry remains owned by the
    container, and should not be modified. The handle is only
    valid untill the next change to the geometry container. Use
    OGR_G_Clone() to make a copy.
    
    This function relates to the SFCOM
    IGeometryCollection::get_Geometry() method.
    
    This function is the same as the CPP method
    OGRGeometryCollection::getGeometryRef().
    
    For a polygon, OGR_G_GetGeometryRef(iSubGeom) returns the
    exterior ring if iSubGeom == 0, and the interior rings for
    iSubGeom \> 0.
    
    
    
    Parameters
    
    hGeom : handle to the geometry container from which to get a
    geometry from.
    
    iSubGeom : the index of the geometry to fetch, between 0 and
    getNumGeometries() - 1.
    
    
    
    Returns
    
    handle to the requested geometry.                            }
  function OGR_G_GetGeometryRef(hGeom: TOGRGeometryH; iSubGeom: Integer ): TOGRGeometryH; cdecl; external external_lib name 'OGR_G_GetGeometryRef';
  { Add a geometry to a geometry container.
    
    
    
    Some subclasses of OGRGeometryCollection restrict the types
    of geometry that can be added, and may return an error. The
    passed geometry is cloned to make an internal copy.
    
    
    
    There is no SFCOM analog to this method.
    
    
    
    This function is the same as the CPP method
    OGRGeometryCollection::addGeometry.
    
    
    
    For a polygon, hNewSubGeom must be a linearring. If the
    polygon is empty, the first added subgeometry will be the
    exterior ring. The next ones will be the interior rings.
    
    
    
    Parameters
    
    hGeom : existing geometry container.
    
    hNewSubGeom : geometry to add to the container.
    
    
    
    Returns:
    
    OGRERR_NONE if successful, or
    OGRERR_UNSUPPORTED_GEOMETRY_TYPE if the geometry type is
    illegal for the type of existing geometry.                  }
  function OGR_G_AddGeometry(hGeom: TOGRGeometryH; hNewSubGeom: TOGRGeometryH ): TOGRErr; cdecl; external external_lib name 'OGR_G_AddGeometry';
  { Add a geometry directly to an existing geometry container.
    
    
    
    Some subclasses of OGRGeometryCollection restrict the types
    of geometry that can be added, and may return an error.
    Ownership of the passed geometry is taken by the container
    rather than cloning as addGeometry() does.
    
    
    
    This function is the same as the CPP method
    OGRGeometryCollection::addGeometryDirectly.
    
    
    
    There is no SFCOM analog to this method.
    
    
    
    For a polygon, hNewSubGeom must be a linearring. If the
    polygon is empty, the first added subgeometry will be the
    exterior ring. The next ones will be the interior rings.
    
    
    
    Parameters
    
    hGeom: existing geometry.
    
    hNewSubGeom: geometry to add to the existing geometry.
    
    
    
    Returns:

    OGRERR_NONE if successful, or
    OGRERR_UNSUPPORTED_GEOMETRY_TYPE if the geometry type is
    illegal for the type of geometry container.                 }
  function OGR_G_AddGeometryDirectly(hGeom: TOGRGeometryH; hNewSubGeom: TOGRGeometryH ): TOGRErr; cdecl; external external_lib name 'OGR_G_AddGeometryDirectly';
  { Remove a geometry from an exiting geometry container.
    
    Removing a geometry will cause the geometry count to drop by
    one, and all "higher" geometries will shuffle down one in
    index.
    
    There is no SFCOM analog to this method.
    
    This function is the same as the CPP method
    OGRGeometryCollection::removeGeometry().
    
    Parameters
    
    hGeom : the existing geometry to delete from.
    
    iGeom : the index of the geometry to delete. A value of -1 is
    a special flag meaning that all geometries should be removed.
    
    bDelete : if TRUE the geometry will be destroyed, otherwise
    it will not. The default is TRUE as the existing geometry is
    considered to own the geometries in it.
    
    Returns
    
    OGRERR_NONE if successful, or OGRERR_FAILURE if the index is
    out of range.                                                 }
  function OGR_G_RemoveGeometry(hGeom: TOGRGeometryH; iGeom, bDelete: Integer): TOGRErr; cdecl; external external_lib name 'OGR_G_RemoveGeometry';

  { Build a ring from a bunch of arcs.
    
    
    
    Parameters
    
    hLines : handle to an OGRGeometryCollection (or
    OGRMultiLineString) containing the line string geometries to
    be built into rings.
    
    bBestEffort: not yet implemented???.
    
    bAutoClose: indicates if the ring should be close when first
    and last points of the ring are the same.
    
    dfTolerance: tolerance into which two arcs are considered
    close enough to be joined.
    
    peErr: OGRERR_NONE on success, or OGRERR_FAILURE on failure.
    
    Returns
    
    an handle to the new geometry, a polygon.                    }
  function OGRBuildPolygonFromEdges(hLinesAsCollection: TOGRGeometryH;
                                       bBestEffort: Integer;
                                       bAutoClose: Integer;
                                       dfTolerance: Integer;
                                       Var peErr: TOGRErr ): TOGRGeometryH; cdecl; external external_lib name 'OGRBuildPolygonFromEdges';

  { Special entry point to enable the hack for generating DB2
    V7.2 style WKB.
    
    
    
    DB2 seems to have placed (and require) an extra 0x30 or'ed
    with the byte order in WKB. This entry point is used to turn
    on or off the generation of such WKB.                        }
  function OGRSetGenerate_DB2_V72_BYTE_ORDER(bGenerate_DB2_V72_BYTE_ORDER: Integer): TOGRErr; cdecl; external external_lib name 'OGRSetGenerate_DB2_V72_BYTE_ORDER';

  function OGRGetGenerate_DB2_V72_BYTE_ORDER: Integer; cdecl; external external_lib name 'OGRGetGenerate_DB2_V72_BYTE_ORDER';


  procedure OGRFree(p : POINTER); cdecl; external External_Lib name 'OGRFree';

  { Create a new field definition.

    This function is the same as the CPP method
    OGRFieldDefn::OGRFieldDefn().
    Parameters
     pszName: the name of the new field definition.
     eType: the type of the new field definition.



    Returns
        handle to the new field definition.            }

  function OGR_Fld_Create( pszName: PChar; hType: OGRFieldType ): OGRFieldDefnH; cdecl; external External_Lib name 'OGR_Fld_Create';
  { Destroy a field definition.
    
    Parameters
    hDefn : handle to the field definition to destroy.  }
  procedure OGR_Fld_Destroy(hField: OGRFieldDefnH ); cdecl; external External_Lib name 'OGR_Fld_Destroy';

  { Reset the name of this field.
    
    This function is the same as the CPP method
    OGRFieldDefn::SetName().
    Parameters
    hDefn : handle to the field definition to apply the new name
    to.
    pszName : the new name to apply.                             }
  procedure OGR_Fld_SetName(hField: OGRFieldDefnH; pszName: PChar); cdecl; external External_Lib name 'OGR_Fld_SetName';
  { Fetch name of this field.
    
    This function is the same as the CPP method
    OGRFieldDefn::GetNameRef().
    Parameters
        hDefn : handle to the field definition.
    Returns:
    the name of the field definition.           }
  function OGR_Fld_GetNameRef(hField: OGRFieldDefnH ): PChar; cdecl; external External_Lib name 'OGR_Fld_GetNameRef';
  { Fetch type of this field.
    
    This function is the same as the CPP method
    OGRFieldDefn::GetType().

    Parameters
    hDefn : handle to the field definition to get type from.

    Returns
    field type.                                              }
  function OGR_Fld_GetType(hField: OGRFieldDefnH ): OGRFieldType; cdecl; external External_Lib name 'OGR_Fld_GetType';
  { Set the type of this field. This should never be done to an
    OGRFieldDefn that is already part of an OGRFeatureDefn.
    This function is the same as the CPP method
    OGRFieldDefn::SetType().
    Parameters
    hDefn : handle to the field definition to set type to.
    eType : the new field type.                                 }
  procedure OGR_Fld_SetType(hField: OGRFieldDefnH; hType: OGRFieldType ); cdecl; external External_Lib name 'OGR_Fld_SetType';
  { Get the justification for this field.
    
    This function is the same as the CPP method
    OGRFieldDefn::GetJustify().
    Parameters
      hDefn : handle to the field definition to get justification
    from.                                                       }
  function OGR_Fld_GetJustify(hField: OGRFieldDefnH ): OGRJustification; cdecl; external External_Lib name 'OGR_Fld_GetJustify';
  { Set the justification for this field.
    
    This function is the same as the CPP method
    OGRFieldDefn::SetJustify().
    Parameters
    hDefn :     handle to the field definition to set
                justification to.
    eJustify :  the new justification.                }
  procedure OGR_Fld_SetJustify(hField: OGRFieldDefnH; hJ: OGRJustification ); cdecl; external External_Lib name 'OGR_Fld_SetJustify';
  { Get the formatting width for this field.
    
    This function is the same as the CPP method
    OGRFieldDefn::GetWidth().

    Parameters
     hDefn : handle to the field definition to get width from.

    Returns
    the width, zero means no specified width.                 }
  function OGR_Fld_GetWidth(hField: OGRFieldDefnH ): Integer; cdecl; external External_Lib name 'OGR_Fld_GetWidth';
  { Set the formatting width for this field in characters.
    
    
    
    This function is the same as the CPP method
    OGRFieldDefn::SetWidth().
    
    
    
    Parameters
    
    hDefn : handle to the field definition to set width to.
    
    nNewWidth : the new width.                              }
  procedure OGR_Fld_SetWidth(hField: OGRFieldDefnH; iWidth: Integer ); cdecl; external External_Lib name 'OGR_Fld_SetWidth';
  { Get the formatting precision for this field. This should
    normally be zero for fields of types other than OFTReal.

    This function is the same as the CPP method
    OGRFieldDefn::GetPrecision().

    Parameters

    hDefn : handle to the field definition to get precision from.

    Returns
    the precision.                                              }
  function OGR_Fld_GetPrecision(hField: OGRFieldDefnH ): Integer; cdecl; external External_Lib name 'OGR_Fld_GetPrecision';
  { Set the formatting precision for this field in characters.
    
    This should normally be zero for fields of types other than
    OFTReal.
    This function is the same as the CPP method
    OGRFieldDefn::SetPrecision().

    Parameters
    hDefn: handle to the field definition to set precision to.
    nPrecision: the new precision.                              }
  procedure OGR_Fld_SetPrecision(hField: OGRFieldDefnH; hP: Integer); cdecl; external External_Lib name 'OGR_Fld_SetPrecision';
  { Set defining parameters for a field in one call.
    
    
    
    This function is the same as the CPP method
    OGRFieldDefn::Set().
    
    
    
    Parameters
    hDefn : handle to the field definition to set to.
    pszNameIn : the new name to assign.
    eTypeIn : the new type (one of the OFT values like
    OFTInteger).
    nWidthIn : the preferred formatting width. Defaults to zero
    indicating undefined.
    nPrecisionIn : number of decimals places for formatting,
    defaults to zero indicating undefined.
    eJustifyIn : the formatting justification (OJLeft or
    OJRight), defaults to OJUndefined.                          }
  procedure OGR_Fld_Set(hField: OGRFieldDefnH; pszName: PChar; hType: OGRFieldType;
                            iWidth, iPrecision: Integer; Justification: OGRJustification ); cdecl; external External_Lib name 'OGR_Fld_Set';


  { Fetch human readable name for a field type.
    
    This function is the same as the CPP method
    OGRFieldDefn::GetFieldTypeName().
    
    Parameters
    
    eType : the field type to get name for.
    
    Returns
    
    the name.                                   }
  function OGR_GetFieldTypeName(hType: OGRFieldType ): PChar; cdecl; external External_Lib name 'OGR_GetFieldTypeName';

{ Feature factory.
  
  
  
  \Note that the OGRFeature will increment the reference count
  of it's defining OGRFeatureDefn. Destruction of the
  OGRFeatureDefn before destruction of all OGRFeatures that
  depend on it is likely to result in a crash.
  
  
  
  This function is the same as the C++ method
  OGRFeature::OGRFeature().
  Parameters
  hFd :  handle to the feature class (layer) definition to which
         the feature will adhere.
  
  Returns
  an handle to the new feature object with null fields and no
  geometry.                                                      }

  function OGR_F_Create(hFd: OGRFeatureDefnH ): OGRFeatureH; cdecl; external External_Lib name 'OGR_F_Create';
{ Destroy feature.
  
  The feature is deleted, but within the context of the
  GDAL/OGR heap. This is necessary when higher level
  applications use GDAL/OGR from a DLL and they want to delete
  a feature created within the DLL. If the delete is done in
  the calling application the memory will be freed onto the
  application heap which is inappropriate.
  
  This function is the same as the C++ method
  OGRFeature::DestroyFeature().
  Parameters
  hF :  handle to the feature to destroy.                      }
procedure    OGR_F_Destroy(hF: OGRFeatureH ); cdecl; external External_Lib name 'OGR_F_Destroy';
{ Fetch feature definition.
  
  This function is the same as the C++ method
  OGRFeature::GetDefnRef().
  Parameters
  hF :  handle to the feature to get the feature definition from.

  Returns
  an handle to the feature definition object on which feature
  depends.                                                        }
function  OGR_F_GetDefnRef(hF: OGRFeatureH ): OGRFeatureDefnH; cdecl; external External_Lib name 'OGR_F_GetDefnRef';

{ Set feature geometry.

  This function updates the features geometry, and operate
  exactly as SetGeometry(), except that this function assumes
  ownership of the passed geometry.

  This function is the same as the C++ method
  OGRFeature::SetGeometryDirectly.
  Parameters
    	hFeat :	handle to the feature on which to apply the geometry.
    	hGeom :	handle to the new geometry to apply to feature.
  Returns
  OGRERR_NONE if successful, or OGR_UNSUPPORTED_GEOMETRY_TYPE
  if the geometry type is illegal for the OGRFeatureDefn
  (checking not yet implemented).                             }
function OGR_F_SetGeometryDirectly( hF: OGRFeatureH; hGeom: OGRGeometryH ): OGRErr; cdecl; external External_Lib name 'OGR_F_SetGeometryDirectly';
{ Set feature geometry.
  
  This function updates the features geometry, and operate
  exactly as SetGeometryDirectly(), except that this function
  does not assume ownership of the passed geometry, but instead
  makes a copy of it.
  
  This function is the same as the C++
  OGRFeature::SetGeometry().
  Parameters
  hF :     handle to the feature on which new geometry is applied
           to. 
  hGeom :  handle to the new geometry to apply to feature.
  
  Returns
  OGRERR_NONE if successful, or OGR_UNSUPPORTED_GEOMETRY_TYPE
  if the geometry type is illegal for the OGRFeatureDefn
  (checking not yet implemented).                                 }
function OGR_F_SetGeometry(hF: OGRFeatureH; hGeom: OGRGeometryH ): OGRErr; cdecl; external External_Lib name 'OGR_F_SetGeometry';
{ Fetch an handle to feature geometry.
  
  This function is the same as the C++ method
  OGRFeature::GetGeometryRef().
  Parameters
  hF :  handle to the feature to get geometry from. 
  
  Returns
  an handle to internal feature geometry. This object should
  not be modified.                                           }
function  OGR_F_GetGeometryRef(hF: OGRFeatureH ):OGRGeometryH; cdecl; external External_Lib name 'OGR_F_GetGeometryRef';
{ Duplicate feature.
  
  The newly created feature is owned by the caller, and will
  have it's own reference to the OGRFeatureDefn.
  
  This function is the same as the C++ method
  OGRFeature::Clone().
  
  
  Parameters
  hF :  handle to the feature to clone. 
  
  Returns
  an handle to the new feature, exactly matching this feature.  }
function  OGR_F_Clone(hF: OGRFeatureH ):OGRFeatureH; cdecl; external External_Lib name 'OGR_F_Clone';
{ Test if two features are the same.
  
  Two features are considered equal if the share them (handle
  equality) same OGRFeatureDefn, have the same field values,
  and the same geometry (as tested by OGR_G_Equal()) as well as
  the same feature id.
  
  This function is the same as the C++ method
  OGRFeature::Equal().
  Parameters
  hF1 :  handle to one of the feature. 
  hF2 :  handle to the other feature to test this one against.
  
  Returns
  TRUE if they are equal, otherwise FALSE.                      }
function OGR_F_Equal(hF1: OGRFeatureH; hF2: OGRFeatureH): Integer; cdecl; external External_Lib name 'OGR_F_Equal';

{ Fetch number of fields on this feature This will always be
  the same as the field count for the OGRFeatureDefn.
  
  This function is the same as the C++ method
  OGRFeature::GetFieldCount().
  Parameters
  hF :  handle to the feature to get the fields count from. 
  
  Returns
  count of fields.                                           }
function OGR_F_GetFieldCount(hF: OGRFeatureH ): Integer; cdecl; external External_Lib name 'OGR_F_GetFieldCount';
{ Fetch definition for this field.
  
  This function is the same as the C++ method
  OGRFeature::GetFieldDefnRef().
  Parameters
  hF :      handle to the feature on which the field is found. 
  iField :  the field to fetch, from 0 to GetFieldCount()\-1.
  
  Returns
  an handle to the field definition (from the OGRFeatureDefn).
  This is an internal reference, and should not be deleted or
  modified.                                                     }
function OGR_F_GetFieldDefnRef(hF: OGRFeatureH; iField: Integer): OGRFieldDefnH; cdecl; external External_Lib name 'OGR_F_GetFieldDefnRef';
{ Fetch the field index given field name.

  This is a cover for the OGRFeatureDefn::GetFieldIndex()
  method.
  
  This function is the same as the C++ method
  OGRFeature::GetFieldIndex().
  Parameters
  hF :          handle to the feature on which the field is
                found. 
  pszFldName :  the name of the field to search for.
  
  Returns
  the field index, or -1 if no matching field is found.     }
function OGR_F_GetFieldIndex(hF: OGRFeatureH; pszFldName: PChar): Integer; cdecl; external External_Lib name 'OGR_F_GetFieldIndex';

{ Test if a field has ever been assigned a value or not.
  
  This function is the same as the C++ method
  OGRFeature::IsFieldSet().
  Parameters
  hF :      handle to the feature on which the field is. 
  iField :  the field to test.
  
  Returns
  TRUE if the field has been set, otherwise false.        }
function OGR_F_IsFieldSet(hF: OGRFeatureH; iField: Integer ): Integer; cdecl; external External_Lib name 'OGR_F_IsFieldSet';
{ Clear a field, marking it as unset.
  
  This function is the same as the C++ method
  OGRFeature::UnsetField().

  Parameters
    hFeat : handle to the feature on which the field is.
  
  iField : the field to unset.                         }
procedure OGR_F_UnsetField(hF: OGRFeatureH; iField: Integer); cdecl; external External_Lib name 'OGR_F_UnsetField';
{ Fetch an handle to the internal field value given the index.
  
  This function is the same as the C++ method
  OGRFeature::GetRawFieldRef().
  Returns
  the returned handle is to an internal data structure, and
  should not be freed, or modified.
  Parameters
  hF :      handle to the feature on which field is found. 
  iField :  the field to fetch, from 0 to GetFieldCount()\-1.  }
function OGR_F_GetRawFieldRef(hF: OGRFeatureH; iField: Integer): OGRField; cdecl; external External_Lib name 'OGR_F_GetRawFieldRef';

{ Fetch field value as integer.
  
  OFTString features will be translated using atoi(). OFTReal
  fields will be cast to integer. Other field types, or errors
  will result in a return value of zero.
  
  This function is the same as the C++ method
  OGRFeature::GetFieldAsInteger().
  Parameters
  hF :      handle to the feature that owned the field. 
  iField :  the field to fetch, from 0 to GetFieldCount()\-1.
  
  Returns
  the field value.                                             }
function OGR_F_GetFieldAsInteger(hF: OGRFeatureH; iField: Integer): Integer; cdecl; external External_Lib name 'OGR_F_GetFieldAsInteger';
{ Fetch field value as a double.
  
  OFTString features will be translated using atof().
  OFTInteger fields will be cast to double. Other field types,
  or errors will result in a return value of zero.
  
  This function is the same as the C++ method
  OGRFeature::GetFieldAsDouble().
  Parameters
  hF :      handle to the feature that owned the field. 
  iField :  the field to fetch, from 0 to GetFieldCount()\-1.
  
  Returns
  the field value.                                             }
function OGR_F_GetFieldAsDouble(hF: OGRFeatureH; iField: Integer): Double; cdecl; external External_Lib name 'OGR_F_GetFieldAsDouble';
{ Fetch field value as a string.
  
  OFTReal and OFTInteger fields will be translated to string
  using sprintf(), but not necessarily using the established
  formatting rules. Other field types, or errors will result in
  a return value of zero.
  
  This function is the same as the C++ method
  OGRFeature::GetFieldAsString().
  Parameters
  hF :      handle to the feature that owned the field. 
  iField :  the field to fetch, from 0 to GetFieldCount()\-1.
  
  Returns
  the field value. This string is internal, and should not be
  modified, or freed. Its lifetime may be very brief.           }
function OGR_F_GetFieldAsString(hF: OGRFeatureH; iField: Integer): PChar; cdecl; external External_Lib name 'OGR_F_GetFieldAsString';
{ Fetch field value as a list of integers.
  
  Currently this function only works for OFTIntegerList fields.
  
  This function is the same as the C++ method
  OGRFeature::GetFieldAsIntegerList().
  Parameters
  hF :       handle to the feature that owned the field.
  iField :   the field to fetch, from 0 to GetFieldCount()\-1.
  pnCount :  an integer to put the list count (number of integers)
             into.
  Returns
  the field value. This list is internal, and should not be
  modified, or freed. Its lifetime may be very brief. If
  \*pnCount is zero on return the returned pointer may be NULL
  or non-NULL.                                                     }
function OGR_F_GetFieldAsIntegerList(hF: OGRFeatureH; iField: Integer; var pnCount: Integer): PIntegerArray; cdecl; external External_Lib name 'OGR_F_GetFieldAsIntegerList';
{ Fetch field value as a list of doubles.
  
  Currently this function only works for OFTRealList fields.
  
  This function is the same as the C++ method
  OGRFeature::GetFieldAsDoubleList().
  Returns
  the field value. This list is internal, and should not be
  modified, or freed. Its lifetime may be very brief.
  Parameters
  hF :       handle to the feature that owned the field. 
  iField :   the field to fetch, from 0 to GetFieldCount()\-1. 
  pnCount :  an integer to put the list count (number of doubles)
             into.                                                }
function OGR_F_GetFieldAsDoubleList(hF: OGRFeatureH; iField: Integer; var pnCount: Integer): PDoubleArray; cdecl; external External_Lib name 'OGR_F_GetFieldAsDoubleList';
{ Fetch field value as a list of strings.
  
  Currently this method only works for OFTStringList fields.
  
  The returned list is terminated by a NULL pointer. The number
  of elements can also be calculated using CSLCount().
  
  This function is the same as the C++ method
  OGRFeature::GetFieldAsStringList().
  Parameters
  hF :      handle to the feature that owned the field. 
  iField :  the field to fetch, from 0 to GetFieldCount()\-1.
  
  Returns
  \ \                                                           }
function OGR_F_GetFieldAsStringList(hF: OGRFeatureH; iField: Integer): PPChar; cdecl; external External_Lib name 'OGR_F_GetFieldAsStringList';
{ Fetch field value as binary.
  
  Currently this method only works for OFTBinary fields.
  
  This function is the same as the C++ method
  OGRFeature::GetFieldAsBinary().
  Parameters
  hF :       handle to the feature that owned the field. 
  iField :   the field to fetch, from 0 to GetFieldCount()\-1. 
  pnBytes :  location to place count of bytes returned.
  
  Returns
  the field value. This list is internal, and should not be
  modified, or freed. Its lifetime may be very brief.           }
function OGR_F_GetFieldAsBinary(hF: OGRFeatureH; iField: Integer; var pnBytes: Integer): PByteArray; cdecl; external External_Lib name 'OGR_F_GetFieldAsBinary';
{ Fetch field value as date and time.
  
  Currently this method only works for OFTDate, OFTTime and
  OFTDateTime fields.
  
  This function is the same as the C++ method
  OGRFeature::GetFieldAsDateTime().
  Parameters
  hF :        handle to the feature that owned the field. 
  iField :    the field to fetch, from 0 to
              GetFieldCount()\-1. 
  pnYear :    (including century) 
  pnMonth :   (1\-12) 
  pnDay :     (1\-31)
  pnHour :    (0\-23)
  pnMinute :  (0\-59)
  pnSecond :  (0\-59)
  pnTZFlag :  (0=unknown, 1=localtime, 100=GMT, see data
              model for details)
  
  Returns
  TRUE on success or FALSE on failure.                      }
function OGR_F_GetFieldAsDateTime(hF: OGRFeatureH; iField: Integer; var pnYear, pnMonth, pnDay, pnHour, pnMinute, pnSecond, pnTZFlag: Integer): Integer; cdecl; external External_Lib name 'OGR_F_GetFieldAsDateTime';

{ Set field to integer value.
  
  OFTInteger and OFTReal fields will be set directly. OFTString
  fields will be assigned a string representation of the value,
  but not necessarily taking into account formatting
  constraints on this field. Other field types may be
  unaffected.
  
  This function is the same as the C++ method
  OGRFeature::SetField().
  Parameters
  hF :      handle to the feature that owned the field. 
  iField :  the field to fetch, from 0 to GetFieldCount()\-1. 
  Value :   the value to assign.                                }
procedure    OGR_F_SetFieldInteger( hF: OGRFeatureH; iField: Integer; Value: Integer ); cdecl; external External_Lib name 'OGR_F_SetFieldInteger';
{ Set field to double value.
  
  OFTInteger and OFTReal fields will be set directly. OFTString
  fields will be assigned a string representation of the value,
  but not necessarily taking into account formatting
  constraints on this field. Other field types may be
  unaffected.
  
  This function is the same as the C++ method
  OGRFeature::SetField().
  Parameters
  hF :      handle to the feature that owned the field. 
  iField :  the field to fetch, from 0 to GetFieldCount()\-1. 
  Value :   the value to assign.                                }
procedure    OGR_F_SetFieldDouble( hF: OGRFeatureH; iField: Integer; Value: double ); cdecl; external External_Lib name 'OGR_F_SetFieldDouble';
{ Set field to string value.
  
  OFTInteger fields will be set based on an atoi() conversion
  of the string. OFTReal fields will be set based on an atof()
  conversion of the string. Other field types may be
  unaffected.
  
  This function is the same as the C++ method
  OGRFeature::SetField().
  Parameters
  hF :      handle to the feature that owned the field. 
  iField :  the field to fetch, from 0 to GetFieldCount()\-1. 
  Value :   the value to assign.                               }
procedure    OGR_F_SetFieldString( hF: OGRFeatureH; iField: Integer; Value: PChar ); cdecl; external External_Lib name 'OGR_F_SetFieldString';
{ Set field to list of integers value.
  
  This function currently on has an effect of OFTIntegerList
  fields.
  
  This function is the same as the C++ method
  OGRFeature::SetField().
  Parameters
  hF :         handle to the feature that owned the field.
  iField :     the field to set, from 0 to GetFieldCount()\-1. 
  nCount :     the number of values in the list being assigned. 
  panValues :  the values to assign.                             }
procedure    OGR_F_SetFieldIntegerList( hF: OGRFeatureH; iField: Integer; nCount: Integer; panValues: PIntegerArray ); cdecl; external External_Lib name 'OGR_F_SetFieldIntegerList';
{ Set field to list of doubles value.
  
  This function currently on has an effect of OFTRealList
  fields.
  
  This function is the same as the C++ method
  OGRFeature::SetField().
  Parameters
  hF :         handle to the feature that owned the field. 
  iField :     the field to set, from 0 to GetFieldCount()\-1. 
  nCount :     the number of values in the list being assigned. 
  panValues :  the values to assign.                             }
procedure    OGR_F_SetFieldDoubleList( hF: OGRFeatureH; iField: Integer; nCount: Integer; panValues: PDoubleArray );cdecl; external External_Lib name 'OGR_F_SetFieldDoubleList';
{ Set field to list of strings value.
  
  This function currently on has an effect of OFTStringList
  fields.
  
  This function is the same as the C++ method
  OGRFeature::SetField().
  Parameters
  hF :         handle to the feature that owned the field. 
  iField :     the field to set, from 0 to GetFieldCount()\-1. 
  panValues :  the values to assign.                            }
procedure    OGR_F_SetFieldStringList( hF: OGRFeatureH; iField: Integer; panValues: PPChar );cdecl; external External_Lib name 'OGR_F_SetFieldStringList';
{ Set field.
  
  The passed value OGRField must be of exactly the same type as
  the target field, or an application crash may occur. The
  passed value is copied, and will not be affected. It remains
  the responsibility of the caller.
  
  This function is the same as the C++ method
  OGRFeature::SetField().
  Parameters
  hF :      handle to the feature that owned the field. 
  iField :  the field to fetch, from 0 to GetFieldCount()\-1. 
  hField :  handle on the value to assign.                      }
procedure    OGR_F_SetFieldRaw( hF: OGRFeatureH; iField: Integer; hField: OGRField); cdecl; external External_Lib name 'OGR_F_SetFieldRaw';
{ Set field to binary data.
  
  This function currently on has an effect of OFTBinary fields.
  
  This function is the same as the C++ method
  OGRFeature::SetField().
  Parameters
  hF :         handle to the feature that owned the field. 
  iField :     the field to set, from 0 to GetFieldCount()\-1. 
  nBytes :     the number of bytes in pabyData array. 
  panValues :  the data to apply.                               }
procedure    OGR_F_SetFieldBinary( hF: OGRFeatureH; iField: Integer; nBytes: Integer; panValues: PByteArray ); cdecl; external External_Lib name 'OGR_F_SetFieldBinary';
{ Set field to datetime.
  
  This method currently only has an effect for OFTDate, OFTTime
  and OFTDateTime fields.
  Parameters
  hF :        handle to the feature that owned the field. 
  iField :    the field to set, from 0 to
              GetFieldCount()\-1. 
  pnYear :    (including century) 
  pnMonth :   (1\-12) 
  pnDay :     (1\-31)
  pnHour :    (0\-23)
  pnMinute :  (0\-59)
  pnSecond :  (0\-59)
  pnTZFlag :  (0=unknown, 1=localtime, 100=GMT, see data
              model for details)                                }
procedure    OGR_F_SetFieldDateTime( hF: OGRFeatureH; iField: Integer;
                                       pnYear, pnMonth, pnDay, pnHour, pnMinute, pnSecond, pnTZFlag: Integer ); cdecl; external External_Lib name 'OGR_F_SetFieldDateTime';

{ Get feature identifier.
  
  This function is the same as the C++ method
  OGRFeature::GetFID().
  Parameters
  hF :  handle to the feature from which to get the feature
        identifier. 
  
  Returns
  feature id or OGRNullFID if none has been assigned.       }
function OGR_F_GetFID(hF: OGRFeatureH ): Cardinal; cdecl; external External_Lib name 'OGR_F_GetFID';
{ Set the feature identifier.
  
  For specific types of features this operation may fail on
  illegal features ids. Generally it always succeeds. Feature
  ids should be greater than or equal to zero, with the
  exception of OGRNullFID (-1) indicating that the feature id
  is unknown.
  
  This function is the same as the C++ method
  OGRFeature::SetFID().
  Parameters
  hF :   handle to the feature to set the feature id to. 
  fID :  the new feature identifier value to assign.
  
  Returns
  On success OGRERR_NONE, or on failure some other value.     }
function  OGR_F_SetFID(hF: OGRFeatureH; fID: Cardinal): OGRErr; cdecl; external External_Lib name 'OGR_F_SetFID';
{ Dump this feature in a human readable form.
  
  This dumps the attributes, and geometry; however, it doesn't
  definition information (other than field types and names),
  nor does it report the geometry spatial reference system.
  
  This function is the same as the C++ method
  OGRFeature::DumpReadable().
  Parameters
  hF :   handle to the feature to dump. 
  pfp :  the stream to write to, such as strout.               }
procedure    OGR_F_DumpReadable(hF: OGRFeatureH; var pfp: File); cdecl; external External_Lib name 'OGR_F_DumpReadable';
{ Set one feature from another.
  
  Overwrite the contents of this feature from the geometry and
  attributes of another. The hOtherFeature does not need to
  have the same OGRFeatureDefn. Field values are copied by
  corresponding field names. Field types do not have to exactly
  match. OGR_F_SetField*() function conversion rules will be
  applied as needed.
  
  This function is the same as the C++ method
  OGRFeature::SetFrom().
  Parameters
  hFeat :       handle to the feature to set to. 
  hOtherFeat :  handle to the feature from which geometry, and
                field values will be copied.
  bForgiving :  TRUE if the operation should continue despite
                lacking output fields matching some of the
                source fields.
  
  Returns
  OGRERR_NONE if the operation succeeds, even if some values
  are not transferred, otherwise an error code.                 }
function  OGR_F_SetFrom(hFeat, hOtherFeat: OGRFeatureH; bForgiving: Integer):OGRErr; cdecl; external External_Lib name 'OGR_F_SetFrom';

{ Fetch style string for this feature.
  
  Set the OGR Feature Style Specification for details on the
  format of this string, and ogr_featurestyle.h for services
  available to parse it.
  
  This function is the same as the C++ method
  OGRFeature::GetStyleString().
  Returns
  a reference to a representation in string format, or NULL if
  there isn't one.
  Parameters
  hF :  handle to the feature to get the style from.           }
function OGR_F_GetStyleString(hF:  OGRFeatureH ): PChar; cdecl; external External_Lib name 'OGR_F_GetStyleString';
{ Set feature style string. This method operate exactly as
  OGR_F_SetStyleStringDirectly() except that it does not assume
  ownership of the passed string, but instead makes a copy of
  it.
  
  This function is the same as the C++ method
  OGRFeature::SetStyleString().
  
  
  
  Parameters
    hFeat : handle to the feature to set style to.
    pszStyle : the style string to apply to this feature, cannot  be NULL.                                                      }
procedure    OGR_F_SetStyleString(hF: OGRFeatureH; pszStyle: PChar); cdecl; external External_Lib name 'OGR_F_SetStyleString';
{ Set feature style string. This method operate exactly as
  OGR_F_SetStyleString() except that it assumes ownership of
  the passed string.
  
  
  
  This function is the same as the C++ method
  OGRFeature::SetStyleStringDirectly().
  
  
  
  Parameters
    hFeat : handle to the feature to set style to.
  
  pszStyle : the style string to apply to this feature, cannot
  be NULL.                                                     }
procedure    OGR_F_SetStyleStringDirectly(hF: OGRFeatureH; pszStyle: PChar); cdecl; external External_Lib name 'OGR_F_SetStyleStringDirectly';


{ This function returns the current spatial filter for this
  layer.
  
  The returned pointer is to an internally owned object, and
  should not be altered or deleted by the caller.
  
  This function is the same as the C++ method
  OGRLayer::GetSpatialFilter().
  
  Parameters
  
  hLayer : handle to the layer to get the spatial filter from.
  
  Returns
  
  an handle to the spatial filter geometry.                    }
function  OGR_L_GetSpatialFilter(hLayer: OGRLayerH ): OGRGeometryH; cdecl; external External_Lib name 'OGR_L_GetSpatialFilter';
{ Set a new spatial filter.
  
  This function set the geometry to be used as a spatial filter
  when fetching features via the OGR_L_GetNextFeature()
  function. Only features that geometrically intersect the
  filter geometry will be returned.
  
  Currently this test is may be inaccurately implemented, but
  it is guaranteed that all features who's envelope (as
  returned by OGR_G_GetEnvelope()) overlaps the envelope of the
  spatial filter will be returned. This can result in more
  shapes being returned that should strictly be the case.
  
  This function makes an internal copy of the passed geometry.
  The passed geometry remains the responsibility of the caller,
  and may be safely destroyed.
  
  For the time being the passed filter geometry should be in
  the same SRS as the layer (as returned by
  OGR_L_GetSpatialRef()). In the future this may be
  generalized.
  
  
  
  This function is the same as the C++ method
  OGRLayer::SetSpatialFilter.
  
  
  
  Parameters
  
  hLayer : handle to the layer on which to set the spatial
  filter.
  
  hGeom : handle to the geometry to use as a filtering region.
  NULL may be passed indicating that the current spatial filter
  should be cleared, but no new one instituted.                 }
procedure    OGR_L_SetSpatialFilter(hLayer: OGRLayerH; hGeom: OGRGeometryH ); cdecl; external External_Lib name 'OGR_L_SetSpatialFilter';
{ Set a new rectangular spatial filter.
  
  This method set rectangle to be used as a spatial filter when
  fetching features via the OGR_L_GetNextFeature() method. Only
  features that geometrically intersect the given rectangle
  will be returned.
  
  The x/y values should be in the same coordinate system as the
  layer as a whole (as returned by OGRLayer::GetSpatialRef()).
  Internally this method is normally implemented as creating a
  5 vertex closed rectangular polygon and passing it to
  OGRLayer::SetSpatialFilter(). It exists as a convenience.
  
  The only way to clear a spatial filter set with this method
  is to call OGRLayer::SetSpatialFilter(NULL).
  
  This method is the same as the C++ method
  OGRLayer::SetSpatialFilterRect().
  
  
  
  Parameters
  
  hLayer: handle to the layer on which to set the spatial
  filter.
  
  dfMinX : the minimum X coordinate for the rectangular region.
  
  dfMinY : the minimum Y coordinate for the rectangular region.
  
  dfMaxX: the maximum X coordinate for the rectangular region.
  
  dfMaxY: the maximum Y coordinate for the rectangular region.  }
procedure    OGR_L_SetSpatialFilterRect(hLayer: OGRLayerH;
                                           dfMinX, dfMinY, dfMaxX, fdMaxY: Double ); cdecl; external External_Lib name 'OGR_L_SetSpatialFilterRect';
{ Set a new attribute query.
  
  This function sets the attribute query string to be used when
  fetching features via the OGR_L_GetNextFeature() function.
  Only features for which the query evaluates as true will be
  returned.
  
  The query string should be in the format of an SQL WHERE
  clause. For instance "population \> 1000000 and population \<
  5000000" where population is an attribute in the layer. The
  query format is a restricted form of SQL WHERE clause as
  defined "eq_format=restricted_where" about half way through
  this document:
  
  
  
  <link http://ogdi.sourceforge.net/prop/6.2.CapabilitiesMetadata.html>
  
  
  
  \Note that installing a query string will generally result in
  resetting the current reading position (ala
  OGR_L_ResetReading()).
  
  This function is the same as the C++ method
  OGRLayer::SetAttributeFilter().
  
  
  
  Parameters
  
  hLayer : handle to the layer on which attribute query will be
  executed.
  
  pszQuery : query in restricted SQL WHERE format, or NULL to
  clear the current query.

  
  
  Returns
  
  OGRERR_NONE if successfully installed, or an error code if
  the query expression is in error, or some other failure
  occurs.                                                               }
function OGR_L_SetAttributeFilter(hLayer: OGRLayerH; pszName: PChar ):OGRErr; cdecl; external External_Lib name 'OGR_L_SetAttributeFilter';
{ Reset feature reading to start on the first feature.
  
  This affects GetNextFeature().
  
  This function is the same as the C++ method
  OGRLayer::ResetReading().
  
  Parameters
  
  hLayer : handle to the layer on which features are read.  }
procedure OGR_L_ResetReading(hLayer: OGRLayerH ); cdecl; external External_Lib name 'OGR_L_ResetReading';
{ Fetch the next available feature from this layer.
  
  The returned feature becomes the responsiblity of the caller
  to delete with OGR_F_Destroy(). It is critical that all
  features associated with an OGRLayer (more specifically an
  OGRFeatureDefn) be deleted before that layer/datasource is
  deleted.
  
  Only features matching the current spatial filter (set with
  SetSpatialFilter()) will be returned.
  
  This function implements sequential access to the features of
  a layer. The OGR_L_ResetReading() function can be used to
  start at the beginning again.
  
  This function is the same as the C++ method
  OGRLayer::GetNextFeature().
  
  Parameters

  hLayer : handle to the layer from which feature are read.
  
  Returns
  
  an handle to a feature, or NULL if no more features are
  available.                                                    }
function OGR_L_GetNextFeature(hLayer: OGRLayerH ): OGRFeatureH; cdecl; external External_Lib name 'OGR_L_GetNextFeature';
{ Move read cursor to the nIndex'th feature in the current
  resultset.
  
  This method allows positioning of a layer such that the
  GetNextFeature() call will read the requested feature, where
  nIndex is an absolute index into the current result set. So,
  setting it to 3 would mean the next feature read with
  GetNextFeature() would have been the 4th feature to have been
  read if sequential reading took place from the beginning of
  the layer, including accounting for spatial and attribute
  filters.
  
  Only in rare circumstances is SetNextByIndex() efficiently
  implemented. In all other cases the default implementation
  which calls ResetReading() and then calls GetNextFeature()
  nIndex times is used. To determine if fast seeking is
  available on the current layer use the TestCapability()
  method with a value of OLCFastSetNextByIndex.
  
  This method is the same as the C++ method
  OGRLayer::SetNextByIndex()
  
  
  
  Parameters
  
  hLayer : handle to the layer
  
  nIndex : the index indicating how many steps into the result
  set to seek.
  

  
  Returns
  
  OGRERR_NONE on success or an error code.                      }
function OGR_L_SetNextByIndex(hLayer: OGRLayerH;  nIndex: long ):OGRErr; cdecl; external External_Lib name 'OGR_L_SetNextByIndex';
{ Fetch a feature by its identifier.
  
  This function will attempt to read the identified feature.
  The nFID value cannot be OGRNullFID. Success or failure of
  this operation is unaffected by the spatial or attribute
  filters.
  
  If this function returns a non-NULL feature, it is guaranteed
  that its feature id (OGR_F_GetFID()) will be the same as
  nFID.
  
  Use OGR_L_TestCapability(OLCRandomRead) to establish if this
  layer supports efficient random access reading via
  OGR_L_GetFeature(); however, the call should always work if
  the feature exists as a fallback implementation just scans
  all the features in the layer looking for the desired
  feature.
  
  Sequential reads are generally considered interrupted by a
  OGR_L_GetFeature() call.
  
  The returned feature should be free with OGR_F_Destroy().
  
  This function is the same as the C++ method
  OGRLayer::GetFeature( ).
  
  
  
  Parameters
  
  hLayer : handle to the layer that owned the feature.
  
  nFeatureId : the feature id of the feature to read.

  Returns
  
  an handle to a feature now owned by the caller, or NULL on
  failure.                                                      }
function OGR_L_GetFeature(hLayer: OGRLayerH; nFID: long ): OGRFeatureH; cdecl; external External_Lib name 'OGR_L_GetFeature';
{ Rewrite an existing feature.
  
  This function will write a feature to the layer, based on the
  feature id within the OGRFeature.
  
  Use OGR_L_TestCapability(OLCRandomWrite) to establish if this
  layer supports random access writing via OGR_L_SetFeature().
  
  This function is the same as the C++ method
  OGRLayer::SetFeature().
  
  
  
  Parameters
  
  hLayer : handle to the layer to write the feature.
  
  hFeat : the feature to write.
  
  Returns
  
  OGRERR_NONE if the operation works, otherwise an appropriate
  error code.                                                   }
function OGR_L_SetFeature(hLayer: OGRLayerH; hF: OGRFeatureH ): OGRErr; cdecl; external External_Lib name 'OGR_L_SetFeature';
{ Create and write a new feature within a layer.
  
  The passed feature is written to the layer as a new feature,
  rather than overwriting an existing one. If the feature has a
  feature id other than OGRNullFID, then the native
  implementation may use that as the feature id of the new
  feature, but not necessarily. Upon successful return the
  passed feature will have been updated with the new feature
  id.

  This function is the same as the C++ method
  OGRLayer::CreateFeature().
  
  Parameters
  
  hLayer : handle to the layer to write the feature to.
  
  hFeat : the handle of the feature to write to disk.
  
  Returns
  
  OGRERR_NONE on success.                                       }
function OGR_L_CreateFeature(hLayer: OGRLayerH; hF:  OGRFeatureH ): OGRErr; cdecl; external External_Lib name 'OGR_L_CreateFeature';
{ Delete feature from layer.
  
  The feature with the indicated feature id is deleted from the
  layer if supported by the driver. Most drivers do not support
  feature deletion, and will return
  OGRERR_UNSUPPORTED_OPERATION. The OGR_L_TestCapability()
  function may be called with OLCDeleteFeature to check if the
  driver supports feature deletion.
  
  This method is the same as the C++ method
  OGRLayer::DeleteFeature().
  
  Parameters
  
  hLayer : handle to the layer
  
  nFID : the feature id to be deleted from the layer
  
  Returns
  
  OGRERR_NONE on success.                                       }
function OGR_L_DeleteFeature(hLayer: OGRLayerH;  nFID: long): OGRErr; cdecl; external External_Lib name 'OGR_L_DeleteFeature';
{ Fetch the schema information for this layer.
  
  The returned handle to the OGRFeatureDefn is owned by the
  OGRLayer, and should not be modified or freed by the
  application. It encapsulates the attribute schema of the
  features of the layer.
  
  This function is the same as the C++ method
  OGRLayer::GetLayerDefn().
  
  Parameters
  
  hLayer : handle to the layer to get the schema information.
  
  Returns
  
  an handle to the feature definition.                        }
function OGR_L_GetLayerDefn(hLayer: OGRLayerH):OGRFeatureDefnH; cdecl; external External_Lib name 'OGR_L_GetLayerDefn';
{ Fetch the spatial reference system for this layer.
  
  The returned object is owned by the OGRLayer and should not
  be modified or freed by the application.
  
  This function is the same as the C++ method
  OGRLayer::GetSpatialRef().
  
  Parameters
  
  hLayer : handle to the layer to get the spatial reference
  from.
  
  Returns
  
  spatial reference, or NULL if there isn't one.              }
function  OGR_L_GetSpatialRef(hLayer: OGRLayerH): OGRSpatialReferenceH; cdecl; external External_Lib name 'OGR_L_GetSpatialRef';
{ Fetch the feature count in this layer.
  
  Returns the number of features in the layer. For dynamic
  databases the count may not be exact. If bForce is FALSE, and
  it would be expensive to establish the feature count a value
  of -1 may be returned indicating that the count isn't know.
  If bForce is TRUE some implementations will actually scan the
  entire layer once to count objects.
  
  The returned count takes the spatial filter into account.
  
  \Note that some implementations of this method may alter the
  read cursor of the layer.
  
  This function is the same as the CPP
  OGRLayer::GetFeatureCount().
  
  
  
  Parameters
  
  hLayer : handle to the layer that owned the features.
  
  bForce : Flag indicating whether the count should be computed
  even if it is expensive.
  
  Returns
  
  feature count, -1 if count not known.                         }
function OGR_L_GetFeatureCount(hLayer: OGRLayerH; bForce: Integer ): Integer;cdecl; external External_Lib name 'OGR_L_GetFeatureCount';
{ Fetch the extent of this layer.
  
  Return the extent (MBR) of the data in the layer. If <b>bForce</b>
  is FALSE, and it would be expensive to establish the extent
  then OGRERR_FAILURE will be returned indicating that the
  extent isn't know. If bForce is TRUE then some
  implementations will actually scan the entire layer once to
  compute the MBR of all the features in the layer.
  
  Depending on the drivers, the returned extent may or may not
  take the spatial filter into account. So it is safer to call
  OGR_L_GetExtent() without setting a spatial filter.
  
  Layers without any geometry may return OGRERR_FAILURE just
  indicating that no meaningful extents could be collected.
  
  This function is the same as the C++ method
  OGRLayer::GetExtent().
  Parameters
  hLayer :    handle to the layer from which to get extent.<p />
  psExtent :  the structure in which the extent value will be
              returned.<p />
  bForce :    Flag indicating whether the extent should be
              computed even if it is expensive.<p />
  Returns
  OGRERR_NONE on success, OGRERR_FAILURE if extent not known.
  Note
  \Note that some implementations of this method may alter the
  read cursor of the layer.                                          }
function OGR_L_GetExtent(hLayer: OGRLayerH;  var hEnv: OGREnvelope; bForce: Integer):OGRErr; cdecl; external External_Lib name 'OGR_L_GetExtent';
{ Test if this layer supported the named capability.
  
  The capability codes that can be tested are represented as
  strings, but #defined constants exists to ensure correct
  spelling. Specific layer types may implement class specific
  capabilities, but this can't generally be discovered by the
  caller.
  
    * OLCRandomRead / "RandomRead": TRUE if the GetFeature()
      method is implemented in an optimized way for this layer, as
      opposed to the default implementation using ResetReading()
      and GetNextFeature() to find the requested feature id.
  
    * OLCSequentialWrite / "SequentialWrite": TRUE if the
      CreateFeature() method works for this layer. Note this means
      that this particular layer is writable. The same OGRLayer
      class may returned FALSE for other layer instances that are
      effectively read&#45;only.
  
    * OLCRandomWrite / "RandomWrite": TRUE if the SetFeature()
      method is operational on this layer. Note this means that
      this particular layer is writable. The same OGRLayer class
      may returned FALSE for other layer instances that are
      effectively read&#45;only.
  
    * OLCFastSpatialFilter / "FastSpatialFilter": TRUE if this
      layer implements spatial filtering efficiently. Layers that
      effectively read all features, and test them with the
      OGRFeature intersection methods should return FALSE. This can
      be used as a clue by the application whether it should build
      and maintain its own spatial index for features in this
      layer.
  
    * OLCFastFeatureCount / "FastFeatureCount": TRUE if this
      layer can return a feature count (via
      OGR_L_GetFeatureCount()) efficiently ... ie. without counting
      the features. In some cases this will return TRUE until a
      spatial filter is installed after which it will return FALSE.
  
    * OLCFastGetExtent / "FastGetExtent": TRUE if this layer
      can return its data extent (via OGR_L_GetExtent())
      efficiently ... ie. without scanning all the features. In
      some cases this will return TRUE until a spatial filter is
      installed after which it will return FALSE.
  
    * \  OLCFastSetNextByIndex / "FastSetNextByIndex": TRUE if
      this layer can perform the SetNextByIndex() call efficiently,
      otherwise FALSE.
  
    * OLCCreateField / "CreateField": TRUE if this layer can
      create new fields on the current layer using CreateField(),
      otherwise FALSE.
  
    * OLCDeleteFeature / "DeleteFeature": TRUE if the
      DeleteFeature() method is supported on this layer, otherwise
      FALSE.
  
    * OLCStringsAsUTF8 / "StringsAsUTF8": TRUE if values of
      OFTString fields are assured to be in UTF&#45;8 format. If
      FALSE the encoding of fields is uncertain, though it might
      still be UTF&#45;8.
  
    * OLCTransactions / "Transactions": TRUE if the
      StartTransaction(), CommitTransaction() and
      RollbackTransaction() methods work in a meaningful way,
      otherwise FALSE.
  
  This function is the same as the C++ method
  OGRLayer::TestCapability().
  
  
  
  Parameters
  
  hLayer : handle to the layer to get the capability from.
  
  pszCap : the name of the capability to test.
  
  
  
  Returns
  
  TRUE if the layer has the requested capability, or FALSE
  otherwise. OGRLayers will return FALSE for any unrecognised
  capabilities.                                                     }
function OGR_L_TestCapability(hLayer: OGRLayerH;  pszName: PChar ): Integer; cdecl; external External_Lib name 'OGR_L_TestCapability';
{ Create a new field on a layer.
  
  You must use this to create new fields on a real layer.
  Internally the OGRFeatureDefn for the layer will be updated
  to reflect the new field. Applications should never modify
  the OGRFeatureDefn used by a layer directly.
  
  This function is the same as the C++ method
  OGRLayer::CreateField().
  
  Parameters
  
  hLayer : handle to the layer to write the field definition.
  
  hField : handle of the field definition to write to disk.
  
  bApproxOK : If TRUE, the field may be created in a slightly
  different form depending on the limitations of the format
  driver.
  
  Returns
  
  OGRERR_NONE on success.                                     }
function OGR_L_CreateField(hLayer: OGRLayerH; hField: OGRFieldDefnH; bApproxOk: Integer):OGRErr; cdecl; external External_Lib name 'OGR_L_CreateField';
{ For datasources which support transactions, StartTransaction
  creates a transaction.
  
  If starting the transaction fails, will return
  OGRERR_FAILURE. Datasources which do not support transactions
  will always return OGRERR_NONE.
  
  This function is the same as the C++ method
  OGRLayer::StartTransaction().
  
  Parameters
  
  hLayer: handle to the layer
  
  Returns
  
  OGRERR_NONE on success.                                       }
function OGR_L_StartTransaction(hLayer: OGRLayerH): OGRErr; cdecl; external External_Lib name 'OGR_L_StartTransaction';
{ For datasources which support transactions, CommitTransaction
  commits a transaction.
  
  If no transaction is active, or the commit fails, will return
  OGRERR_FAILURE. Datasources which do not support transactions
  will always return OGRERR_NONE.
  
  This function is the same as the C++ method
  OGRLayer::CommitTransaction().
  
  Parameters
  
  hLayer: handle to the layer
  
  Returns
  
  OGRERR_NONE on success.                                       }
function OGR_L_CommitTransaction(hLayer: OGRLayerH): OGRErr; cdecl; external External_Lib name 'OGR_L_CommitTransaction';
{ For datasources which support transactions,
  RollbackTransaction will roll back a datasource to its state
  before the start of the current transaction. If no
  transaction is active, or the rollback fails, will return
  OGRERR_FAILURE. Datasources which do not support transactions
  will always return OGRERR_NONE.
  
  This function is the same as the C++ method
  OGRLayer::RollbackTransaction().
  
  Parameters
  
  hLayer : handle to the layer
  
  
  
  Returns
  
  OGRERR_NONE on success.                                       }
function OGR_L_RollbackTransaction(hLayer: OGRLayerH): OGRErr; cdecl; external External_Lib name 'OGR_L_RollbackTransaction';
function OGR_L_Reference(hLayer: OGRLayerH): Integer; cdecl; external External_Lib name 'OGR_L_Reference';
{ \ \  }
function OGR_L_Dereference(hLayer: OGRLayerH): Integer; cdecl; external External_Lib name 'OGR_L_Dereference';
function OGR_L_GetRefCount(hLayer: OGRLayerH): Integer; cdecl; external External_Lib name 'OGR_L_GetRefCount';
{ Flush pending changes to disk.
  
  This call is intended to force the layer to flush any pending
  writes to disk, and leave the disk file in a consistent
  state. It would not normally have any effect on read-only
  datasources.
  
  Some layers do not implement this method, and will still
  return OGRERR_NONE. The default implementation just returns
  OGRERR_NONE. An error is only returned if an error occurs
  while attempting to flush to disk.

  In any event, you should always close any opened datasource
  with OGR_DS_Destroy() that will ensure all data is correctly
  flushed.
  
  This method is the same as the C++ method
  OGRLayer::SyncToDisk()
  
  Parameters
  
  hLayer : handle to the layer
  
  Returns
  
  OGRERR_NONE if no error occurs (even if nothing is done) or
  an error code.                                                }
function OGR_L_SyncToDisk(hLayer: OGRLayerH):OGRErr; cdecl; external External_Lib name 'OGR_L_SyncToDisk';
function OGR_L_GetFeaturesRead(hLayer: OGRLayerH): GIntBig; cdecl; external External_Lib name 'OGR_L_GetFeaturesRead';
{ This method returns the name of the underlying database
  column being used as the FID column, or "" if not supported.
  
  This method is the same as the C++ method
  OGRLayer::GetFIDColumn()
  
  Parameters
  
  hLayer : handle to the layer
  
  Returns
  
  fid column name.                                             }
function OGR_L_GetFIDColumn(hLayer: OGRLayerH): PChar; cdecl; external External_Lib name 'OGR_L_GetFIDColumn';
{ This method returns the name of the underlying database
  column being used as the geometry column, or "" if not
  supported.
  
  This method is the same as the C++ method
  OGRLayer::GetGeometryColumn()
  
  Parameters
  
  hLayer : handle to the layer
  
  Returns
  
  geometry column name                                    }
function OGR_L_GetGeometryColumn(hLayer: OGRLayerH): PChar; cdecl; external External_Lib name 'OGR_L_GetGeometryColumn';

{ Closes opened datasource and releases allocated resources.
  
  This method is the same as the C++ method
  OGRDataSource::DestroyDataSource().
  Parameters
  hDS :  handle to allocated datasource object.              }

procedure    OGR_DS_Destroy(hDS: OGRDataSourceH ); cdecl; external External_Lib name 'OGR_DS_Destroy';
{ Returns the name of the data source.
  
  This string should be sufficient to open the data source if
  passed to the same OGRSFDriver that this data source was
  opened with, but it need not be exactly the same string that
  was used to open the data source. Normally this is a
  filename.
  
  This function is the same as the C++ method
  OGRDataSource::GetName().
  
  
  Parameters
  hDS :  handle to the data source to get the name from. 
  
  Returns
  pointer to an internal name string which should not be
  modified or freed by the caller.                             }
function OGR_DS_GetName(hDS: OGRDataSourceH ): PChar; cdecl; external External_Lib name 'OGR_DS_GetName';
{ Get the number of layers in this data source.
  
  This function is the same as the C++ method
  OGRDataSource::GetLayerCount().
  
  
  Parameters
  hDS :  handle to the data source from which to get the number
         of layers. 
  
  Returns
  layer count.                                                  }
function OGR_DS_GetLayerCount(hDS: OGRDataSourceH ): Integer; cdecl; external External_Lib name 'OGR_DS_GetLayerCount';
{ Fetch a layer by index.
  
  
  
  The returned layer remains owned by the OGRDataSource and
  should not be deleted by the application.
  
  
  
  This function is the same as the C++ method
  OGRDataSource::GetLayer().
  Parameters
  hDS :     handle to the data source from which to get the layer.
  iLayer :  a layer number between 0 and OGR_DS_GetLayerCount()\-1.
  
  Returns
  an handle to the layer, or NULL if iLayer is out of range or
  an error occurs.                                                  }
function OGR_DS_GetLayer(hDS: OGRDataSourceH; iLayer: Integer): OGRLayerH  ; cdecl; external External_Lib name 'OGR_DS_GetLayer';
{ Fetch a layer by name.
  
  The returned layer remains owned by the OGRDataSource and
  should not be deleted by the application.
  
  This function is the same as the C++ method
  OGRDataSource::GetLayerByName().
  Parameters
  hDS :      handle to the data source from which to get the
             layer. 
  pszName :  Layer the layer name of the layer to fetch.
  
  Returns
  an handle to the layer, or NULL if the layer is not found or
  an error occurs.                                             }
function OGR_DS_GetLayerByName(hDS: OGRDataSourceH; pszName: PChar ):OGRLayerH  ; cdecl; external External_Lib name 'OGR_DS_GetLayerByName';
{ Delete the indicated layer from the datasource.
  
  If this method is supported the ODsCDeleteLayer capability
  will test TRUE on the OGRDataSource.
  
  This method is the same as the C++ method
  OGRDataSource::DeleteLayer().
  Parameters
  hDS :     handle to the datasource 
  iLayer :  the index of the layer to delete.
  
  Returns
  OGRERR_NONE on success, or OGRERR_UNSUPPORTED_OPERATION if
  deleting layers is not supported for this datasource.      }
function OGR_DS_DeleteLayer(hDS: OGRDataSourceH; iLayer: Integer ): OGRErr; cdecl; external External_Lib name 'OGR_DS_DeleteLayer';
{ Returns the driver that the dataset was opened with.
  
  This method is the same as the C++ method
  OGRDataSource::GetDriver()
  Returns
  NULL if driver info is not available, or pointer to a driver
  owned by the OGRSFDriverManager.
  Parameters
  hDS :  handle to the datasource                              }
function OGR_DS_GetDriver(hDS: OGRDataSourceH ): OGRSFDriverH ; cdecl; external External_Lib name 'OGR_DS_GetDriver';
{ This function attempts to create a new layer on the data
  source with the indicated name, coordinate system, geometry
  type.
  
  The papszOptions argument can be used to control driver
  specific creation options. These options are normally
  documented in the format specific documentation.
  
  This function is the same as the C++ method
  OGRDataSource::CreateLayer().
  Parameters
  hDS :           The dataset handle. 
  pszName :       the name for the new layer. This should
                  ideally not match any existing layer on the
                  datasource. 
  hRef :          handle to the coordinate system to use for the
                  new layer, or NULL if no coordinate system is
                  available. 
  eGType :        the geometry type for the layer. Use
                  wkbUnknown if there are no constraints on the
                  types geometry to be written. 
  papszOptions :  a StringList of name=value options. Options
                  are driver specific, and driver information
                  can be found at the following url\:
                  http\://www.gdal.org/ogr/ogr_formats.html
  
  Returns
  NULL is returned on failure, or a new OGRLayer handle on
  success.                                                       }
function OGR_DS_CreateLayer(hDS: OGRDataSourceH; pszName: PChar;
                                      hRef: OGRSpatialReferenceH; eGType: TOGRwkbGeometryType;
                                      papszOptions: PPChar ): OGRLayerH; cdecl; external External_Lib name 'OGR_DS_CreateLayer';
{ Duplicate an existing layer.
  
  This function creates a new layer, duplicate the field
  definitions of the source layer and then duplicate each
  features of the source layer. The papszOptions argument can
  be used to control driver specific creation options. These
  options are normally documented in the format specific
  documentation. The source layer may come from another
  dataset.
  
  This function is the same as the C++ method
  OGRDataSource::CopyLayer
  
  
  Parameters
  hDS :           handle to the data source where to create the
                  new layer 
  poSrcLayer :    handle to the source layer. 
  pszNewName :    the name of the layer to create. 
  papszOptions :  a StringList of name=value options. Options
                  are driver specific.
  
  Returns
  an handle to the layer, or NULL if an error occurs.           }
function OGR_DS_CopyLayer(hDS: OGRDataSourceH; poSrcLayer: OGRLayerH; pszNewName: PChar;
                                    papszOptions: PPChar): OGRLayerH; cdecl; external External_Lib name 'OGR_DS_CopyLayer';
                                    
{ Test if capability is available.
  
  One of the following data source capability names can be
  passed into this function, and a TRUE or FALSE value will be
  returned indicating whether or not the capability is
  available for this object.
  
    * ODsCCreateLayer: True if this datasource can create new
      layers.
  
  The &#35;define macro forms of the capability names should be
  used in preference to the strings themselves to avoid
  mispelling.
  
  This function is the same as the C++ method
  OGRDataSource::TestCapability().
  
  
  Parameters
  hDS :      handle to the data source against which to test the
             capability. 
  pszName :  the capability to test.
  
  Returns
  TRUE if capability available otherwise FALSE.                  }
function OGR_DS_TestCapability(hDS: OGRDataSourceH; pszName: PChar ): Integer; cdecl; external External_Lib name 'OGR_DS_TestCapability';
{ Execute an SQL statement against the data store.

  The result of an SQL query is either NULL for statements that
  are in error, or that have no results set, or an OGRLayer
  handle representing a results set from the query. Note that
  this OGRLayer is in addition to the layers in the data store
  and must be destroyed with OGR_DS_ReleaseResultSet() before
  the data source is closed (destroyed).
  
  For more information on the SQL dialect supported internally
  by OGR review the OGR SQL document. Some drivers (ie. Oracle
  and PostGIS) pass the SQL directly through to the underlying
  RDBMS.
  
  This function is the same as the C++ method
  OGRDataSource::ExecuteSQL();
  
  
  Parameters
  hDS :              handle to the data source on which the SQL
                     query is executed. 
  pszStatement :     the SQL statement to execute. 
  poSpatialFilter :  handle to a geometry which represents a
                     spatial filter. Can be NULL. 
  pszDialect :       allows control of the statement dialect. If
                     set to NULL, the OGR SQL engine will be
                     used, except for RDBMS drivers that will
                     use their dedicated SQL engine, unless
                     OGRSQL is explicitely passed as the
                     dialect.
  
  Returns
  an handle to a OGRLayer containing the results of the query.
  Deallocate with OGR_DS_ReleaseResultSet().                     }
function OGR_DS_ExecuteSQL(hDS: OGRDataSourceH; pszStatement: PChar;
                                     poSpatialFilter: OGRGeometryH; pszDialect: PChar): OGRLayerH ; cdecl; external External_Lib name 'OGR_DS_ExecuteSQL';
{ Release results of OGR_DS_ExecuteSQL().
  
  This function should only be used to deallocate OGRLayers
  resulting from an OGR_DS_ExecuteSQL() call on the same
  OGRDataSource. Failure to deallocate a results set before
  destroying the OGRDataSource may cause errors.
  
  This function is the same as the C++ method
  OGRDataSource::ReleaseResultSet().
  Parameters
  hDS :     an handle to the data source on which was executed
            an SQL query. 
  hLayer :  handle to the result of a previous
            OGR_DS_ExecuteSQL() call.                          }
procedure    OGR_DS_ReleaseResultSet(hDS: OGRDataSourceH; hLayer: OGRLayerH ); cdecl; external External_Lib name 'OGR_DS_ReleaseResultSet';
function OGR_DS_Reference(hDS: OGRDataSourceH ): Integer; cdecl; external External_Lib name 'OGR_DS_Reference';
function OGR_DS_Dereference(hDS: OGRDataSourceH ): Integer; cdecl; external External_Lib name 'OGR_DS_Dereference';
function OGR_DS_GetRefCount(hDS: OGRDataSourceH ): Integer; cdecl; external External_Lib name 'OGR_DS_GetRefCount';
function OGR_DS_GetSummaryRefCount(hDS: OGRDataSourceH ): Integer; cdecl; external External_Lib name 'OGR_DS_GetSummaryRefCount';
{ Flush pending changes to disk.
  
  This call is intended to force the datasource to flush any
  pending writes to disk, and leave the disk file in a
  consistent state. It would not normally have any effect on
  read-only datasources.
  
  Some data sources do not implement this method, and will
  still return OGRERR_NONE. An error is only returned if an
  error occurs while attempting to flush to disk.
  
  The default implementation of this method just calls the
  SyncToDisk() method on each of the layers. Conceptionally,
  calling SyncToDisk() on a datasource should include any work
  that might be accomplished by calling SyncToDisk() on layers
  in that data source.
  
  In any event, you should always close any opened datasource
  with OGR_DS_Destroy() that will ensure all data is correctly
  flushed.
  
  This method is the same as the C++ method
  OGRDataSource::SyncToDisk()
  

  Parameters
  hDS :  handle to the data source
  
  Returns
  OGRERR_NONE if no error occurs (even if nothing is done) or
  an error code.                                               }
function OGR_DS_SyncToDisk(hDS: OGRDataSourceH ):OGRErr  ; cdecl; external External_Lib name 'OGR_DS_SyncToDisk';


{ OGR_Dr_GetName()
  
  Fetch name of driver (file format). This name should be
  relatively short (10-40 characters), and should reflect the
  underlying file format. For instance "ESRI Shapefile".
  
  This function is the same as the C++ method
  OGRSFDriver::GetName().
  
  
  Returns
  driver name. This is an internal string and should not be
  modified or freed.
  Parameters
  hDriver :  handle to the the driver to get the name from.   }
 function OGR_Dr_GetName(hDriver: TOGRSFDriverH ): PChar; cdecl; external External_Lib name 'OGR_Dr_GetName';
 { OGR_Dr_Open()
   
   Attempt to open file with this driver.
   
   This function is the same as the C++ method
   OGRSFDriver::Open().
   
   
   Parameters
   hDriver :  handle to the driver that is used to open file. 
   pszName :  the name of the file, or data source to try and
              open. 
   pUpdate :  TRUE if update access is required, otherwise FALSE
              (the default).
   
   Returns
   NULL on error or if the pass name is not supported by this
   driver, otherwise an handle to an OGRDataSource. This
   OGRDataSource should be closed by deleting the object when it
   is no longer needed.                                          }
 function OGR_Dr_Open(hDriver: TOGRSFDriverH; pszName: pChar; pUpdate: Integer): TOGRDataSourceH; cdecl; external External_Lib name 'OGR_Dr_Open';
 { OGR_Dr_TestCapability()
   
   
   
   Test if capability is available.
   
   One of the following data source capability names can be
   passed into this function, and a TRUE or FALSE value will be
   returned indicating whether or not the capability is
   available for this object.
     * \  ODrCCreateDataSource: True if this driver can support
       creating data sources.
     * \  ODrCDeleteDataSource: True if this driver supports
       deleting data sources.
   The &#35;define macro forms of the capability names should be
   used in preference to the strings themselves to avoid
   mispelling.
   
   This function is the same as the C++ method
   OGRSFDriver::TestCapability().
   Parameters
   hDriver :  handle to the driver to test the capability
              against. 
   pszCap :   the capability to test.
   
   Returns
   TRUE if capability available otherwise FALSE.                 }
 function OGR_Dr_TestCapability(hDriver: TOGRSFDriverH; pszCap: PChar): Integer; cdecl; external External_Lib name 'OGR_Dr_TestCapability';
 { OGR_Dr_CreateDataSource()
   
   This function attempts to create a new data source based on
   the passed driver.
   
   The papszOptions argument can be used to control driver
   specific creation options. These options are normally
   documented in the format specific documentation.
   
   It is important to call OGR_DS_Destroy() when the datasource
   is no longer used to ensure that all data has been properly
   flushed to disk.
   
   This function is the same as the C++ method
   OGRSFDriver::CreateDataSource().
   Parameters
   hDriver :       handle to the driver on which data source
                   creation is based.
   pszName :       the name for the new data source. UTF\-8
                   encoded.
   papszOptions :  a StringList of name=value options. Options
                   are driver specific, and driver information
                   can be found at the following url\:<extlink http://www.gdal.org/ogr/ogr_formats.html>http\://www.gdal.org/ogr/ogr_formats.html</extlink>
   Note
   This method does NOT attach driver instance to the returned
   data source, so caller should expect that
   OGRDataSource::GetDriver() will return NULL pointer. In order
   to attach driver to the returned data source, it is required
   to use C function OGR_Dr_CreateDataSource. This behavior is
   related to fix of issue reported in Ticket #1233.
   Returns
   NULL is returned on failure, or a new OGRDataSource handle on
   success.                                                                                                                                                 }
 function OGR_Dr_CreateDataSource(hDriver: TOGRSFDriverH; pszName: PChar; papszOptions: PPChar ): TOGRDataSourceH; cdecl; external External_Lib name 'OGR_Dr_CreateDataSource';
 { OGR_Dr_CopyDataSource()
   
   This function creates a new datasource by copying all the
   layers from the source datasource.
   
   It is important to call OGR_DS_Destroy() when the datasource
   is no longer used to ensure that all data has been properly
   flushed to disk.

   This function is the same as the C++ method
   OGRSFDriver::CopyDataSource().
   Parameters
   hDriver :       handle to the driver on which data source
                   creation is based.
   hSource :       source datasource
   pszNewName :    the name for the new data source.
   papszOptions :  a StringList of name=value options. Options
                   are driver specific, and driver information
                   can be found at the following url\: <extlink http://www.gdal.org/ogr/ogr_formats.html>http\://www.gdal.org/ogr/ogr_formats.html</extlink>
   Returns
   NULL is returned on failure, or a new OGRDataSource handle on
   success.                                                                                                                                                  }
 function OGR_Dr_CopyDataSource(hDriver: TOGRSFDriverH; hSource:  TOGRDataSourceH;
                                              pszNewName: PChar; papszOptions: PPChar ): TOGRDataSourceH; cdecl; external External_Lib name 'OGR_Dr_CopyDataSource';
 { OGR_Dr_DeleteDataSource()
   
   Delete a datasource.
   
   Delete (from the disk, in the database, ...) the named
   datasource. Normally it would be safest if the datasource was
   not open at the time.
   
   Whether this is a supported operation on this driver case be
   tested using TestCapability() on ODrCDeleteDataSource.
   
   This method is the same as the C++ method
   OGRSFDriver::DeleteDataSource().
   Parameters
   hDriver :        handle to the driver on which data source
                    deletion is based.
   pszDataSource :  the name of the datasource to delete.
   
   Returns
   OGRERR_NONE on success, and OGRERR_UNSUPPORTED_OPERATION if
   this is not supported by this driver.                         }
 function OGR_Dr_DeleteDataSource(hDriver: TOGRSFDriverH; pszDataSource: PChar): TOGRErr; cdecl; external External_Lib name 'OGR_Dr_DeleteDataSource';


{ Open a file / data source with one of the registered drivers.

  This function loops through all the drivers registered with
  the driver manager trying each until one succeeds with the
  given data source. This function is static. Applications
  don't normally need to use any other OGRSFDriverRegistrar
  function, not do they normally need to have a pointer to an
  OGRSFDriverRegistrar instance.

  If this function fails, CPLGetLastErrorMsg() can be used to
  check if there is an error message explaining why.

  This function is the same as the C++ method
  OGRSFDriverRegistrar::Open().

  Parameters
  pszName : the name of the file, or data source to open.
  bUpdate : FALSE for read-only access (the default) or TRUE
  for read-write access.
  pahDriverList : if non-NULL, this argument will be updated
  with a pointer to the driver which was used to open the data
  source.
  Returns
  NULL on error or if the pass name is not supported by this
  driver, otherwise an handle to an OGRDataSource. This
  OGRDataSource should be closed by deleting the object when it
  is no longer needed.
  Example
  <code lang="c++">
  OGRDataSourceH hDS;
  OGRSFDriverH *pahDriver;

  hDS = OGROpen( "polygon.shp", 0, pahDriver );
  if( hDS == NULL )
  {
  return;
  &#125;
  
  ... use the data source ...
  
  OGRReleaseDataSource( hDS );
  </code>                                                       }

  function OGROpen(pszName: PChar; bUpdate: Integer; hDriver: TOGRSFDriverH): TOGRSFDriverH; cdecl; external External_Lib name 'OGROpen';
  function OGROpenShared( pszName: PChar; bUpdate: Integer; hDriver: TOGRSFDriverH): TOGRDataSourceH; cdecl; external External_Lib name 'OGROpenShared';
  { Drop a reference to this datasource, and if the reference
    count drops to zero close (destroy) the datasource.
    
    Internally this actually calls the
    OGRSFDriverRegistrar::ReleaseDataSource() method. This method
    is essentially a convenient alias.
    
    This method is the same as the C++ method
    OGRDataSource::Release()
    
    Parameters
    
    hDS : handle to the data source to release
    
    Returns
    
    OGRERR_NONE on success or an error code.                      }
  function OGRReleaseDataSource(hDS: TOGRDataSourceH ): TOGRErr; cdecl; external External_Lib name 'OGRReleaseDataSource';
  { Remove the passed driver from the list of registered drivers.
    
    This function is the same as the C++ method
    OGRSFDriverRegistrar::DeregisterDriver().
    
    
    Parameters
    
    hDriver : handle to the driver to deregister.                 }
  procedure OGRRegisterDriver(hDriver: TOGRSFDriverH ); cdecl; external External_Lib name 'OGRRegisterDriver';
  { Fetch the number of registered drivers.
    
    This function is the same as the C++ method
    OGRSFDriverRegistrar::GetDriverCount().

    
    
    Returns
    
    the drivers count.                          }
  function OGRGetDriverCount: Integer; cdecl; external External_Lib name 'OGRGetDriverCount';
  { Fetch the indicated driver.
    
    This function is the same as the C++ method
    OGRSFDriverRegistrar::GetDriver().
    
    Parameters
    
    iDriver : the driver index, from 0 to GetDriverCount()-1.
    
    Returns
    
    handle to the driver, or NULL if iDriver is out of range.  }
  function OGRGetDriver(iDrv: Integer): TOGRSFDriverH; cdecl; external External_Lib name 'OGRGetDriver';
  { Fetch the indicated driver.
    
    This function is the same as the C++ method
    OGRSFDriverRegistrar::GetDriverByName()
    

    Parameters
    pszName : the driver name
    
    Returns
    
    the driver, or NULL if no driver with that name is found  }
  function OGRGetDriverByName( lpszDrvName: PChar): TOGRSFDriverH; cdecl; external External_Lib name 'OGRGetDriverByName';
  { Return the number of opened datasources.
    
    This function is the same as the C++ method
    OGRSFDriverRegistrar::GetOpenDSCount()
    
    

    Returns
    
    the number of opened datasources.           }
  function OGRGetOpenDSCount: Integer; cdecl; external External_Lib name 'OGRGetOpenDSCount';
  { Return the iDS th datasource opened.
    
    This function is the same as the C++ method
    OGRSFDriverRegistrar::GetOpenDS.
    
    
    
    Parameters
    
    iDS : the index of the dataset to return (between 0 and
    GetOpenDSCount() - 1)                                   }
  function OGRGetOpenDS(iDS: Integer): TOGRDataSourceH; cdecl; external External_Lib name 'OGRGetOpenDS';


{ 	Register all drivers.  }
   procedure OGRRegisterAll; cdecl; external External_Lib name 'OGRRegisterAll';
   { Cleanup all OGR related resources.

     This function will destroy the OGRSFDriverRegistrar along
     with all registered drivers, and then cleanup long lived OSR
     (OGRSpatialReference) and CPL resources. This may be called
     in an application when OGR services are no longer needed. It
     is not normally required, but by freeing all dynamically
     allocated memory it can make memory leak testing easier.

     In addition to destroying the OGRDriverRegistrar, this
     function also calls:
     
       * OSRCleanup()
       * CPLFinderClean()
       * VSICleanupFileManager()
       * CPLFreeConfig()
       * CPLCleanupTLS()                                          }
   procedure OGRCleanupAll; cdecl; external External_Lib name 'OGRCleanupAll';




  { Constructor.  }
  function OSRNewSpatialReference(const WKT : PCHAR
                                 ) : TOGRSpatialReferenceH;
                                 stdcall; external External_Lib name '_OSRNewSpatialReference@4';
  { Make a duplicate of the GEOGCS node of this
    OGRSpatialReference object.                 }
  function OSRCloneGeogCS(const Handle : TOGRSpatialReferenceH
                         ) : TOGRSpatialReferenceH;
                         stdcall; external External_Lib name '_OSRCloneGeogCS@4';

  { Make a duplicate of this OGRSpatialReference.  }
  function  OSRClone(const Handle : TOGRSpatialReferenceH ) : TOGRSpatialReferenceH ;
                    stdcall; external External_Lib name '_OSRClone@4';
  { OGRSpatialReference destructor.
    
    This function is the same as
    OGRSpatialReference::~OGRSpatialReference() and
    OGRSpatialReference::DestroySpatialReference()
    
    
    
    Parameters
    
    hSRS : the object to delete                     }
  procedure OSRDestroySpatialReference (const Handle : TOGRSpatialReferenceH); stdcall; external External_Lib name '_OSRDestroySpatialReference@4';

  { Increments the reference count by one.  }
  function  OSRReference(hSRS : TOGRSpatialReferenceH) : longint; cdecl; external External_Lib name 'OSRReference';
  { Decrements the reference count by one.  }
  function  OSRDereference(hSRS : TOGRSpatialReferenceH) : longint; cdecl; external External_Lib name 'OSRDereference';
  { Decrements the reference count by one, and destroy if zero.  }
  procedure OSRRelease(hSRS : TOGRSpatialReferenceH ); cdecl; external External_Lib name 'OSRRelease';
  { Validate SRS tokens. }
  function  OSRValidate(hSRS : TOGRSpatialReferenceH ) : TOGRErr; cdecl; external External_Lib name 'OSRValidate';
  { Correct parameter ordering to match CT Specification.  }
  function  OSRFixupOrdering(hSRS : TOGRSpatialReferenceH) : TOGRErr; cdecl; external External_Lib name 'OSRFixupOrdering';
  { Fixup as needed.  }
  function  OSRFixup(hSRS : TOGRSpatialReferenceH ) : TOGRErr; cdecl; external External_Lib name 'OSRFixup';
  { Strip OGC CT Parameters.  }
  function  OSRStripCTParms(hSRS : TOGRSpatialReferenceH) : TOGRErr; cdecl; external External_Lib name 'OSRStripCTParms';

  { Initialize SRS based on EPSG GCS or PCS code. }
  function  OSRImportFromEPSG(hSRS : TOGRSpatialReferenceH; EPSGCode : longint
                                 ) : TOGRErr; stdcall; external External_Lib name '_OSRImportFromEPSG@8';
  { Initialize SRS based on EPSG GCS or PCS code. }                               
  function  OSRImportFromEPSGA(hSRS : TOGRSpatialReferenceH; EPSGCode : longint
                                  ) : TOGRErr; stdcall; external External_Lib name '_OSRImportFromEPSGA@8';

  { Import from WKT string.  }
  function  OSRImportFromWkt(hSRS : TOGRSpatialReferenceH;
                             ppszInput : PPCHAR
                             ) : TOGRErr; cdecl; external External_Lib name 'OSRImportFromWkt';
  { Import PROJ.4 coordinate string.  }
  function  OSRImportFromProj4(hSRS : TOGRSpatialReferenceH;
                               pszProj4 : PCHAR) : TOGRErr; cdecl; external External_Lib name 'OSRImportFromProj4';
  { Import coordinate system from ESRI .prj format(s).  }
  function  OSRImportFromESRI(hSRS : TOGRSpatialReferenceH;
                               papszPrj : PPCHAR) : TOGRErr; cdecl; external External_Lib name 'OSRImportFromESRI';
  { Import coordinate system from PCI projection definition. }
  function  OSRImportFromPCI(hSRS : TOGRSpatialReferenceH; pszProj : PCHAR;
		             pszUnits : PCHAR; padfPrjParams : PDOUBLE
                             ) : TOGRErr; cdecl; external External_Lib name 'OSRImportFromPCI';
  { Import coordinate system from USGS projection definition.  }
  function  OSRImportFromUSGS(hSRS : TOGRSpatialReferenceH; iProjsys : longint;
                              iZone : longint; padfPrjParams : PDOUBLE;
                              iDatum : longint
                              ) : TOGRErr; cdecl; external External_Lib name 'OSRImportFromUSGS';
  { Import coordinate system from XML format (GML only
    currently).                                        }
  function  OSRImportFromXML(hSRS : TOGRSpatialReferenceH;
                              pszXML : PCHAR) : TOGRErr; cdecl; external External_Lib name 'OSRImportFromXML';

  function OSRImportFromDict(hSRS: OGRSpatialReferenceH;
                              pszDict :PChar; pszCode: PChar): TOGRErr; cdecl; external External_Lib name 'OSRImportFromDict';

  function OSRImportFromPanorama(hSRS: OGRSpatialReferenceH; iProjSys: long; iDatum: long; iEllips: long;
                                      padfPrjParams: PDouble ): TOGRErr;  cdecl; external External_Lib name 'OSRImportFromPanorama';

  { Import Mapinfo style CoordSys definition.  }
  function  OSRImportFromMICoordSys(hSRS : TOGRSpatialReferenceH;
                                    pszCoordSys : PCHAR
                                   ) : TOGRErr; cdecl; external External_Lib name 'OSRImportFromMICoordSys';
  { Set spatial reference from a URL.  }
  function  OSRImportFromUrl(hSRS : TOGRSpatialReferenceH;
                             pszUrl : PCHAR
                            ) : TOGRErr; cdecl; external External_Lib name 'OSRImportFromUrl';

  { Convert this SRS into WKT format.  }
  function  OSRExportToWkt(hSRS : TOGRSpatialReferenceH;
                           ppszReturn : PPCHAR) : TOGRErr; stdcall; external External_Lib name '_OSRExportToWkt@8';
  { Convert this SRS into a a nicely formatted WKT string for
    display to a person.                                      }
  function  OSRExportToPrettyWkt(hSRS : TOGRSpatialReferenceH;
                                 ppszReturn : PPCHAR;
                                 bSimplify: longint) : TOGRErr; stdcall; external External_Lib name '_OSRExportToPrettyWkt@12';

  { Export coordinate system in PROJ.4 format.  }
  function  OSRExportToProj4(hSRS : TOGRSpatialReferenceH;
                             ppszReturn : PPCHAR) : TOGRErr; stdcall; external External_Lib name '_OSRExportToProj4@8';
  { Export coordinate system in PCI projection definition.  }
  function  OSRExportToPCI(hSRS : TOGRSpatialReferenceH; ppszProj : PPCHAR;
                           ppszUnits : PPCHAR;
                           ppadfPrjParams : PPDOUBLE
                           ) : TOGRErr; cdecl; external External_Lib name 'OSRExportToPCI';
  { Export coordinate system in USGS GCTP projection definition.  }
  function  OSRExportToUSGS(hSRS : TOGRSpatialReferenceH; piProjSys : PINTEGER;
                            piZone : PLONGINT;
                            ppadfPrjParams : PPDOUBLE;
                            piDatum : PLONGINT
                            ) : TOGRErr; cdecl; external External_Lib name 'OSRExportToUSGS';
  { Export coordinate system in XML format.  }
  function  OSRExportToXML(hSRS : TOGRSpatialReferenceH; ppszRawXML : PPCHAR;
                           pszDialect : PCHAR
                           ) : TOGRErr; cdecl; external External_Lib name 'OSRExportToXML';
  function OSRExportToPanorama(hSRS: TOGRSpatialReferenceH; var piProjSys: long; var piDatum: Long;
                           var piEllips: Long; var piZone: long; padfPrjParams: PDouble
                           ) : TOGRErr; cdecl; external External_Lib name 'OSRExportToPanorama';

  { Export coordinate system in Mapinfo style CoordSys format.  }
  function  OSRExportToMICoordSys(hSRS : TOGRSpatialReferenceH;
                                  ppszReturn : PPCHAR) : TOGRErr; cdecl; external External_Lib name 'OSRExportToMICoordSys';

  { Convert in place to ESRI WKT format.  }
  function  OSRMorphToESRI(hSRS : TOGRSpatialReferenceH) : TOGRErr; cdecl; external External_Lib name 'OSRMorphToESRI';
  { Convert in place from ESRI WKT format.  }
  function  OSRMorphFromESRI(hSRS : TOGRSpatialReferenceH) : TOGRErr; cdecl; external External_Lib name 'OSRMorphFromESRI';

  { Set attribute value in spatial reference.  }
  function  OSRSetAttrValue(hSRS : TOGRSpatialReferenceH;
                            pszNodePath : PCHAR;
                            pszNewNodeValue : PCHAR) : TOGRErr; stdcall; external External_Lib name '_OSRSetAttrValue@12';
  function  OSRGetAttrValue(hSRS : TOGRSpatialReferenceH;
                            pszName : PCHAR;
                            iChild  : longint(* = 0 *)) : PCHAR; stdcall; external External_Lib name '_OSRGetAttrValue@12';

  { Set the angular units for the geographic coordinate system. }
  function  OSRSetAngularUnits(hSRS : TOGRSpatialReferenceH; pszUnits : PCHAR;
                               dfInRadians : double) : TOGRErr; cdecl; external External_Lib name 'OSRSetAngularUnits';
  { Fetch angular geographic coordinate system units.  }
  function  OSRGetAngularUnits(hSRS : TOGRSpatialReferenceH;
                               ppszName : PPCHAR) : double; cdecl; external External_Lib name 'OSRGetAngularUnits';
  { Set the linear units for the projection.  }
  function  OSRSetLinearUnits(hSRS : TOGRSpatialReferenceH; pszUnits : PCHAR;
                              dfInMeters : double) : TOGRErr; cdecl; external External_Lib name 'OSRSetLinearUnits';
  { Set the linear units for the projection. }
  function  OSRSetLinearUnitsAndUpdateParameters(hSRS : TOGRSpatialReferenceH;
                                                 pszUnits : PCHAR;
                                                 dfInMeters : double
                                                 ) : TOGRErr; cdecl; external External_Lib name 'OSRSetLinearUnitsAndUpdateParameters';
  { Fetch linear projection units.  }
  function  OSRGetLinearUnits(hSRS : TOGRSpatialReferenceH;
                              ppszName : PPCHAR) : double; cdecl; external External_Lib name 'OSRGetLinearUnits';
  { Fetch prime meridian info.  }
  function  OSRGetPrimeMeridian(hSRS : TOGRSpatialReferenceH;
                                ppszName : PPCHAR) : double; cdecl; external External_Lib name 'OSRGetPrimeMeridian';
  { Check if geographic coordinate system.  }
  function  OSRIsGeographic(hSRS : TOGRSpatialReferenceH) : longint; cdecl; external External_Lib name 'OSRIsGeographic';
  { Check if local coordinate system }
  function  OSRIsLocal(hSRS : TOGRSpatialReferenceH) : longint; cdecl; external External_Lib name 'OSRIsLocal';
  { Check if projected coordinate system }
  function  OSRIsProjected(hSRS : TOGRSpatialReferenceH) : longint; cdecl; external External_Lib name 'OSRIsProjected';
  { Do the GeogCS'es match?  }
  function  OSRIsSameGeogCS(hSRS1 : TOGRSpatialReferenceH;
                            hSRS2 : TOGRSpatialReferenceH) : longint; cdecl; external External_Lib name 'OSRIsSameGeogCS';
  { Do these two spatial references describe the same system ?  }
  function  OSRIsSame(hSRS1 : TOGRSpatialReferenceH;
                      hSRS2 : TOGRSpatialReferenceH) : longint; cdecl; external External_Lib name 'OSRIsSame';
  { Set the user visible LOCAL_CS name.  }
  function  OSRSetLocalCS(hSRS : TOGRSpatialReferenceH;
                          pszName : PCHAR) : TOGRErr; cdecl; external External_Lib name 'OSRSetLocalCS';
  { Set the user visible PROJCS name.  }
  function  OSRSetProjCS(hSRS : TOGRSpatialReferenceH;
                         pszName : PCHAR) : TOGRErr; cdecl; external External_Lib name 'OSRSetProjCS';
  { Set a GeogCS based on well known name }
  function  OSRSetWellKnownGeogCS(hSRS : TOGRSpatialReferenceH;
                                  pszName : PCHAR) : TOGRErr; cdecl; external External_Lib name 'OSRSetWellKnownGeogCS';

  { Set spatial reference from various text formats.  }
  function  OSRSetFromUserInput(hSRS : TOGRSpatialReferenceH;
                                pszDef : PCHAR): TOGRErr; stdcall; external External_Lib name '_OSRSetFromUserInput@8';

  { Copy GEOGCS from another OGRSpatialReference.  }
  function  OSRCopyGeogCSFrom(hSRS : TOGRSpatialReferenceH;
                              hSrcSRS : TOGRSpatialReferenceH
                              ) : TOGRErr; cdecl; external External_Lib name 'OSRCopyGeogCSFrom';
  { Set the Bursa-Wolf conversion to WGS84.  }
  function  OSRSetTOWGS84(hSRS :  TOGRSpatialReferenceH;
                          dfDX, dfDY,
                          dfDZ, dfEX,
                          dfEY, dfEZ,
                          dfPPM : double
                          ) : TOGRErr; cdecl; external External_Lib name 'OSRSetTOWGS84';
  { Fetch TOWGS84 parameters, if available }
  function  OSRGetTOWGS84(hSRS : TOGRSpatialReferenceH; padfCoeff : PDOUBLE;
                          nCoeffCount : longint) : TOGRErr; cdecl; external External_Lib name 'OSRGetTOWGS84';
  function  OSRSetGeogCS(hSRS : TOGRSpatialReferenceH;
                         pszGeogName, pszDatumName, pszEllipsoidName : PCHAR;
                         dfSemiMajor, dfInvFlattening : double;
                         pszPMName : PCHAR; (* = NULL *)
                         dfPMOffset : double; (* = 0.0 *)
                         pszUnits : PCHAR;(* = NULL *)
                         dfConvertToRadians : double(* = 0.0 *)
                         ) : TOGRErr; cdecl; external External_Lib name 'OSRSetGeogCS';

  { Get spheroid semi major axis.  }
  function  OSRGetSemiMajor(hSRS : TOGRSpatialReferenceH;
                            pnErr : POGRErr) : double; cdecl; external External_Lib name 'OSRGetSemiMajor';
  { Get spheroid semi minor axis.  }
  function  OSRGetSemiMinor(hSRS : TOGRSpatialReferenceH;
                            pnErr : POGRErr) : double; cdecl; external External_Lib name 'OSRGetSemiMinor';
  { Get spheroid inverse flattening.  }
  function  OSRGetInvFlattening(hSRS : TOGRSpatialReferenceH;
                                pnErr : POGRErr) : double; cdecl; external External_Lib name 'OSRGetInvFlattening';
  { Set the authority for a node. }
  function  OSRSetAuthority(hSRS :  TOGRSpatialReferenceH;
                            pszTargetKey : PCHAR;
                            pszAuthority : PCHAR;
                            nCode : longint) : TOGRErr; cdecl; external External_Lib name 'OSRSetAuthority';
  { Get the authority code for a node.  }
  function  OSRGetAuthorityCode(hSRS : TOGRSpatialReferenceH;
                                pszTargetKey : PCHAR) : PCHAR; cdecl; external External_Lib name 'OSRGetAuthorityCode';
  { Get the authority name for a node.  }
  function  OSRGetAuthorityName(hSRS : TOGRSpatialReferenceH;
                                pszTargetKey : PCHAR) : PCHAR; cdecl; external External_Lib name 'OSRGetAuthorityName';
  { Set a projection name.  }
  function  OSRSetProjection(hSRS : TOGRSpatialReferenceH;
                             pszProjection : PCHAR) : TOGRErr; cdecl; external External_Lib name 'OSRSetProjection';
  { Set a projection parameter value.  }
  function  OSRSetProjParm(hSRS : TOGRSpatialReferenceH;
                           pszParmName : PCHAR;
                           dfValue : double
                           ) : TOGRErr; cdecl; external External_Lib name 'OSRSetProjParm';
  { Fetch a projection parameter value. }
  function  OSRGetProjParm(hSRS :  TOGRSpatialReferenceH;
                           pszParmName : PCHAR;
                           dfDefault : double;
                           pnErr : POGRErr
                           ) : double; cdecl; external External_Lib name 'OSRGetProjParm';
  { Set a projection parameter with a normalized value.  }
  function   OSRSetNormProjParm(hSRS : TOGRSpatialReferenceH;
                                pszParmName : PCHAR;
                                dfValue : double
                                ) : TOGRErr; cdecl; external External_Lib name 'OSRSetNormProjParm';
  function   OSRGetNormProjParm(hSRS :  TOGRSpatialReferenceH;
                                pszParmName : PCHAR;
                                dfDefault : double;
                                pnErr : POGRErr
                                ) : double; cdecl; external External_Lib name 'OSRGetNormProjParm';
  { Set UTM projection definition. }
  function   OSRSetUTM(hSRS :  TOGRSpatialReferenceH;
                       nZone : longint;
                       bNorth : longint
                       ) : TOGRErr; cdecl; external External_Lib name 'OSRSetUTM';
  { Get utm zone information.  }
  function   OSRGetUTMZone(hSRS :  TOGRSpatialReferenceH;
                           pbNorth : PINTEGER
                           ) : longint; cdecl; external External_Lib name 'OSRGetUTMZone';

  { Set State Plane projection definition.  }
  function OSRSetStatePlane(hSRS: OGRSpatialReferenceH;
                                 nZone: Integer; bNAD83: Integer ): TOGRErr; cdecl; external External_Lib name 'OSRSetStatePlane';
  { Set State Plane projection definition.  }
  function OSRSetStatePlaneWithUnits(hSRS: OGRSpatialReferenceH;
                                          nZone: Integer; bNAD83: Integer;
                                          pszOverrideUnitName: PChar;
                                          dfOverrideUnit: double): TOGRErr; cdecl; external External_Lib name 'OSRSetStatePlaneWithUnits';
  { Set EPSG authority info if possible.  }
  function OSRAutoIdentifyEPSG(hSRS: OGRSpatialReferenceH): TOGRErr; cdecl; external External_Lib name 'OSRAutoIdentifyEPSG';
  { Fetch the orientation of one axis.  }
  function OSRGetAxis(hSRS: OGRSpatialReferenceH;
                                pszTargetKey: PChar; iAxis: Integer;
                                peOrientation: TOGRAxisOrientation): PChar; cdecl; external External_Lib name 'OSRGetAxis';


  { Albers Conic Equal Area }
  function   OSRSetACEA(hSRS :  TOGRSpatialReferenceH;
                        dfStdP1, dfStdP2,
                        dfCenterLat, dfCenterLong,
                        dfFalseEasting, dfFalseNorthing : double
                        ) : TOGRErr; cdecl; external External_Lib name 'OSRSetACEA';

  { Azimuthal Equidistant }
  function    OSRSetAE(hSRS :  TOGRSpatialReferenceH;
                       dfCenterLat, dfCenterLong,
                       dfFalseEasting, dfFalseNorthing : double
                       ) : TOGRErr; cdecl; external External_Lib name 'OSRSetACEA';

  { Bonne }
  function   OSRSetBonne(hSRS : TOGRSpatialReferenceH;
                         dfStandardParallel, dfCentralMeridian,
                         dfFalseEasting, dfFalseNorthing : double
                         ) : TOGRErr; cdecl; external External_Lib name 'OSRSetBonne';

  { Cylindrical Equal Area }
  function   OSRSetCEA(hSRS :  TOGRSpatialReferenceH;
                       dfStdP1, dfCentralMeridian,
                       dfFalseEasting, dfFalseNorthing : double
                       ) : TOGRErr; cdecl; external External_Lib name 'OSRSetCEA';

  { Cassini-Soldner }
  function   OSRSetCS(hSRS :  TOGRSpatialReferenceH;
                      dfCenterLat, dfCenterLong,
                      dfFalseEasting, dfFalseNorthing : double
                      ) : TOGRErr; cdecl; external External_Lib name 'OSRSetCS';

  { Equidistant Conic }
  function   OSRSetEC(hSRS :  TOGRSpatialReferenceH;
                      dfStdP1, dfStdP2,
                      dfCenterLat, dCenterLong,
                      dfFalseEasting, dfFalseNorthing : double
                      ) : TOGRErr; cdecl; external External_Lib name 'OSRSetEC';

  { Eckert I-VI }
  function   OSRSetEckert(hSRS :  TOGRSpatialReferenceH;
                          nVariation : longint;
                          dfCentralMeridian,
                          dfFalseEasting,
                          dfFalseNorthing : double
                          ) : TOGRErr; cdecl; external External_Lib name 'OSRSetEckert';

  { Eckert IV }
  function   OSRSetEckertIV(hSRS :  TOGRSpatialReferenceH;
                            dfCentralMeridian,
                            dfFalseEasting,
                            dfFalseNorthing : double
                            ) : TOGRErr; cdecl; external External_Lib name 'OSRSetEckertIV';

  { Eckert VI }
  function   OSRSetEckertVI(hSRS :  TOGRSpatialReferenceH;
                            dfCentralMeridian,
                            dfFalseEasting,
                            dfFalseNorthing : double
                            ) : TOGRErr; cdecl; external External_Lib name 'OSRSetEckertVI';

  { Equirectangular }
  function   OSRSetEquirectangular(hSRS : TOGRSpatialReferenceH;
                                   dfCenterLat, dfCenterLong,
                                   dfFalseEasting, dfFalseNorthing : double
                                   ) : TOGRErr; cdecl; external External_Lib name 'OSRSetEquirectangular';
  { Equirectangular generalized form }
  function   OSRSetEquirectangular2(hSRS : TOGRSpatialReferenceH;
                                    dfCenterLat, dfCenterLong,
                                    dfPseudoStdParallel1,
                                    dfFalseEasting,
                                    dfFalseNorthing : double
                                    ) : TOGRErr; cdecl; external External_Lib name 'OSRSetEquirectangular2';

  { Gall Stereograpic }
  function   OSRSetGS(hSRS :  TOGRSpatialReferenceH;
                      dfCentralMeridian,
                      dfFalseEasting,
                      dfFalseNorthing : double
                      ) : TOGRErr; cdecl; external External_Lib name 'OSRSetGS';

  { Goode Homolosine }
  function   OSRSetGH(hSRS :  TOGRSpatialReferenceH;
                      dfCentralMeridian,
                      dfFalseEasting,
                      dfFalseNorthing : double
                      ) : TOGRErr; cdecl; external External_Lib name 'OSRSetGH';

  { GEOS - Geostationary Satellite View }
  function   OSRSetGEOS(hSRS : TOGRSpatialReferenceH;
                        dfCentralMeridian, dfSatelliteHeight,
                        dfFalseEasting, dfFalseNorthing : double
                        ) : TOGRErr; cdecl; external External_Lib name 'OSRSetGEOS';

  { Gauss Schreiber Transverse Mercator }
  function   OSRSetGaussSchreiberTMercator(hSRS :  TOGRSpatialReferenceH;
                                           dfCenterLat, dfCenterLong,
                                           dfScale,
                                           dfFalseEasting,
                                           dfFalseNorthing : double
                                           ) : TOGRErr; cdecl; external External_Lib name 'OSRSetGaussSchreiberTMercator';
  { Gnomonic }
  function   OSRSetGnomonic(hSRS : TOGRSpatialReferenceH;
                            dfCenterLat, dfCenterLong,
                            dfFalseEasting, dfFalseNorthing : double
                            ) : TOGRErr; cdecl; external External_Lib name 'OSRSetGnomonic';
  { Hotine Oblique Mercator using azimuth angle }
  function   OSRSetHOM(hSRS :  TOGRSpatialReferenceH;
                       dfCenterLat, dfCenterLong,
                       dfAzimuth, dfRectToSkew,
                       dfScale,
                       dfFalseEasting, dfFalseNorthing : double
                       ) : TOGRErr; cdecl; external External_Lib name 'OSRSetHOM';
  { Hotine Oblique Mercator using two points on centerline }
  function   OSRSetHOM2PNO(hSRS :  TOGRSpatialReferenceH;
                           dfCenterLat,
                           dfLat1, dfLong1,
                           dfLat2, dfLong2,
                           dfScale,
                           dfFalseEasting, dfFalseNorthing : double
                           ) : TOGRErr; cdecl; external External_Lib name 'OSRSetHOM2PNO';
  { International Map of the World Polyconic }
  function   OSRSetIWMPolyconic(hSRS :  TOGRSpatialReferenceH;
                                dfLat1, dfLat2,
                                dfCenterLong,
                                dfFalseEasting,
                                dfFalseNorthing : double
                                ) : TOGRErr; cdecl; external External_Lib name 'OSRSetIWMPolyconic';

  { Krovak Oblique Conic Conformal }
  function   OSRSetKrovak(hSRS :  TOGRSpatialReferenceH;
                          dfCenterLat, dfCenterLong,
                          dfAzimuth, dfPseudoStdParallelLat,
                          dfScale,
                          dfFalseEasting,
                          dfFalseNorthing : double
                          ) : TOGRErr; cdecl; external External_Lib name 'OSRSetKrovak';
  { Lambert Azimuthal Equal-Area }
  function   OSRSetLAEA(hSRS :  TOGRSpatialReferenceH;
                        dfCenterLat, dfCenterLong,
                        dfFalseEasting, dfFalseNorthing : double
                        ) : TOGRErr; cdecl; external External_Lib name 'OSRSetLAEA';
  { Lambert Conformal Conic }
  function   OSRSetLCC(hSRS :  TOGRSpatialReferenceH;
                       dfStdP1, dfStdP2,
                       dfCenterLat, dfCenterLong,
                       dfFalseEasting, dfFalseNorthing : double
                       ) : TOGRErr; cdecl; external External_Lib name 'OSRSetLCC';

  { Lambert Conformal Conic 1SP }
  function   OSRSetLCC1SP(hSRS :  TOGRSpatialReferenceH;
                          dfCenterLat, dfCenterLong,
                          dfScale,
                          dfFalseEasting,
                          dfFalseNorthing : double
                          ) : TOGRErr; cdecl; external External_Lib name 'OSRSetLCC1SP';

  { Lambert Conformal Conic (Belgium) }
  function   OSRSetLCCB(hSRS :  TOGRSpatialReferenceH;
                        dfStdP1, dfStdP2,
                        dfCenterLat, dfCenterLong,
                        dfFalseEasting, dfFalseNorthing : double
                        ) : TOGRErr; cdecl; external External_Lib name 'OSRSetLCCB';
  { Miller Cylindrical }
  function   OSRSetMC(hSRS :  TOGRSpatialReferenceH;
                      dfCenterLat, dfCenterLong,
                      dfFalseEasting, dfFalseNorthing : double
                      ) : TOGRErr; cdecl; external External_Lib name 'OSRSetMC';

  { Mercator }
  function   OSRSetMercator(hSRS :  TOGRSpatialReferenceH;
                            dfCenterLat, dfCenterLong,
                            dfScale,
                            dfFalseEasting, dfFalseNorthing : double
                            ) : TOGRErr; cdecl; external External_Lib name 'OSRSetMercator';

  { Mollweide }
  function    OSRSetMollweide(hSRS :  TOGRSpatialReferenceH;
                              dfCentralMeridian,
                              dfFalseEasting,
                              dfFalseNorthing : double
                              ) : TOGRErr; cdecl; external External_Lib name 'OSRSetMollweide';

  { New Zealand Map Grid }
  function   OSRSetNZMG(hSRS :  TOGRSpatialReferenceH;
                        dfCenterLat, dfCenterLong,
                        dfFalseEasting, dfFalseNorthing : double
                        ) : TOGRErr; cdecl; external External_Lib name 'OSRSetNZMG';

  { Oblique Stereographic }
  function   OSRSetOS(hSRS :  TOGRSpatialReferenceH;
                      dfOriginLat, dfCMeridian,
                      dfScale,
                      dfFalseEasting, dfFalseNorthing : double
                      ) : TOGRErr; cdecl; external External_Lib name 'OSRSetOS';
  { Orthographic }
  function   OSRSetOrthographic(hSRS :  TOGRSpatialReferenceH;
                                dfCenterLat, dfCenterLong,
                                dfFalseEasting,
                                dfFalseNorthing : double
                                ) : TOGRErr; cdecl; external External_Lib name 'OSRSetOrthographic';
  { Polyconic }
  function   OSRSetPolyconic(hSRS :  TOGRSpatialReferenceH;
                             dfCenterLat, dfCenterLong,
                             dfFalseEasting, dfFalseNorthing : double
                             ) : TOGRErr; cdecl; external External_Lib name 'OSRSetPolyconic';
  { Polar Stereographic }
  function   OSRSetPS(hSRS :  TOGRSpatialReferenceH;
                      dfCenterLat, dfCenterLong,
                      dfScale,
                      dfFalseEasting, dfFalseNorthing : double
                      ) : TOGRErr; cdecl; external External_Lib name 'OSRSetPS';
  { Robinson }
  function   OSRSetRobinson(hSRS :  TOGRSpatialReferenceH;
                            dfCenterLong,
                            dfFalseEasting, dfFalseNorthing : double
                            ) : TOGRErr; cdecl; external External_Lib name 'OSRSetRobinson';
  { Sinusoidal }
  function   OSRSetSinusoidal(hSRS :  TOGRSpatialReferenceH;
                              dfCenterLong,
                              dfFalseEasting,
                              dfFalseNorthing : double
                              ) : TOGRErr; cdecl; external External_Lib name 'OSRSetSinusoidal';

  { Stereographic }
  function   OSRSetStereographic(hSRS :  TOGRSpatialReferenceH;
                                 dfCenterLat, dfCenterLong,
                                 dfScale,
                                 dfFalseEasting,
                                 dfFalseNorthing : double
                                 ) : TOGRErr; cdecl; external External_Lib name 'OSRSetStereographic';

  { Swiss Oblique Cylindrical }
  function   OSRSetSOC(hSRS :  TOGRSpatialReferenceH;
                       dfLatitudeOfOrigin, dfCentralMeridian,
                       dfFalseEasting, dfFalseNorthing : double
                       ) : TOGRErr; cdecl; external External_Lib name 'OSRSetSOC';

  { Transverse Mercator }
  function   OSRSetTM(hSRS :  TOGRSpatialReferenceH;
                      dfCenterLat, dfCenterLong,
                      dfScale,
                      dfFalseEasting, dfFalseNorthing : double
                      ) : TOGRErr; cdecl; external External_Lib name 'OSRSetTM';

  { Transverse Mercator variant }
  function   OSRSetTMVariant(hSRS :  TOGRSpatialReferenceH;
                             pszVariantName : PCHAR;
                             dfCenterLat, dfCenterLong,
                             dfScale,
                             dfFalseEasting, dfFalseNorthing : double
                             ) : TOGRErr; cdecl; external External_Lib name 'OSRSetTMVariant';

  { Tunesia Mining Grid  }
  function   OSRSetTMG(hSRS :  TOGRSpatialReferenceH;
                       dfCenterLat, dfCenterLong,
                       dfFalseEasting, dfFalseNorthing : double
                       ) : TOGRErr; cdecl; external External_Lib name 'OSRSetTMG';

  { Transverse Mercator (South Oriented) }
  function   OSRSetTMSO(hSRS :  TOGRSpatialReferenceH;
                        dfCenterLat, dfCenterLong,
                        dfScale,
                        dfFalseEasting, dfFalseNorthing : double
                        ) : TOGRErr; cdecl; external External_Lib name 'OSRSetTMSO';

  { VanDerGrinten }
  function   OSRSetVDG(hSRS :  TOGRSpatialReferenceH;
                       dfCenterLong,
                       dfFalseEasting, dfFalseNorthing : double
                       ) : TOGRErr; cdecl; external External_Lib name 'OSRSetVDG';

  { Wagner I -- VII }
  function   OSRSetWagner(hSRS :  TOGRSpatialReferenceH; nVariation : longint;
                          dfFalseEasting,
                          dfFalseNorthing : double
                          ) : TOGRErr; cdecl; external External_Lib name 'OSRSetWagner';

{ Cleanup cached SRS related memory.
  
  
  
  This function will attempt to cleanup any cache spatial
  reference related information, such as cached tables of
  coordinate systems.                                     }
procedure OSRCleanup; cdecl; external External_Lib name 'OSRCleanup';


{ * Fetch list of possible projection methods. * }

function OPTGetProjectionMethods : PPCHAR; cdecl; external External_Lib name 'OPTGetProjectionMethods';
{ Fetch the parameters for a given projection method.
  
  
  
  Parameters
  
  pszProjectionMethod : internal name of projection methods to
  fetch the parameters for, such as "Transverse_Mercator"
  (SRS_PT_TRANSVERSE_MERCATOR).
  
  ppszUserName : pointer in which to return a user visible name
  for the projection name. The returned string should not be
  modified or freed by the caller. Legal to pass in NULL if
  user name not required.                                       }
function OPTGetParameterList(pszProjectionMethod : PCHAR;
                             ppszUserName : PPCHAR
                             ) : PPCHAR; cdecl; external External_Lib name 'OPTGetParameterList';
{ Fetch information about a single parameter of a projection
  method.
  
  
  
  Parameters
  
  pszProjectionMethod : name of projection method for which the
  parameter applies. Not currently used, but in the future this
  could affect defaults. This is the internal projection method
  name, such as "Tranverse_Mercator".
  
  pszParameterName : name of the parameter to fetch information
  about. This is the internal name such as "central_meridian"
  (SRS_PP_CENTRAL_MERIDIAN).
  
  ppszUserName : location at which to return the user visible
  name for the parameter. This pointer may be NULL to skip the
  user name. The returned name should not be modified or freed.
  
  ppszType : location at which to return the parameter type for
  the parameter. This pointer may be NULL to skip. The returned
  type should not be modified or freed. The type values are
  described above.
  
  pdfDefaultValue : location at which to put the default value
  for this parameter. The pointer may be NULL.
  
  
  
  Returns
  
  TRUE if parameter found, or FALSE otherwise.
  
  
                                                                }
function OPTGetParameterInfo(pszProjectionMethod : PCHAR;
                             pszParameterName : PCHAR;
                             ppszUserName : PPCHAR;
                             ppszType : PPCHAR;
                             pdfDefaultValue : PDOUBLE
                             ) : longint; cdecl; external External_Lib name 'OPTGetParameterInfo';


  { Create transformation object.
    
    This is the same as the C++ function
    OGRCreateCoordinateTransformation().
    
    \Input spatial reference system objects are assigned by copy
    (calling clone() method) and no ownership transfer occurs.
    
    OCTDestroyCoordinateTransformation() should be used to
    destroy transformation objects.
    
    The PROJ.4 library must be available at run-time.
    
    
    
    Parameters
    
    hSourceSRS : source spatial reference system.
    
    hTargetSRS : target spatial reference system.
    
    
    
    Returns
    
    NULL on failure or a ready to use transformation object.     }
  function  OCTNewCoordinateTransformation (hSourceSRS : TOGRCoordinateTransformationH;
                                            hTargetSRS : TOGRCoordinateTransformationH
                                            ) : TOGRCoordinateTransformationH ;
                                            stdcall; external External_Lib name '_OCTNewCoordinateTransformation@8';
  { OGRCoordinateTransformation destructor.  }
  procedure OCTDestroyCoordinateTransformation (const Handle : TOGRCoordinateTransformationH); stdcall; external External_Lib name '_OCTDestroyCoordinateTransformation@4';
  { \ \ 
    Parameters
    Handle :      _nt_
    PointCount :  _nt_
    X :           _nt_
    Y :           _nt_
    Z :           _nt_
    pSuccess :    _nt_ }
  function  OCTTransformEx (const Handle : TOGRCoordinateTransformationH; const PointCount : longint;
                                 X, Y, Z : PDOUBLE;
                                 pSuccess : PINTEGER) : longint; stdcall; external External_Lib name '_OCTTransformEx@24';
  { \ \ 
    Parameters
    Handle :      _nt_
    PointCount :  _nt_
    X :           _nt_
    Y :           _nt_
    Z :           _nt_ }
  function  OCTTransform (const Handle : TOGRCoordinateTransformationH; const PointCount : longint;
                                 X, Y, Z : PDOUBLE
                                 ) : longint; stdcall; external External_Lib name '_OCTTransform@20';
                                 
//  function OCTProj4Normalize(pszProj4Src: PChar): PChar; cdecl; external External_Lib name 'OCTProj4Normalize';
{$ELSE}
var
  OGR_G_CreateFromWkb: function (pabyData: PChar; hSRS: TOGRSpatialReferenceH; phGeometry: TOGRGeometryH; nBytes: Integer): TOGRErr; cdecl;
  OGR_G_DestroyGeometry: procedure (hGeom: TOGRGeometryH); cdecl;
  OGR_G_CreateFromWkt: function (ppszData: PPChar; poSR: TOGRSpatialReferenceH; ppoReturn: TOGRGeometryH): TOGRErr; cdecl;
  OGR_G_CreateFromFgf: function ( Fgf: PChar; Ref: TOGRSpatialReferenceH; hGeom: TOGRGeometryH; nBytes: integer; var nBytesConsumed: Integer): TOGRErr; cdecl;
  
  OGR_G_CreateGeometry: function (GeomType: TOGRwkbGeometryType): TOGRGeometryH; cdecl;
  OGR_G_GetDimension: function (hGeom: TOGRGeometryH ): integer; cdecl;
  OGR_G_GetCoordinateDimension: function (hGeom: TOGRGeometryH ): Integer; cdecl;
  OGR_G_SetCoordinateDimension: procedure (hGeom: TOGRGeometryH; nNewDimension: Integer ); cdecl;
  OGR_G_Clone: function (hGeom: TOGRGeometryH ): TOGRGeometryH; cdecl;
  OGR_G_GetEnvelope: procedure (hGeom: TOGRGeometryH; var psEnvelope: TOGREnvelope); cdecl;
  OGR_G_ImportFromWkb: function (hGeom: TOGRGeometryH; pabyData: PChar; nSize: Integer): TOGRErr; cdecl;
  OGR_G_ExportToWkb: function (hGeom: TOGRGeometryH; ByteOrder: TOGRwkbByteOrder; Wkb: PChar): TOGRErr; cdecl;
  OGR_G_WkbSize: function (hGeom: TOGRGeometryH): Integer; cdecl;
  OGR_G_ImportFromWkt: function (hGeom: TOGRGeometryH; ppszSrcText: PPChar): TOGRErr; cdecl;
  OGR_G_ExportToWkt: function (hGeom: TOGRGeometryH; Wkt: PPChar ): TOGRErr; cdecl;
  OGR_G_GetGeometryType: function (hGeom: TOGRGeometryH): TOGRwkbGeometryType; cdecl;
  OGR_G_GetGeometryName: function (hGeom: TOGRGeometryH): PChar; cdecl;
  OGR_G_DumpReadable: procedure (hGeom: TOGRGeometryH; var pfp : file; pszPrefix: pChar ); cdecl;
  OGR_G_FlattenTo2D: procedure (hGeom: TOGRGeometryH ); cdecl;
  OGR_G_CloseRings: procedure (hGeom: TOGRGeometryH ); cdecl;

  OGR_G_CreateFromGML: function (GML: PChar): TOGRGeometryH; cdecl;
  OGR_G_ExportToGML: function (hGeom: TOGRGeometryH ): PChar; cdecl;

  OGR_G_ExportToKML: function (hGeom: TOGRGeometryH; pszAltitudeMode: PChar): PChar; cdecl;

  OGR_G_ExportToJson: function (hGeom: TOGRGeometryH ): PChar; cdecl;
  OGR_G_CreateGeometryFromJson: function (Json: PChar): TOGRGeometryH; cdecl;
  OGR_G_AssignSpatialReference: procedure (hGeom: TOGRGeometryH;
                                             hRef: TOGRSpatialReferenceH ); cdecl; 
  OGR_G_GetSpatialReference: function (hGeom: TOGRGeometryH): TOGRSpatialReferenceH; cdecl;
  OGR_G_Transform: function (hGeom: TOGRGeometryH; hTransform: TOGRCoordinateTransformationH ): TOGRErr; cdecl;
  OGR_G_TransformTo: function (hGeom: TOGRGeometryH; hRef: TOGRSpatialReferenceH ): TOGRErr; cdecl;
  OGR_G_Segmentize: procedure (hGeom: TOGRGeometryH; dfMaxLength: Double ); cdecl;
  OGR_G_Intersects: function (hGeom: TOGRGeometryH; hOtherGeom: TOGRGeometryH ): Integer;
  OGR_G_Equals: function (hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl;
  OGR_G_Disjoint: function (hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl;
  OGR_G_Touches: function (hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl;
  OGR_G_Crosses: function (hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl;
  OGR_G_Within: function (hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl;
  OGR_G_Contains: function (hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl;
  OGR_G_Overlaps: function (hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl;
  OGR_G_GetBoundary: function (hGeom: TOGRGeometryH): TOGRGeometryH; cdecl;
  OGR_G_ConvexHull: function (hTarget: TOGRGeometryH): TOGRGeometryH; cdecl;
  OGR_G_Buffer: function (hGeom: TOGRGeometryH; dfDist: double; nQuadSegs: integer): TOGRGeometryH; cdecl;
  OGR_G_Intersection: function (hGeom: TOGRGeometryH; hOther: TOGRGeometryH): TOGRGeometryH; cdecl;
  OGR_G_Union: function (hGeom1: TOGRGeometryH; hGeom2: TOGRGeometryH): TOGRGeometryH; cdecl;
  OGR_G_Difference: function (hGeom1: TOGRGeometryH; hGeom2: TOGRGeometryH): TOGRGeometryH; cdecl;
  OGR_G_SymmetricDifference: function (hGeom1: TOGRGeometryH; hGeom2: TOGRGeometryH): TOGRGeometryH; cdecl;
  OGR_G_Distance: function (hGeom1: TOGRGeometryH; hGeom2: TOGRGeometryH): Double; cdecl;
  OGR_G_GetArea: function (hGeom: TOGRGeometryH ): Double;cdecl;
  OGR_G_Centroid: function (hPolygon: TOGRGeometryH; hCentroidPoint: TOGRGeometryH): Integer;cdecl;
  OGR_G_Empty: procedure (hGeom: TOGRGeometryH); cdecl;
  OGR_G_IsEmpty: function  (hGeom: TOGRGeometryH): Integer; cdecl;
  OGR_G_IsValid: function (hGeom: TOGRGeometryH): Integer; cdecl;
  OGR_G_IsSimple: function (hGeom: TOGRGeometryH): Integer; cdecl;
  OGR_G_IsRing: function (hGeom: TOGRGeometryH): Integer; cdecl;

  OGR_G_Intersect: function (hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl;
  OGR_G_Equal: function (hGeom: TOGRGeometryH; hOther: TOGRGeometryH ): Integer; cdecl;
  OGR_G_GetPointCount: function (hGeom: TOGRGeometryH ): Integer; cdecl;
  OGR_G_GetX: function (hGeom: TOGRGeometryH; i: Integer): Double; cdecl;
  OGR_G_GetY: function (hGeom: TOGRGeometryH; i: Integer): Double; cdecl;
  OGR_G_GetZ: function (hGeom: TOGRGeometryH; i: Integer): Double; cdecl;
  OGR_G_GetPoint: procedure (hGeom: TOGRGeometryH; iPoint: Integer;  var pdfX: double; var pdfY:double;  var pdfZ: double); cdecl;
  OGR_G_SetPoint: procedure (hGeom: TOGRGeometryH; iPoint: Integer;  dfX, dfY, dfZ: double); cdecl;
  OGR_G_SetPoint_2D: procedure (hGeom: TOGRGeometryH; iPoint: Integer;  dfX, dfY :double); cdecl;
  OGR_G_AddPoint: procedure (hGeom: TOGRGeometryH; dfX, dfY, dfZ: Double ); cdecl;
  OGR_G_AddPoint_2D: procedure (hGeom: TOGRGeometryH; dfX, dfY: double ); cdecl;


  OGR_G_GetGeometryCount: function (hGeom: TOGRGeometryH ): Integer; cdecl;
  OGR_G_GetGeometryRef: function (hGeom: TOGRGeometryH; iSubGeom: Integer ): TOGRGeometryH; cdecl;
  OGR_G_AddGeometry: function (hGeom: TOGRGeometryH; hNewSubGeom: TOGRGeometryH ): TOGRErr; cdecl;
  OGR_G_AddGeometryDirectly: function (hGeom: TOGRGeometryH; hNewSubGeom: TOGRGeometryH ): TOGRErr; cdecl;
  OGR_G_RemoveGeometry: function (hGeom: TOGRGeometryH; iGeom, bDelete: Integer): TOGRErr; cdecl;
  OGRBuildPolygonFromEdges: function (hLinesAsCollection: TOGRGeometryH;
                                       bBestEffort: Integer;
                                       bAutoClose: Integer;
                                       dfTolerance: Integer;
                                       Var peErr: TOGRErr ): TOGRGeometryH; cdecl;
  OGRSetGenerate_DB2_V72_BYTE_ORDER: function (bGenerate_DB2_V72_BYTE_ORDER: Integer): TOGRErr; cdecl;

  OGRGetGenerate_DB2_V72_BYTE_ORDER: function : Integer; cdecl;


  OGRFree: procedure (p : POINTER); cdecl;
  
  OGR_Fld_Create: function ( pszName: PChar; hType: OGRFieldType ): OGRFieldDefnH; cdecl;
  OGR_Fld_Destroy: procedure (hField: OGRFieldDefnH ); cdecl;
  OGR_Fld_SetName: procedure (hField: OGRFieldDefnH; pszName: PChar); cdecl;
  OGR_Fld_GetNameRef: function (hField: OGRFieldDefnH ): PChar; cdecl;
  OGR_Fld_GetType: function (hField: OGRFieldDefnH ): OGRFieldType; cdecl;
  OGR_Fld_SetType: procedure (hField: OGRFieldDefnH; hType: OGRFieldType ); cdecl;
  OGR_Fld_GetJustify: function (hField: OGRFieldDefnH ): OGRJustification; cdecl;
  OGR_Fld_SetJustify: procedure (hField: OGRFieldDefnH; hJ: OGRJustification ); cdecl;
  OGR_Fld_GetWidth: function (hField: OGRFieldDefnH ): Integer; cdecl;
  OGR_Fld_SetWidth: procedure (hField: OGRFieldDefnH; iWidth: Integer ); cdecl;
  OGR_Fld_GetPrecision: function (hField: OGRFieldDefnH ): Integer; cdecl;
  OGR_Fld_SetPrecision: procedure (hField: OGRFieldDefnH; hP: Integer); cdecl;
  OGR_Fld_Set: procedure (hField: OGRFieldDefnH; pszName: PChar; hType: OGRFieldType;
                            iWidth, iPrecision: Integer; Justification: OGRJustification ); cdecl;

  OGR_GetFieldTypeName: function (hType: OGRFieldType ): PChar; cdecl;

  OGR_F_Create: function (hFd: OGRFeatureDefnH ): OGRFeatureH; cdecl;
  OGR_F_Destroy: procedure (hF: OGRFeatureH ); cdecl;
  OGR_F_GetDefnRef: function (hF: OGRFeatureH ): OGRFeatureDefnH; cdecl;
  OGR_F_SetGeometryDirectly: function ( hF: OGRFeatureH; hGeom: OGRGeometryH ): OGRErr; cdecl;
  OGR_F_SetGeometry : function (hF: OGRFeatureH; hGeom: OGRGeometryH ): OGRErr; cdecl;
  OGR_F_GetGeometryRef: function  (hF: OGRFeatureH ):OGRGeometryH; cdecl;
  OGR_F_Clone: function  (hF: OGRFeatureH ):OGRFeatureH; cdecl;
  OGR_F_Equal: function (hF1: OGRFeatureH; hF2: OGRFeatureH): Integer; cdecl;
  OGR_F_GetFieldCount: function (hF: OGRFeatureH ): Integer; cdecl;
  OGR_F_GetFieldDefnRef: function (hF: OGRFeatureH; iField: Integer): OGRFieldDefnH; cdecl;
  OGR_F_GetFieldIndex: function (hF: OGRFeatureH; pszFldName: PChar): Integer; cdecl;
  OGR_F_IsFieldSet: function (hF: OGRFeatureH; iField: Integer ): Integer; cdecl;
  OGR_F_UnsetField: procedure (hF: OGRFeatureH; iField: Integer); cdecl;
  OGR_F_GetRawFieldRef: function (hF: OGRFeatureH; iField: Integer): OGRField; cdecl;
  OGR_F_GetFieldAsInteger: function (hF: OGRFeatureH; iField: Integer): Integer; cdecl;
  OGR_F_GetFieldAsDouble: function (hF: OGRFeatureH; iField: Integer): Double; cdecl;
  OGR_F_GetFieldAsString: function (hF: OGRFeatureH; iField: Integer): PChar; cdecl;
  OGR_F_GetFieldAsIntegerList: function (hF: OGRFeatureH; iField: Integer; var pnCount: Integer): PIntegerArray; cdecl;
  OGR_F_GetFieldAsDoubleList: function (hF: OGRFeatureH; iField: Integer; var pnCount: Integer): PDoubleArray; cdecl;
  OGR_F_GetFieldAsStringList: function (hF: OGRFeatureH; iField: Integer): PPChar; cdecl;
  OGR_F_GetFieldAsBinary: function (hF: OGRFeatureH; iField: Integer; var pnBytes: Integer): PByteArray; cdecl;
  OGR_F_GetFieldAsDateTime: function (hF: OGRFeatureH; iField: Integer; var pnYear, pnMonth, pnDay, pnHour, pnMinute, pnSecond, pnTZFlag: Integer): Integer; cdecl;
  OGR_F_SetFieldInteger: procedure    ( hF: OGRFeatureH; iField: Integer; Value: Integer ); cdecl;
  OGR_F_SetFieldDouble: procedure    ( hF: OGRFeatureH; iField: Integer; Value: double ); cdecl;
  OGR_F_SetFieldString: procedure    ( hF: OGRFeatureH; iField: Integer; Value: PChar ); cdecl;
  OGR_F_SetFieldIntegerList: procedure    ( hF: OGRFeatureH; iField: Integer; nCount: Integer; panValues: PIntegerArray ); cdecl;
  OGR_F_SetFieldDoubleList: procedure    ( hF: OGRFeatureH; iField: Integer; nCount: Integer; panValues: PDoubleArray );cdecl;
  OGR_F_SetFieldStringList: procedure    ( hF: OGRFeatureH; iField: Integer; panValues: PPChar );cdecl;
  OGR_F_SetFieldRaw: procedure    ( hF: OGRFeatureH; iField: Integer; hField: OGRField); cdecl;
  OGR_F_SetFieldBinary: procedure    ( hF: OGRFeatureH; iField: Integer; nBytes: Integer; panValues: PByteArray ); cdecl;
  OGR_F_SetFieldDateTime: procedure    ( hF: OGRFeatureH; iField: Integer;
                                       pnYear, pnMonth, pnDay, pnHour, pnMinute, pnSecond, pnTZFlag: Integer ); cdecl;
  OGR_F_GetFID: function (hF: OGRFeatureH ): Cardinal; cdecl;
  OGR_F_SetFID: function  (hF: OGRFeatureH; fID: Cardinal): OGRErr; cdecl;
  OGR_F_DumpReadable: procedure    (hF: OGRFeatureH; var pfp: File); cdecl;
  OGR_F_SetFrom: function  (hFeat, hOtherFeat: OGRFeatureH; bForgiving: Integer):OGRErr; cdecl;
  OGR_F_GetStyleString: function (hF:  OGRFeatureH ): PChar; cdecl;
  OGR_F_SetStyleString: procedure    (hF: OGRFeatureH; pszStyle: PChar); cdecl;
  OGR_F_SetStyleStringDirectly: procedure    (hF: OGRFeatureH; pszStyle: PChar); cdecl; 

OGR_L_GetSpatialFilter: function  (hLayer: OGRLayerH ): OGRGeometryH; cdecl;
OGR_L_SetSpatialFilter: procedure    (hLayer: OGRLayerH; hGeom: OGRGeometryH ); cdecl;
OGR_L_SetSpatialFilterRect: procedure    (hLayer: OGRLayerH;
                                           dfMinX, dfMinY, dfMaxX, fdMaxY: Double ); cdecl;
OGR_L_SetAttributeFilter: function (hLayer: OGRLayerH; pszName: PChar ):OGRErr; cdecl;
OGR_L_ResetReading: procedure (hLayer: OGRLayerH ); cdecl;
OGR_L_GetNextFeature: function (hLayer: OGRLayerH ): OGRFeatureH; cdecl; 
OGR_L_SetNextByIndex: function (hLayer: OGRLayerH;  nIndex: long ):OGRErr; cdecl;
OGR_L_GetFeature: function (hLayer: OGRLayerH; nFID: long ): OGRFeatureH; cdecl;
OGR_L_SetFeature: function (hLayer: OGRLayerH; hF: OGRFeatureH ): OGRErr; cdecl; 
OGR_L_CreateFeature: function (hLayer: OGRLayerH; hF:  OGRFeatureH ): OGRErr; cdecl;
OGR_L_DeleteFeature: function (hLayer: OGRLayerH;  nFID: long): OGRErr; cdecl;
OGR_L_GetLayerDefn: function (hLayer: OGRLayerH):OGRFeatureDefnH; cdecl;
OGR_L_GetSpatialRef: function  (hLayer: OGRLayerH): OGRSpatialReferenceH; cdecl;
OGR_L_GetFeatureCount: function (hLayer: OGRLayerH; bForce: Integer ): Integer;cdecl;
OGR_L_GetExtent: function (hLayer: OGRLayerH;  var hEnv: OGREnvelope; bForce: Integer):OGRErr; cdecl;
OGR_L_TestCapability: function (hLayer: OGRLayerH;  pszName: PChar ): Integer; cdecl;
OGR_L_CreateField: function (hLayer: OGRLayerH; hField: OGRFieldDefnH; bApproxOk: Integer):OGRErr; cdecl;
OGR_L_StartTransaction: function (hLayer: OGRLayerH): OGRErr; cdecl;
OGR_L_CommitTransaction: function (hLayer: OGRLayerH): OGRErr; cdecl;
OGR_L_RollbackTransaction: function (hLayer: OGRLayerH): OGRErr; cdecl;
OGR_L_Reference: function (hLayer: OGRLayerH): Integer; cdecl;
OGR_L_Dereference: function (hLayer: OGRLayerH): Integer; cdecl;
OGR_L_GetRefCount: function (hLayer: OGRLayerH): Integer; cdecl;
OGR_L_SyncToDisk: function (hLayer: OGRLayerH):OGRErr; cdecl;
OGR_L_GetFeaturesRead: function (hLayer: OGRLayerH): GIntBig; cdecl;
OGR_L_GetFIDColumn: function (hLayer: OGRLayerH): PChar; cdecl;
OGR_L_GetGeometryColumn: function (hLayer: OGRLayerH): PChar; cdecl;

OGR_DS_Destroy: procedure    (hDS: OGRDataSourceH ); cdecl;
OGR_DS_GetName: function (hDS: OGRDataSourceH ): PChar; cdecl;
OGR_DS_GetLayerCount: function (hDS: OGRDataSourceH ): Integer; cdecl;
OGR_DS_GetLayer: function (hDS: OGRDataSourceH; iLayer: Integer): OGRLayerH  ; cdecl;
OGR_DS_GetLayerByName: function (hDS: OGRDataSourceH; pszName: PChar ):OGRLayerH  ; cdecl;
OGR_DS_DeleteLayer: function (hDS: OGRDataSourceH; iLayer: Integer ): OGRErr; cdecl;
OGR_DS_GetDriver: function (hDS: OGRDataSourceH ): OGRSFDriverH ; cdecl;
OGR_DS_CreateLayer: function (hDS: OGRDataSourceH; pszName: PChar;
                                      hRef: OGRSpatialReferenceH; eGType: TOGRwkbGeometryType;
                                      papszOptions: PPChar ): OGRLayerH; cdecl;
OGR_DS_CopyLayer: function (hDS: OGRDataSourceH; poSrcLayer: OGRLayerH; pszNewName: PChar;
                                    papszOptions: PPChar): OGRLayerH; cdecl;

OGR_DS_TestCapability: function (hDS: OGRDataSourceH; pszName: PChar ): Integer; cdecl;
OGR_DS_ExecuteSQL: function (hDS: OGRDataSourceH; pszStatement: PChar;
                                     poSpatialFilter: OGRGeometryH; pszDialect: PChar): OGRLayerH ; cdecl;
OGR_DS_ReleaseResultSet: procedure    (hDS: OGRDataSourceH; hLayer: OGRLayerH ); cdecl;
OGR_DS_Reference: function (hDS: OGRDataSourceH ): Integer; cdecl;
OGR_DS_Dereference: function (hDS: OGRDataSourceH ): Integer; cdecl;
OGR_DS_GetRefCount: function (hDS: OGRDataSourceH ): Integer; cdecl;
OGR_DS_GetSummaryRefCount: function (hDS: OGRDataSourceH ): Integer; cdecl;
OGR_DS_SyncToDisk: function (hDS: OGRDataSourceH ):OGRErr  ; cdecl;

OGR_Dr_GetName:function (hDriver: TOGRSFDriverH ): PChar; cdecl; 
OGR_Dr_Open: function (hDriver: TOGRSFDriverH; pszName: pChar; pUpdate: Integer): TOGRDataSourceH; cdecl;
OGR_Dr_TestCapability: function (hDriver: TOGRSFDriverH; pszCap: PChar): Integer; cdecl;
OGR_Dr_CreateDataSource: function (hDriver: TOGRSFDriverH; pszName: PChar; papszOptions: PPChar ): TOGRDataSourceH; cdecl;
OGR_Dr_CopyDataSource: function (hDriver: TOGRSFDriverH; hSource:  TOGRDataSourceH;
                                              pszNewName: PChar; papszOptions: PPChar ): TOGRDataSourceH; cdecl;
OGR_Dr_DeleteDataSource: function (hDriver: TOGRSFDriverH; pszDataSource: PChar): TOGRErr; cdecl;


OGROpen: function (pszName: PChar; bUpdate: Integer; hDriver: TOGRSFDriverH): TOGRSFDriverH; cdecl;
OGROpenShared: function ( pszName: PChar; bUpdate: Integer; hDriver: TOGRSFDriverH): TOGRDataSourceH; cdecl;
OGRReleaseDataSource: function (hDS: TOGRDataSourceH ): TOGRErr; cdecl;
OGRRegisterDriver: procedure (hDriver: TOGRSFDriverH ); cdecl;
OGRGetDriverCount:  function : Integer; cdecl;
OGRGetDriver: function (iDrv: Integer): TOGRSFDriverH; cdecl;
OGRGetDriverByName: function ( lpszDrvName: PChar): TOGRSFDriverH; cdecl;
OGRGetOpenDSCount:  function : Integer; cdecl;
OGRGetOpenDS:  function (iDS: Integer): TOGRDataSourceH; cdecl;

   OGRRegisterAll: procedure ; cdecl;
   OGRCleanupAll: procedure ; cdecl; 




  OSRNewSpatialReference: function (const WKT : PCHAR
                                 ) : TOGRSpatialReferenceH;
                                 stdcall;
  OSRCloneGeogCS: function (const Handle : TOGRSpatialReferenceH
                         ) : TOGRSpatialReferenceH;
                         stdcall;

  OSRClone: function  (const Handle : TOGRSpatialReferenceH ) : TOGRSpatialReferenceH ;
                    stdcall;
  OSRDestroySpatialReference: procedure  (const Handle : TOGRSpatialReferenceH); stdcall;

  OSRReference: function  (hSRS : TOGRSpatialReferenceH) : longint; cdecl;
  OSRDereference: function  (hSRS : TOGRSpatialReferenceH) : longint; cdecl;
  OSRRelease: procedure (hSRS : TOGRSpatialReferenceH ); cdecl;
  OSRValidate: function  (hSRS : TOGRSpatialReferenceH ) : TOGRErr; cdecl;
  OSRFixupOrdering: function  (hSRS : TOGRSpatialReferenceH) : TOGRErr; cdecl;
  OSRFixup: function  (hSRS : TOGRSpatialReferenceH ) : TOGRErr; cdecl;
  OSRStripCTParms: function  (hSRS : TOGRSpatialReferenceH) : TOGRErr; cdecl;
  OSRImportFromEPSG: function  (hSRS : TOGRSpatialReferenceH; EPSGCode : longint
                                 ) : TOGRErr; stdcall;
  OSRImportFromEPSGA: function  (hSRS : TOGRSpatialReferenceH; EPSGCode : longint
                                  ) : TOGRErr; stdcall;
  OSRImportFromWkt: function  (hSRS : TOGRSpatialReferenceH;
                             ppszInput : PPCHAR
                             ) : TOGRErr; cdecl;
  OSRImportFromProj4: function  (hSRS : TOGRSpatialReferenceH;
                               pszProj4 : PCHAR) : TOGRErr; cdecl;
  OSRImportFromESRI: function  (hSRS : TOGRSpatialReferenceH;
                               papszPrj : PPCHAR) : TOGRErr; cdecl;
  OSRImportFromPCI: function  (hSRS : TOGRSpatialReferenceH; pszProj : PCHAR;
		             pszUnits : PCHAR; padfPrjParams : PDOUBLE
                             ) : TOGRErr; cdecl;
  OSRImportFromUSGS: function  (hSRS : TOGRSpatialReferenceH; iProjsys : longint;
                              iZone : longint; padfPrjParams : PDOUBLE;
                              iDatum : longint
                              ) : TOGRErr; cdecl;
  OSRImportFromXML: function  (hSRS : TOGRSpatialReferenceH;
                              pszXML : PCHAR) : TOGRErr; cdecl;

  OSRImportFromDict: function (hSRS: OGRSpatialReferenceH;
                              pszDict :PChar; pszCode: PChar): TOGRErr; cdecl;

  OSRImportFromPanorama: function (hSRS: OGRSpatialReferenceH; iProjSys: long; iDatum: long; iEllips: long;
                                      padfPrjParams: PDouble ): TOGRErr;  cdecl;

  OSRImportFromMICoordSys: function  (hSRS : TOGRSpatialReferenceH;
                                    pszCoordSys : PCHAR
                                   ) : TOGRErr; cdecl;
  OSRImportFromUrl: function  (hSRS : TOGRSpatialReferenceH;
                             pszUrl : PCHAR
                            ) : TOGRErr; cdecl;

  OSRExportToWkt: function  (hSRS : TOGRSpatialReferenceH;
                           ppszReturn : PPCHAR) : TOGRErr; stdcall;
  OSRExportToPrettyWkt: function  (hSRS : TOGRSpatialReferenceH;
                                 ppszReturn : PPCHAR;
                                 bSimplify: longint) : TOGRErr; stdcall;

  OSRExportToProj4: function  (hSRS : TOGRSpatialReferenceH;
                             ppszReturn : PPCHAR) : TOGRErr; stdcall;
  OSRExportToPCI: function  (hSRS : TOGRSpatialReferenceH; ppszProj : PPCHAR;
                           ppszUnits : PPCHAR;
                           ppadfPrjParams : PPDOUBLE
                           ) : TOGRErr; cdecl;
  OSRExportToUSGS: function  (hSRS : TOGRSpatialReferenceH; piProjSys : PINTEGER;
                            piZone : PLONGINT;
                            ppadfPrjParams : PPDOUBLE;
                            piDatum : PLONGINT
                            ) : TOGRErr; cdecl;
  OSRExportToXML: function  (hSRS : TOGRSpatialReferenceH; ppszRawXML : PPCHAR;
                           pszDialect : PCHAR
                           ) : TOGRErr; cdecl;
  OSRExportToPanorama: function (hSRS: TOGRSpatialReferenceH; var piProjSys: long; var piDatum: Long;
                           var piEllips: Long; var piZone: long; padfPrjParams: PDouble
                           ) : TOGRErr; cdecl;

  OSRExportToMICoordSys: function  (hSRS : TOGRSpatialReferenceH;
                                  ppszReturn : PPCHAR) : TOGRErr; cdecl;

  OSRMorphToESRI: function  (hSRS : TOGRSpatialReferenceH) : TOGRErr; cdecl;
  OSRMorphFromESRI: function  (hSRS : TOGRSpatialReferenceH) : TOGRErr; cdecl;
  OSRSetAttrValue: function  (hSRS : TOGRSpatialReferenceH;
                            pszNodePath : PCHAR;
                            pszNewNodeValue : PCHAR) : TOGRErr; stdcall;
  OSRGetAttrValue: function  (hSRS : TOGRSpatialReferenceH;
                            pszName : PCHAR;
                            iChild  : longint(* = 0 *)) : PCHAR; stdcall;
  OSRSetAngularUnits: function  (hSRS : TOGRSpatialReferenceH; pszUnits : PCHAR;
                               dfInRadians : double) : TOGRErr; cdecl;
  OSRGetAngularUnits: function  (hSRS : TOGRSpatialReferenceH;
                               ppszName : PPCHAR) : double; cdecl;
  OSRSetLinearUnits: function  (hSRS : TOGRSpatialReferenceH; pszUnits : PCHAR;
                              dfInMeters : double) : TOGRErr; cdecl;
  OSRSetLinearUnitsAndUpdateParameters: function  (hSRS : TOGRSpatialReferenceH;
                                                 pszUnits : PCHAR;
                                                 dfInMeters : double
                                                 ) : TOGRErr; cdecl;
  OSRGetLinearUnits: function  (hSRS : TOGRSpatialReferenceH;
                              ppszName : PPCHAR) : double; cdecl;
  OSRGetPrimeMeridian: function  (hSRS : TOGRSpatialReferenceH;
                                ppszName : PPCHAR) : double; cdecl;
  OSRIsGeographic: function  (hSRS : TOGRSpatialReferenceH) : longint; cdecl;
  OSRIsLocal: function  (hSRS : TOGRSpatialReferenceH) : longint; cdecl;
  OSRIsProjected: function  (hSRS : TOGRSpatialReferenceH) : longint; cdecl;
  OSRIsSameGeogCS: function  (hSRS1 : TOGRSpatialReferenceH;
                            hSRS2 : TOGRSpatialReferenceH) : longint; cdecl;
  OSRIsSame: function  (hSRS1 : TOGRSpatialReferenceH;
                      hSRS2 : TOGRSpatialReferenceH) : longint; cdecl;
  OSRSetLocalCS: function  (hSRS : TOGRSpatialReferenceH;
                          pszName : PCHAR) : TOGRErr; cdecl;
  OSRSetProjCS: function  (hSRS : TOGRSpatialReferenceH;
                         pszName : PCHAR) : TOGRErr; cdecl;
  OSRSetWellKnownGeogCS: function  (hSRS : TOGRSpatialReferenceH;
                                  pszName : PCHAR) : TOGRErr; cdecl;
  OSRSetFromUserInput: function  (hSRS : TOGRSpatialReferenceH;
                                pszDef : PCHAR): TOGRErr; stdcall;
  OSRCopyGeogCSFrom: function  (hSRS : TOGRSpatialReferenceH;
                              hSrcSRS : TOGRSpatialReferenceH
                              ) : TOGRErr; cdecl;
  OSRSetTOWGS84: function  (hSRS :  TOGRSpatialReferenceH;
                          dfDX, dfDY,
                          dfDZ, dfEX,
                          dfEY, dfEZ,
                          dfPPM : double
                          ) : TOGRErr; cdecl;
  OSRGetTOWGS84: function  (hSRS : TOGRSpatialReferenceH; padfCoeff : PDOUBLE;
                          nCoeffCount : longint) : TOGRErr; cdecl;
  OSRSetGeogCS: function  (hSRS : TOGRSpatialReferenceH;
                         pszGeogName, pszDatumName, pszEllipsoidName : PCHAR;
                         dfSemiMajor, dfInvFlattening : double;
                         pszPMName : PCHAR; (* = NULL *)
                         dfPMOffset : double; (* = 0.0 *)
                         pszUnits : PCHAR;(* = NULL *)
                         dfConvertToRadians : double(* = 0.0 *)
                         ) : TOGRErr; cdecl;

  OSRGetSemiMajor: function  (hSRS : TOGRSpatialReferenceH;
                            pnErr : POGRErr) : double; cdecl;
  OSRGetSemiMinor: function  (hSRS : TOGRSpatialReferenceH;
                            pnErr : POGRErr) : double; cdecl;
  OSRGetInvFlattening: function  (hSRS : TOGRSpatialReferenceH;
                                pnErr : POGRErr) : double; cdecl;
  OSRSetAuthority: function  (hSRS :  TOGRSpatialReferenceH;
                            pszTargetKey : PCHAR;
                            pszAuthority : PCHAR;
                            nCode : longint) : TOGRErr; cdecl;
  OSRGetAuthorityCode: function  (hSRS : TOGRSpatialReferenceH;
                                pszTargetKey : PCHAR) : PCHAR; cdecl;
  OSRGetAuthorityName: function  (hSRS : TOGRSpatialReferenceH;
                                pszTargetKey : PCHAR) : PCHAR; cdecl;
  OSRSetProjection: function  (hSRS : TOGRSpatialReferenceH;
                             pszProjection : PCHAR) : TOGRErr; cdecl;
  OSRSetProjParm: function  (hSRS : TOGRSpatialReferenceH;
                           pszParmName : PCHAR;
                           dfValue : double
                           ) : TOGRErr; cdecl;
  OSRGetProjParm: function  (hSRS :  TOGRSpatialReferenceH;
                           pszParmName : PCHAR;
                           dfDefault : double;
                           pnErr : POGRErr
                           ) : double; cdecl;
  OSRSetNormProjParm: function   (hSRS : TOGRSpatialReferenceH;
                                pszParmName : PCHAR;
                                dfValue : double
                                ) : TOGRErr; cdecl;
  OSRGetNormProjParm: function   (hSRS :  TOGRSpatialReferenceH;
                                pszParmName : PCHAR;
                                dfDefault : double;
                                pnErr : POGRErr
                                ) : double; cdecl; 
  OSRSetUTM: function   (hSRS :  TOGRSpatialReferenceH;
                       nZone : longint;
                       bNorth : longint
                       ) : TOGRErr; cdecl;
  OSRGetUTMZone: function   (hSRS :  TOGRSpatialReferenceH;
                           pbNorth : PINTEGER
                           ) : longint; cdecl;

  { Set State Plane projection definition.  }
  OSRSetStatePlane: function (hSRS: OGRSpatialReferenceH;
                                 nZone: Integer; bNAD83: Integer ): TOGRErr; cdecl;
  { Set State Plane projection definition.  }
  OSRSetStatePlaneWithUnits: function (hSRS: OGRSpatialReferenceH;
                                          nZone: Integer; bNAD83: Integer;
                                          pszOverrideUnitName: PChar;
                                          dfOverrideUnit: double): TOGRErr; cdecl; 
  { Set EPSG authority info if possible.  }
  OSRAutoIdentifyEPSG: function (hSRS: OGRSpatialReferenceH): TOGRErr; cdecl;
  { Fetch the orientation of one axis.  }
  OSRGetAxis: function (hSRS: OGRSpatialReferenceH;
                                pszTargetKey: PChar; iAxis: Integer;
                                peOrientation: TOGRAxisOrientation): PChar; cdecl;


  { Albers Conic Equal Area }
  OSRSetACEA: function   (hSRS :  TOGRSpatialReferenceH;
                        dfStdP1, dfStdP2,
                        dfCenterLat, dfCenterLong,
                        dfFalseEasting, dfFalseNorthing : double
                        ) : TOGRErr; cdecl;

  { Azimuthal Equidistant }
  OSRSetAE: function  (hSRS :  TOGRSpatialReferenceH;
                       dfCenterLat, dfCenterLong,
                       dfFalseEasting, dfFalseNorthing : double
                       ) : TOGRErr; cdecl; 

  { Bonne }
  OSRSetBonne: function   (hSRS : TOGRSpatialReferenceH;
                         dfStandardParallel, dfCentralMeridian,
                         dfFalseEasting, dfFalseNorthing : double
                         ) : TOGRErr; cdecl;

  { Cylindrical Equal Area }
  OSRSetCEA: function (hSRS :  TOGRSpatialReferenceH;
                       dfStdP1, dfCentralMeridian,
                       dfFalseEasting, dfFalseNorthing : double
                       ) : TOGRErr; cdecl; 

  { Cassini-Soldner }
  OSRSetCS: function   (hSRS :  TOGRSpatialReferenceH;
                      dfCenterLat, dfCenterLong,
                      dfFalseEasting, dfFalseNorthing : double
                      ) : TOGRErr; cdecl;

  { Equidistant Conic }
  OSRSetEC: function   (hSRS :  TOGRSpatialReferenceH;
                      dfStdP1, dfStdP2,
                      dfCenterLat, dCenterLong,
                      dfFalseEasting, dfFalseNorthing : double
                      ) : TOGRErr; cdecl;

  { Eckert I-VI }
  OSRSetEckert: function   (hSRS :  TOGRSpatialReferenceH;
                          nVariation : longint;
                          dfCentralMeridian,
                          dfFalseEasting,
                          dfFalseNorthing : double
                          ) : TOGRErr; cdecl;

  { Eckert IV }
  OSRSetEckertIV: function   (hSRS :  TOGRSpatialReferenceH;
                            dfCentralMeridian,
                            dfFalseEasting,
                            dfFalseNorthing : double
                            ) : TOGRErr; cdecl;

  { Eckert VI }
  OSRSetEckertVI: function   (hSRS :  TOGRSpatialReferenceH;
                            dfCentralMeridian,
                            dfFalseEasting,
                            dfFalseNorthing : double
                            ) : TOGRErr; cdecl;

  { Equirectangular }
  OSRSetEquirectangular: function   (hSRS : TOGRSpatialReferenceH;
                                   dfCenterLat, dfCenterLong,
                                   dfFalseEasting, dfFalseNorthing : double
                                   ) : TOGRErr; cdecl;
  { Equirectangular generalized form }
  OSRSetEquirectangular2: function   (hSRS : TOGRSpatialReferenceH;
                                    dfCenterLat, dfCenterLong,
                                    dfPseudoStdParallel1,
                                    dfFalseEasting,
                                    dfFalseNorthing : double
                                    ) : TOGRErr; cdecl;

  { Gall Stereograpic }
  OSRSetGS: function   (hSRS :  TOGRSpatialReferenceH;
                      dfCentralMeridian,
                      dfFalseEasting,
                      dfFalseNorthing : double
                      ) : TOGRErr; cdecl; 

  { Goode Homolosine }
  OSRSetGH: function   (hSRS :  TOGRSpatialReferenceH;
                      dfCentralMeridian,
                      dfFalseEasting,
                      dfFalseNorthing : double
                      ) : TOGRErr; cdecl;

  { GEOS - Geostationary Satellite View }
  OSRSetGEOS: function   (hSRS : TOGRSpatialReferenceH;
                        dfCentralMeridian, dfSatelliteHeight,
                        dfFalseEasting, dfFalseNorthing : double
                        ) : TOGRErr; cdecl; 

  { Gauss Schreiber Transverse Mercator }
  OSRSetGaussSchreiberTMercator: function   (hSRS :  TOGRSpatialReferenceH;
                                           dfCenterLat, dfCenterLong,
                                           dfScale,
                                           dfFalseEasting,
                                           dfFalseNorthing : double
                                           ) : TOGRErr; cdecl;
  { Gnomonic }
  OSRSetGnomonic: function   (hSRS : TOGRSpatialReferenceH;
                            dfCenterLat, dfCenterLong,
                            dfFalseEasting, dfFalseNorthing : double
                            ) : TOGRErr; cdecl;
  { Hotine Oblique Mercator using azimuth angle }
  OSRSetHOM: function   (hSRS :  TOGRSpatialReferenceH;
                       dfCenterLat, dfCenterLong,
                       dfAzimuth, dfRectToSkew,
                       dfScale,
                       dfFalseEasting, dfFalseNorthing : double
                       ) : TOGRErr; cdecl; 
  { Hotine Oblique Mercator using two points on centerline }
  OSRSetHOM2PNO: function   (hSRS :  TOGRSpatialReferenceH;
                           dfCenterLat,
                           dfLat1, dfLong1,
                           dfLat2, dfLong2,
                           dfScale,
                           dfFalseEasting, dfFalseNorthing : double
                           ) : TOGRErr; cdecl;
  { International Map of the World Polyconic }
  OSRSetIWMPolyconic: function   (hSRS :  TOGRSpatialReferenceH;
                                dfLat1, dfLat2,
                                dfCenterLong,
                                dfFalseEasting,
                                dfFalseNorthing : double
                                ) : TOGRErr; cdecl;

  { Krovak Oblique Conic Conformal }
  OSRSetKrovak: function   (hSRS :  TOGRSpatialReferenceH;
                          dfCenterLat, dfCenterLong,
                          dfAzimuth, dfPseudoStdParallelLat,
                          dfScale,
                          dfFalseEasting,
                          dfFalseNorthing : double
                          ) : TOGRErr; cdecl;
  { Lambert Azimuthal Equal-Area }
  OSRSetLAEA: function   (hSRS :  TOGRSpatialReferenceH;
                        dfCenterLat, dfCenterLong,
                        dfFalseEasting, dfFalseNorthing : double
                        ) : TOGRErr; cdecl;
  { Lambert Conformal Conic }
  OSRSetLCC: function   (hSRS :  TOGRSpatialReferenceH;
                       dfStdP1, dfStdP2,
                       dfCenterLat, dfCenterLong,
                       dfFalseEasting, dfFalseNorthing : double
                       ) : TOGRErr; cdecl;

  { Lambert Conformal Conic 1SP }
  OSRSetLCC1SP: function   (hSRS :  TOGRSpatialReferenceH;
                          dfCenterLat, dfCenterLong,
                          dfScale,
                          dfFalseEasting,
                          dfFalseNorthing : double
                          ) : TOGRErr; cdecl;

  { Lambert Conformal Conic (Belgium) }
  OSRSetLCCB: function   (hSRS :  TOGRSpatialReferenceH;
                        dfStdP1, dfStdP2,
                        dfCenterLat, dfCenterLong,
                        dfFalseEasting, dfFalseNorthing : double
                        ) : TOGRErr; cdecl;
  { Miller Cylindrical }
  OSRSetMC: function   (hSRS :  TOGRSpatialReferenceH;
                      dfCenterLat, dfCenterLong,
                      dfFalseEasting, dfFalseNorthing : double
                      ) : TOGRErr; cdecl;

  { Mercator }
  OSRSetMercator: function   (hSRS :  TOGRSpatialReferenceH;
                            dfCenterLat, dfCenterLong,
                            dfScale,
                            dfFalseEasting, dfFalseNorthing : double
                            ) : TOGRErr; cdecl;

  { Mollweide }
  OSRSetMollweide: function    (hSRS :  TOGRSpatialReferenceH;
                              dfCentralMeridian,
                              dfFalseEasting,
                              dfFalseNorthing : double
                              ) : TOGRErr; cdecl;

  { New Zealand Map Grid }
  OSRSetNZMG: function   (hSRS :  TOGRSpatialReferenceH;
                        dfCenterLat, dfCenterLong,
                        dfFalseEasting, dfFalseNorthing : double
                        ) : TOGRErr; cdecl;

  { Oblique Stereographic }
  OSRSetOS: function   (hSRS :  TOGRSpatialReferenceH;
                      dfOriginLat, dfCMeridian,
                      dfScale,
                      dfFalseEasting, dfFalseNorthing : double
                      ) : TOGRErr; cdecl;
  { Orthographic }
  OSRSetOrthographic: function   (hSRS :  TOGRSpatialReferenceH;
                                dfCenterLat, dfCenterLong,
                                dfFalseEasting,
                                dfFalseNorthing : double
                                ) : TOGRErr; cdecl; 
  { Polyconic }
  OSRSetPolyconic: function (hSRS :  TOGRSpatialReferenceH;
                             dfCenterLat, dfCenterLong,
                             dfFalseEasting, dfFalseNorthing : double
                             ) : TOGRErr; cdecl;
  { Polar Stereographic }
  OSRSetPS: function   (hSRS :  TOGRSpatialReferenceH;
                      dfCenterLat, dfCenterLong,
                      dfScale,
                      dfFalseEasting, dfFalseNorthing : double
                      ) : TOGRErr; cdecl;
  { Robinson }
  OSRSetRobinson: function   (hSRS :  TOGRSpatialReferenceH;
                            dfCenterLong,
                            dfFalseEasting, dfFalseNorthing : double
                            ) : TOGRErr; cdecl;
  { Sinusoidal }
  OSRSetSinusoidal: function   (hSRS :  TOGRSpatialReferenceH;
                              dfCenterLong,
                              dfFalseEasting,
                              dfFalseNorthing : double
                              ) : TOGRErr; cdecl;

  { Stereographic }
  OSRSetStereographic: function   (hSRS :  TOGRSpatialReferenceH;
                                 dfCenterLat, dfCenterLong,
                                 dfScale,
                                 dfFalseEasting,
                                 dfFalseNorthing : double
                                 ) : TOGRErr; cdecl; 

  { Swiss Oblique Cylindrical }
  OSRSetSOC: function   (hSRS :  TOGRSpatialReferenceH;
                       dfLatitudeOfOrigin, dfCentralMeridian,
                       dfFalseEasting, dfFalseNorthing : double
                       ) : TOGRErr; cdecl;

  { Transverse Mercator }
  OSRSetTM: function   (hSRS :  TOGRSpatialReferenceH;
                      dfCenterLat, dfCenterLong,
                      dfScale,
                      dfFalseEasting, dfFalseNorthing : double
                      ) : TOGRErr; cdecl;

  { Transverse Mercator variant }
  OSRSetTMVariant: function   (hSRS :  TOGRSpatialReferenceH;
                             pszVariantName : PCHAR;
                             dfCenterLat, dfCenterLong,
                             dfScale,
                             dfFalseEasting, dfFalseNorthing : double
                             ) : TOGRErr; cdecl;

  { Tunesia Mining Grid  }
  OSRSetTMG: function (hSRS :  TOGRSpatialReferenceH;
                       dfCenterLat, dfCenterLong,
                       dfFalseEasting, dfFalseNorthing : double
                       ) : TOGRErr; cdecl;

  { Transverse Mercator (South Oriented) }
  OSRSetTMSO: function   (hSRS :  TOGRSpatialReferenceH;
                        dfCenterLat, dfCenterLong,
                        dfScale,
                        dfFalseEasting, dfFalseNorthing : double
                        ) : TOGRErr; cdecl;

  { VanDerGrinten }
  OSRSetVDG: function   (hSRS :  TOGRSpatialReferenceH;
                       dfCenterLong,
                       dfFalseEasting, dfFalseNorthing : double
                       ) : TOGRErr; cdecl;

  OSRSetWagner: function (hSRS :  TOGRSpatialReferenceH; nVariation : longint;
                          dfFalseEasting,
                          dfFalseNorthing : double
                          ) : TOGRErr; cdecl;
  OSRCleanup: procedure ; cdecl;

OPTGetProjectionMethods: function  : PPCHAR; cdecl;
OPTGetParameterList: function (pszProjectionMethod : PCHAR;
                             ppszUserName : PPCHAR
                             ) : PPCHAR; cdecl;
OPTGetParameterInfo: function (pszProjectionMethod : PCHAR;
                             pszParameterName : PCHAR;
                             ppszUserName : PPCHAR;
                             ppszType : PPCHAR;
                             pdfDefaultValue : PDOUBLE
                             ) : longint; cdecl;

OCTNewCoordinateTransformation:  function   (hSourceSRS : TOGRCoordinateTransformationH;
                                            hTargetSRS : TOGRCoordinateTransformationH
                                            ) : TOGRCoordinateTransformationH ;
                                            stdcall;
OCTDestroyCoordinateTransformation:  procedure  (const Handle : TOGRCoordinateTransformationH); stdcall;
OCTTransformEx:  function   (const Handle : TOGRCoordinateTransformationH; const PointCount : longint;
                                 X, Y, Z : PDOUBLE;
                                 pSuccess : PINTEGER) : longint; stdcall;
OCTTransform:  function  (const Handle : TOGRCoordinateTransformationH; const PointCount : longint;
                                 X, Y, Z : PDOUBLE
                                 ) : longint; stdcall;


{$ENDIF}




implementation

end.


{ Fetch indicated attribute of named node. }
